@part[Appendix3,root "thesis.mss"]

@appendix[Sample Library Session]

Following is an example of a library session, consisting of starting up the
library system, performing a series of commands, and terminating the
session.  The files used in this session are given in section C.1.
Italics indicate notes, and are not actually part of the session.

@begin[verbatim]
->@b[hglib]
               @i[Start the library session from unix.]
hglib: @b[create test]
               @i[Create a new library and file test.hglib]
hglib: @b[input blue mtest.blue auto *]
               @i[Scan in all modules from the file mtest.blue]
hglib: @b[view *]
               @i[List all modules in the library]
c_fseq
c_one
c_port
c_seq
c_str
im
n_fseq
n_one
n_port
n_seq
n_str
n_typ
n_un
p_typ
x1

hglib: @b[uses n_un]
               @i[List modules used by n_un]
im
n_one

hglib: @b[group create first]
               @i[Create a new group called "first"]
hglib: @b[group add first *_port]
               @i[Add the modules c_port and n_port to first]
hglib: @b[group view first]
               @i[View members of the group first]
c_port
n_port

hglib: @b[input argus gtest.arg auto *]
               @i[Scan in all modules from the file gtest.blue]
hglib: @b[view language argus]
               @i[View all modules entered in ARGUS]
alpha
beta
delta
gamma
first
second
third

hglib: @b[output blue screen beta]
               @i[Output the module beta to the screen in blue]
(defabstype beta t1 t2 (PORT ((in1 INT32) (in2 REAL64))
                             ((*a INT32) (*b REAL64))
                             ()))
hglib: @b[*in John Doe]
               @i[Enter the string "John Doe" into the data field]
hglib: @b[notes n_port owner]
               @i[Set the owner of n_port to the data field]
hglib: @b[view owner *Doe]
               @i[View all ports owner by "*Doe"]
n_port

hglib: @b[delete *_*]
               @i[Delete all modules named "*_*"]
hglib: @b[view language blue]
               @i[View all modules entered in blue]
im
x1

hglib: @b[quit]
->             @i[The library session has been completed.]
@end[verbatim]
@blankspace[2]
@appendixsection[Sample Session Files]

@tabdivide[7]
@\@\@b[@u[mtest.blue]]
@begin[verbatim]

(defmtype p_typ BOOL)

(defmtype im IMAGE)

(defmtype c_seq (SEQUENCE BYTE))

(defmtype c_fseq (FIXED-SEQUENCE BYTE 4))

(defmtype c_str (STRUCT (fld1 BYTE) (fld2 BOOL)
			(fld3 INT32)))

(defmtype c_one (ONEOF (fld1 BYTE) (fld2 BOOL) (fld3 BYTE)))

(defmtype c_port (PORT ((in1 BOOL) (in2 BYTE))
		       ((out1 BOOL) (out2 BOOL) (out3 INT32))
		       ((exc1 ((exc1_f1 BOOL)
		       	       (exc1_f2 REAL64)))
			(exc2))))

(defmtype n_typ c_fseq)

(defmtype n_seq (SEQUENCE p_typ))

(defmtype n_fseq (FIXED-SEQUENCE n_seq 4))

(defmtype n_str (STRUCT (fld1 p_typ) (fld2 n_seq)
			(fld3 c_str)))

(defmtype n_one (ONEOF (fld1 n_str) (fld2 c_port)
		       (fld3 BYTE)))

(defmtype n_un (UNION (fld1 n_one) (fld2 BOOL) (fld3 im)))

(defmtype n_port (PORT ((in1 n_seq) (in2 n_one))
		       ((out1 p_typ) (out2 c_port)
		       	(out3 p_typ))
		       ((exc1 ((exc1_f1 BOOL)
		       	       (exc2_f2 c_port)))
			(exc2 ()))))

(defmtype x1 (FIXED-SEQUENCE (ONEOF (f1 (SEQUENCE BOOL))
				    (f2 INT32)) 10))

@end[verbatim]
@blankspace[2]
@\@\@b[@u[gtest.arg]]
@begin[verbatim]
gtest = guardian is create handles alpha, beta, gamma, delta

    first = bool
    second = sequence [first]
    third = oneof [f1: first, f2, f3: second, f4: null]

    create = creator ()

	end create

    alpha = handler [t: type] ()

	end alpha

    beta = handler [t1, t2: type] (in1: int, in2: real)
    	      returns (int, real)

	end beta

    gamma = handler [t1: type, t2: type]
			(in1: t1, in2: second) returns (t1)
	      signals (sig1 (third, third, t1), sig2,
	      	       sig3 (int, t2, t2))

	end gamma

    delta = handler [t1, t2, t3, t4: type, t5, t6: type]
			(in1: array [sequence [t5]],
			 in2, in3, in4: t4)
	      returns (int, int, t3, t3, t2)
	      signals (sig1 (third,
	      		     record [f1: t1,
			 	     f2: sequence [t1]]),
		       sig2,
		       sig3 (bool, bool, bool, int, t1))

	end delta

    end gtest
@end[verbatim]
