@part[chapter1,root "thesis.mss"]

@chapter[Background and Introduction]

@section[Mercury]

Mercury is an inter-language communication system under development
at the M.I.T.  Laboratory for Computer Science.  Its ultimate goal is to
allow programs written in different languages and running on different
machines using different operating systems, to utilize each other
@cite[mercury].
Such a system will be important in the future, as more and more
specialized languages and systems are being developed, often most
efficient in one particular area.  Using Mercury, a program with many
different components could
have each component written in the language which best suits it, and
running on a computer designed for that function.  For example, a
database with an advanced graphics user-interface could have the database
written in a language designed for such systems, and the graphics
component written in a graphics language and running on a machine
designed for efficient graphics.

Programs using Mercury communicate via ports.  A port ties the programs
together, one as the client, and the other as the server.  After code
defining the port and its constituent data types is created in blue, a stub
generator is used @cite[stubgen].  This results in @i[stubs],
which consist of code
to translate data types from some language to Mercury's
global representation for data types, or vice-versa.
These stubs are linked into the program as
procedures, to be called upon invocation of a Mercury port.
Currently, Mercury is being written to handle ARGUS (a distributed
language also under development at the lab), C, and LISP.

The blue language is a subset of LISP used to define Mercury types.
It allows for specific primitives, constructed types, abstract types,
and parameterized types intended to represent any data type from any
Mercury-capable language @cite[cveneer].  In addition, blue
allows for the definition of ports.  A port represents the interface
Mercury provides between client and server programs.
During runtime, when a program
invokes a port, a sort of software link between the client (calling)
program and the server (being called) procedure is created.  The port's
arguments are translated, using the stubs, from the client language
into the global runtime representations, transmitted to the server, and
then translated into the server language and passed to the server procedure.
Then the results are stored in the space
allocated (or an exception is invoked, and its values are set), the
arguments are translated back into the global representations, sent back
to the client, and then translated into the client
language.  It is with the definitions of ports and their constituent data
types that the library system works.

@section[The Mercury Library]

The subject of this thesis is the design and implementation of a
library system for Mercury.  A library keeps track of information
describing how various software modules interact.  The system
described in this paper holds information to facilitate
interaction between programs using Mercury, and performs other
functions as well.

The purpose of this library is, basically, to make Mercury easier to
use.  The operations supplied by the library are the result of
conversations with three of the Lab for Computer Science staff members
developing Mercury.  The functions of the library break down into five
points.  First the library should provide an
easy way to view the ports and their component variable types.  This
allows easy use of ports without looking at the actual code, which may
be written in a language unfamiliar to the programmer.  Second,
there should be an easy way to access ports created by other users, or
even a central library with useful procedures having Mercury ports.
Third, a facility should exist that allows for a frontend to
be written in any of Mercury's languages, using the library via
Mercury.  Fourth, to facilitate the previous points and to avoid
requiring knowledge of blue, the library should be able to translate
between the various languages which Mercury uses.  Fifth, data types
and port types should be easy to enter.

All these points are part of the original purpose, simplifying
Mercury in two ways.  Some of them minimize knowledge of blue or the
language in which a particular procedure is written.  The others allow
for easy access to Mercury-capable programs by centralizing and
simplifying information pertaining to them.  The next chapter follows
the development of these five basic points through a general design, a
specific design, and implementation.

@section[Terminology]

This section describes the use of various terms used throughout this
paper which could otherwise be confusing or ambiguous.  The terms
@i[library] and @i[library system], while seeming similar, are
actually different.  Library system (or @i[Mercury library]) refers to
the entire program
described herein, while library refers to an individual database of
Mercury interface information.  The library system provides access to
any number of libraries.

@i[Blue], @i[module], @i[specification], and @i[Mercury-capable
language] also require definition.  As previously mentioned, blue
is the name of the language used by Mercury for defining Mercury
types.  Mercury-capable language, refers to any language which can
utilize the mercury system, currently C, ARGUS, and LISP.  An
individual specification of a Mercury type will be called a module or
a specification (@i[spec] for short).  For example, a port definition
called @i[MyPort] would be written in blue, but is called a module,
specification, or spec.

In addition to these terms, the use of boldface or italics is relevant to
meaning.  Depending on context, a boldface term represents either the
name of a program component (a cluster, guardian, or procedure; see
description of ARGUS in the next chapter) or a command.  Italics, also
depending on context, either introduce a new term, name a variable, or
represent a user-defined argument to a command.
