@part[chapter3,root "thesis.mss"]

@chapter[The Interface]

@section[Summary]

The library system is purposefully designed without a user-friendly
frontend.  This is because no single frontend would be able to fit every
user's needs.  The Mercury port in the interface guardian allows a
frontend to be written in any Mercury-capable language and connected
to the @b[merc] guardian via Mercury.  These frontends can organize
their commands to suit the users of that particular language, and use
only those @b[merc] commands which are useful.  The port has a single
string as input, and an array of strings as output, listing all
results and errors.  Appendix B gives a concise listing of commands,
while complete descriptions are provided below.  Additionally,
appendix C contains a sample library session.

When a legitimate command is given, there are two errors that can
occur immediately.  If the wrong number of arguments is given, an error
message will result.  If any operation requiring a current library is
given, and no library has been set, an error indicating so will result.

@section[Library System Operations]

There are three general commands in this category.  @b[Help]
obtains a listing of the help file (see appendix B), and is a sort of
command summary for the interface.  @b[Quit] closes the current
library, if any, and terminates the interface guardian.  @b[Source]
@i[directory_name] sets the program's default directory, and @b[xfile]
@i[file_name] reads in the contents of @i[file_name].xfile and
executes each line as a command.

@section[Library Operations]
Library operations are very straightforward.  The @b[create]
@i[library_name]
command closes down the current library (if any) and creates a new
library, which then becomes the current library.  If the file
@i[library_name].hglib cannot be created, though, then the operation
is aborted.  @b[Library] @i[library_name] searches for the file
@i[library_name].hglib, and the information contained within becomes
the current library, any existing library being closed.  Any problems,
such as the file not existing, or having data in an incorrect format,
will abort the operation.  There is a public directory for Mercury
libraries.  The command @b[publicize] @i[filename] will output the current
library to the file @i[filename] in this directory.  Using any library
access operation (@b[create], @b[library], @b[merge], and @b[refer]) with
"public/@i[filename]" as an argument will access @i[filename] in this
public directory.

@section[The Data Field]

The command parser removes punctuation from commands, which is
generally not relevant.  For entry of notes or screen entry of modules,
though, exact entry of data is required.  Therefore, a string called
@i[data] is provided for in the @b[merc] guardian for this purpose.
When performing either of these operations, the string representing
the notes or the type should first be entered into the data field, an
operation that bypasses the standard command parsing routine.
Then when the commands are given, they will automatically use the
value of this string as their argument.  There are two commands which
utilize the data field: @b[*in], and @b[*out].  @b[*in] @i[value]
enters @i[value] as the value for the string @i[data].  @b[*out]
returns the value stored in @i[data].

@section[Database Mutating and Browsing Operations]

This section contains more commands than any other section, since the
manipulation and viewing of a database can be done in many different
ways.  None of the commands in this (or the remaining) section may be
performed unless a working library has been chosen.  There are two
types of commands here: those which alter the library, and those which
view it; they are listed together since they can overlap, and since
they are all database commands.  Database commands and input and output
operations (see below) generally allow wildcarding (using '*' to match any
character string, and '?' to match any single character).

There are four commands which are specifically browsing commands.
@b[References] simply returns a list of the libraries to which the
current library refers.  The other three commands allow viewing
of library specs, not including groups (groups have their own set of
commands, described below in this section).  @b[Users-of] @i[module_name]
returns all modules that utilize @i[module_name] in their type
definitions.  @b[Uses] @i[module_name] returns all modules that are
used in the definition of @i[module_name].  When the library is looked
at as the tree structure it really is, @b[users-of] gives ancestors, and
@b[uses] gives descendants.

The last viewing commands is @b[view]
@i[subcommand pattern], which supplies a variety of ways of looking at
specs.  The default subcommand is view-by-name, which will return all
modules whose names match the @i[pattern] given.  @b[View abstract]
@i[pattern] will list all
modules which are parameterized, or are defined as abstract in their
original language.  @b[View language] @i[pattern] returns all modules
originally listed
in the language @i[pattern].  @b[View owner] @i[pattern] and @b[view
notes] @i[pattern] simply list all
modules whose owner or notes (respectively) match with @i[pattern].

There are also two subcommands to view modules by type, @b[view type]
@i[pattern], and @b[view typein] @i[pattern].  Both of these commands
create a temporary type, @i[temp], by parsing the data field in the
language @i[pattern].  The @b[type] subcommand returns all modules of
the same type as @i[temp], while @b[typein] returns all modules
equivalent to or containing the type @i[temp].

There are five commands which alter the current library.  @b[Delete]
@i[module_name] removes @i[module_name] from the library, unless it is
used by other modules (if so, those must be deleted first, or they can
be deleted at the same time by simply giving a list of modules).
@b[Merge] @i[library_name] merges in the specs and references from the
library @i[library_name] into the current library.  Modules with the same
name will not be added unless the command @b[merge/mutate] is used,
which replaces old modules with those from the new library in the case
of duplicate names.  @b[Refer/add] and @b[refer/delete]
@i[library_name] respectively add and delete @i[library_name] as a
reference library for the current library.  

@b[Type] @i[module_name] can be used to define an unspecified type
called @i[module_name].  In the case of circular definitions, this
command can be used to define the circular type, and then when the
actual type definition is parsed, the unspecified definition will be
replaced.  @b[Notes] @i[name subcommand] alters the notes or
owner entry for the module or group @i[name].  If no subcommand is
given, the owner and notes for the specified library element will be
listed.  Otherwise, @i[owner] or @i[notes] can be given as a
subcommand to set the relevant field of the element to the value of
the data field.

The command @b[group] @i[subcommand group_name optional_argument] can
view or alter the library.  It has five
subcommands, all of which are followed by a group, and two of which
have an additional argument.  The @i[create] subcommand creates a group
@i[group_name], with no elements.  @i[Delete] deletes the given group,
as long as it has no elements.  @i[Add] and @i[subtract] subcommands use
the command field @i[optional_argument], and add or delete the group
elements with that name (these can be modules or groups, since groups
can have other groups as elements).  Finally, the subcommand @i[view]
lists all the elements of the given group.

@section[Input and Output Operations]

The other operations are for adding, translating, and outputting
library modules, and are called @b[input] and @b[output].  @b[Output]
@i[language place module(s)] takes all @i[modules] listed, translates them
into @i[language], and outputs them to @i[place].  The value of
@i[place] can be either the word "screen", which outputs the results
to the screen, or a file name, which outputs the results to the given
file.

@b[Input{/merge}] @i[language file flag module(s)] allows for input of
modules in a variety of ways.  @b[/Merge] is optional, and informs the
program to replace duplicate names.  Without @b[/merge], duplicate names
are unchanged.  In either case,
duplicates are listed as output.  @i[Language] gives the language that
the modules are in, @i[file] gives the file they are contained in.
@i[Flag] is either "manual" or "auto".  In some cases, the default
values for translations to blue (done by "auto"), will not be
desired.  "Manual" informs the computer to translate the specs into
blue, and then output the results to the file @i[file].blue.  Then,
these can be altered as required, and re-input via @b[input/merge],
in the blue language.  Only those modules listed in @i[modules] will be
read in to the library.

There is one other option for inputting modules, screen entry.  In this
case, the string representing the new spec is entered into the data
field, and then the command @b[input{/merge}] @i[language] @b[screen]
is used, which inputs the value of the data field in the given
language.
