@part[chapter4,root "thesis.mss"]

@chapter[Conclusion and Future Work]

@section[Summary]

The Mercury library is an effective utility for the Mercury system,
keeping track of blue types and translating them.  It also allows
programmers using Mercury to look at other libraries and see what has
been done by others, helping to avoid redundant work.  It serves to make the
Mercury system easier to use, by collecting relevant information in one place,
and by minimizing knowledge of blue and Mercury-capable languages.  This is
done in ARGUS, based on a multi-hash-table database of blue modules, where
these blue modules are also arranged into a hierarchical tree structure. This
design, presented in this paper, results in an efficient and effective Mercury
library system.

@section[Future Work]

Although most normal use of the library system will not run into any
problems, it is incomplete in some 
areas and could be improved in others.  This chapter describes where
the library system falls short, and could be improved in the future.

@subsection[Additional Languages]

Although Mercury uses C, LISP, and ARGUS, and blue, the current system
only has facilities for handling blue and ARGUS. The cluster
@b[language] exists to facilitate the addition of other languages.
Additionally, an interface to libby, the ARGUS library system, has been
anticipated. To add C, LISP, or libby, a cluster of the
language's or system's name must be created with two procedures:
@b[parse] and @b[unparse]. Then the @b[language] cluster must be
slightly altered in a straightforward manner (see appendix D).

@subsection[Nested Type Definition Translating]

Nested type definitions are not allowed in the blue language, but are in some
other languages (specifically ARGUS).  While both the blue and ARGUS
parsers will accept nested type definitions, when modules are output, they are
not nested.  This is legitimate for blue, but an addition to the @b[argus]
cluster could allow the user to set the level of nesting desired when
translating modules to ARGUS.  For example, if the following modules were
entered in the blue language:

@begin[quotation]
(defmtype low INT32)@*

(defmtype mid (SEQUENCE low))@*

(defmtype high mid)
@end[quotation]

blue (unnested) translating of high would give:
@begin[quotation]
(defmtype high mid)
@end[quotation]

and ARGUS (unnested) translating of high would give:
@begin[quotation]
high = mid
@end[quotation]

but ARGUS nested translating of high would give:

@begin[quotation]
high = sequence [int]
@end[quotation]

@subsection[ARGUS Mutexes]

Mutexes are an ARGUS type which cannot currently be accurately
translated to blue.  They have a locking mechanism which has no
counterpart in blue, so they are represented as oneofs.  Attempting to
access a mutex via Mercury using the translated type given by the
library system will not work until a way of handling this locking is added.

@subsection[Case sensitivity]

Some languages are case sensitive (e.g.  blue, C), while others aren't
(e.g.  ARGUS).  So two variables in C, such as 'var' and 'VAR' are
different, while in ARGUS they are the same.  The library system has
no case sensitivity, so the
declarations of 'var' and 'VAR' will not be allowed by the Mercury
library.  The alternative could have caused major problems in ARGUS,
while this decision simply makes a bad programming practice illegal
when using the library system.  Of course, the best method would somehow work
with the rules of both languages.

@subsection[Viewing by type]

The view command can be used to view modules equivalent to or
containing a specific type.  This sub-command has not yet been
implemented.

@subsection[Unspecified Blue Types]

There are two types in blue which, although planned for, are not
currently allowed by the library system.  These are containers and pointers.
They have not yet been fully designed in the actual Mercury system, so
it was impossible to design a reasonable implementation for them.  When
they are added to Mercury, they should also be fully implemented in
the library system.

@subsection[Circular Definitions]

Circular definitions were not planned for in the original design of
the library system, but added in after the basic system already
existed.  As the current design seemed to work rather well for
non-circular definitions, it was not altered, but modified to account for
circular definitions.  Although they do work, their late design and
implementation resulted in an inefficient method of handling them;
they are "added on" rather than an integral part of the system.  These
could be somewhat reworked for a more reliable and efficient system.

@subsection[Stub Generation]

A very helpful addition to the Mercury library would be the inclusion of stub
generation.  This would make the library system able to perform all functions
a Mercury user would normally require.  Such simplicity would
definitely be useful.

@subsection[Improved Command Parsing]

The command parser used in the library system is a standard ARGUS
command parsing procedure called
@b[all_cmds], which takes in a string and breaks it up into an array
of strings (individual words).  This process destroys spacing and
punctuation, which may be important in type definitions or notes.
The data field avoids this problem, having the @b[*in] @i[string]
command bypass the command parser and enter the exact value @i[string] into
the data field of the @b[merc] guardian.  A better way of handling
this would be to write a command parser which could handle both
standard commands and commands requiring string entry.  This would be
more efficient and more comprehensible than the current method.

@section[Closing]

In closing, this paper presents a useful tool for the Mercury system.  This
tool, in the form of a library system, will hopefully be useful either as a
utility for Mercury users or as a prototype for a more advanced library system
for Mercury.
