@Part(overview, Root "manual.mss")

@Chapter(Overview)

Argus is an experimental language/system designed to support
the construction and execution of distributed programs.
Argus is intended to support only a subset of the applications that
could benefit from being implemented by a distributed program.
Two properties distinguish these applications:  they make use
of on-line data that must remain consistent in spite of concurrency
and hardware failures,
and they provide services under
real-time constraints that are not severe.
Examples of such applications are office automation systems
and banking systems.

Argus is based on CLU.
@index(CLU)
It is largely an extension of CLU, but
there are number of differences (see Appendix @ref(apdx-diffs)).
Like CLU, Argus provides @i(procedures) for procedural abstraction,
@i(iterators) for control abstraction, and @i(clusters) for data abstraction.
In addition, Argus provides @i(guardians) that encapsulate and
control access to one or more resources.  These are discussed in more
detail in Section @ref(sect-guardian-overview).
Argus also provides equate modules as a convenient way to refer
to constants (see Section @ref(sect-equate-module)).
As in CLU, modules may be parameterized, so that a single module 
can define a class
of related abstractions.


@Section(Objects and Variables)

The semantics of Argus deal with objects and variables.
@i(Objects)
@index(object)
are the data entities that are created and manipulated by
applications.
@i(Variables)
@index(variable)
are the names used in a program to refer to objects.

Every object has a @i(type) that characterizes its behavior.
@index(type)@SeeAlso(Primary="type", Other="cluster")
@SeeAlso(Primary="type", Other="guardian")
A type defines a set of primitive operations to create and manipulate objects
of that type.

An object may refer@index(refer) to other objects or even to itself.
@IndexSecondary(Primary="object", Secondary="references")
It is also possible
for an object to be referred to or shared@index(sharing)
@IndexSecondary(Primary="object", Secondary="sharing") by several objects.
Objects exist independently of procedure and iterator invocations.

There are several categories of objects in Argus.
An object that exhibits time-varying behavior is called
a @i(mutable) object.
A @i(mutable) 
@index(mutable)@IndexSecondary(Primary="object", Secondary="mutable") object 
has state that may be modified by operations without changing
the object's identity.
A mutable object can thus exhibit behavior that varies with time.
An @i(immutable) 
@index(immutable)@IndexSecondary(Primary="object", Secondary="immutable") object's 
state is inseparable from its identity.  An immutable
object cannot exhibit time-variant behavior.
Objects are @i(atomic)
@index(atomic)@IndexSecondary(Primary="object", Secondary="atomic")
if they provide synchronization and recovery
for actions that manipulate them (see Section @ref(sect-atomic-obj)).
Objects are @i(transmissible)
@index(transmissible)@IndexSecondary(Primary="object", Secondary="transmissible")
if they can be sent as arguments or results of remote
procedure calls (see Section @ref(sect-transmissible)).
Since guardian, handler, creator, and @b[node] objects can be shared among 
guardians, these objects 
are said to be @i(global)
@index(global object)@IndexSecondary(Primary="object", Secondary="global") objects.
All other objects, such as characters, integers, or procedures,
can only be shared within a single guardian
and are called @i(local) 
@index(local)@IndexSecondary(Primary="object", Secondary="local") objects.

Variables are names used in programs to denote particular objects at execution
time.  It is possible for two variables to denote the same object.
Variables are not objects; they cannot be denoted by other variables or
referred to by objects.
@IndexSecondary(Primary="object", Secondary="versus variable")
@IndexSecondary(Primary="variable", Secondary="versus object")

Variables in guardian modules can be declared to be @i(stable).
@IndexSecondary(Primary="stable", Secondary="variable")
@IndexSecondary(Primary="variable", Secondary="stable")
The objects denoted by stable variables survive crashes
(see Section @ref(sect-guardian-overview)) and are called @i(stable objects).
@IndexSecondary(Primary="stable", Secondary="object")
@IndexSecondary(Primary="object", Secondary="stable")

@Section(Assignment and Calls)

The basic events in Argus are @i(assignments) and @i(calls).
@index(assignment)
The assignment statement @i(x := E), where @i(x) is a variable and @i(E)
is an expression, causes @i(x) to denote the object resulting from the
evaluation of @i(E).  The object is not copied.

A call@index(call) involves passing argument objects from the caller to the called routine
and returning result objects from the routine to the caller.
For local calls, argument passing is defined in terms of assignment, or
@i(call by sharing);
@IndexSecondary(Primary="call", Secondary="by sharing")
@IndexSecondary(Primary="call", Secondary="by value")
for remote calls, call by value is used.
In a local call, the formal arguments of a routine are considered to be local
variables of the routine and are initialized, by assignment, to the objects
resulting from the evaluation of the argument expressions.
In a remote call (see Section @ref(sect-remote-call)),
a copy of the objects resulting from the evaluation of the
argument expressions is made and transmitted to the called handler or creator
(see Section @ref(sect-transmissible)).
These copies are then used to initialize the formal arguments as before.
Local objects are shared between the caller and a called procedure or iterator,
but local objects are never shared between the caller and a called handler
or creator.

@Section(Type Correctness)
@IndexSecondary(Primary="type", Secondary="correctness")

The declaration of a variable specifies the type@index(type) of the objects which the
variable may denote.
In a legal assignment statement, @i(x := E),
@index(assignment)
the type of the expression @i(E) must be @i(included) in
the type of the variable @i(x).
Type inclusion
@index(type inclusion)@IndexSecondary(Primary="type", Secondary="inclusion")
is essentially equality of types (see Section @ref(sect-instantiations)),
except for routine types.
(A routine type with fewer exceptions is included in
an otherwise identical routine type with more exceptions.
See Section @ref(sect-type-incl) for details.)

Argus is a type-safe language,
@IndexSecondary(Primary="type", Secondary="safety")
in that it is not possible to treat an object
of type @i(T) as if it were an object of some other type @i(S)
(the one exception is when @i(T) is a routine type and @i(S) includes @i(T)).
The type safety of Argus, plus the restriction that only the code in a cluster
may convert between the abstract type and the concrete representation (see
Section @ref(sect-cluster)),
ensure that the behavior of an object can be characterized completely by the
operations of its type.

@Section(Rules and Guidelines)

Throughout this manual, and especially in the discussions of atomicity,
there are pragmatic rules and guidelines for the use of the language.
Certain properties that the language would like to guarantee, for example
that atomic actions are really atomic,
are difficult or impossible for the language to guarantee completely.
As in any useful programming language, programmers have enough rope to hang
themselves.
The rules and guidelines noted throughout the manual
(and collected in Appendix @ref(apdx-rules))
try to make the responsibilities of the language and the
programmer clear.

@Section(Program Structure)

An Argus distributed 
application consists of one or more guardians, defined by guardian
modules.
@index(guardian)
Guardian modules may in turn use all the other kinds of modules that Argus
provides.
@index(module)
Argus programmers may also write single-machine programs with no stable state,
using Argus as essentially a "concurrent CLU."  Such programs may be used to
start up multi-guardian applications.
Each module is a separate textual unit, and is compiled independently of other
modules.  Compilation is discussed in Section @ref(sect-environ).
