@Part(concepts, Root "manual.mss")

@Chapter(Concepts for Distributed Programs)
@Label(sect-guardian-overview)@Label(sect-handler)

In this chapter we present an overview of
the new concepts in Argus that support distributed programs.
In Section 2.1, we discuss @i(guardians),
the module used in Argus to distribute data.
Next, in Section 2.2, we present @i(atomic actions),
which are used to cope with concurrency and failure.
In Section 2.3
we describe @i(remote calls),
the inter-guardian communication mechanism.
In Section 2.4 we discuss transmissible types:
types whose objects can be sent as arguments or results of remote calls.
Finally, in Section 2.4 we discuss @i(orphans).

@Section(Guardians)

Distributed applications are implemented in Argus by one or more modules called
@i(guardians).
@index(guardian)
A guardian abstraction is a kind of data abstraction,
but it differs from the data abstractions supported by clusters
(as found in CLU).
In general, data abstractions@index(data abstraction) consist 
of a set of operations and a set of objects.
In a cluster the operations are considered to belong to the
abstraction as a whole.
However, guardian instances are objects and 
their handlers are their operations.
Guardian abstraction is similar to the data abstractions in Simula
and Smalltalk-80;  guardians are like class instances.

A @i(node) is a single physical location, which may have multiple processors.
A guardian instance resides at a single node,
although a node may support several guardians.
A guardian encapsulates and controls access to one or more
resources, such as data or devices.
Access to the protected resource is provided by a set of operations
called @i(handlers)@index(handler).  Internally, 
a guardian consists of a collection of data objects
and processes that can be used to manipulate those objects.
In general, there will be many processes executing concurrently
in a guardian:
a new process is created to execute each handler call, processes may be
explicitly created,
and there may be other processes that carry out
background activity of the guardian.

The data objects encapsulated by a guardian are @i(local):
@IndexSecondary(Primary="local", Secondary="object")
@IndexSecondary(Primary="object", Secondary="local")
they cannot be accessed directly by a process in another guardian.
In contrast, guardians are @i(global) objects:
@index(global object)@IndexSecondary(Primary="object", Secondary="global")
a single guardian
may be shared among processes at several different guardians.
A process with a reference to a guardian can call the guardian's handlers,
and
these handlers can access the data objects inside the guardian.
Handler calls allow access to a guardian's local data, but the guardian
controls
how that data can be manipulated.

When a node fails, it @i(crashes).  A crash is a "clean" failure, as opposed
to a "Byzantine" failure.
A guardian survives crashes of its node (with as high a probability
as needed).
A guardian's state consists of @i(stable) and @i(volatile) objects.
@IndexSecondary(Primary="stable", Secondary="object")
@IndexSecondary(Primary="volatile", Secondary="object")
@IndexSecondary(Primary="object", Secondary="stable")
@IndexSecondary(Primary="object", Secondary="volatile")
When a guardian's node crashes,
all processes running inside the guardian at the time of the crash are lost,
along with the guardian's volatile objects, but
the guardian's stable objects survive the crash.
Upon recovery of the guardian's node,
the guardian runs a special recovery process to reconstruct its
volatile objects from its stable objects.
Since the volatile objects are lost in a crash,
they typically consist only of redundant data 
that is used to improve performance
(for example,
an index into a database).
The persistent state of an application should be kept in stable objects.

Guardians are implemented by @i(guardian definitions).
These define:
@begin(enumerate)
The @i(creators).
@index(creator)
These are operations that can be called to create new
guardian instances that perform in accordance with
the guardian definition.

The guardian's stable and volatile state.
@IndexSecondary(Primary="stable", Secondary="state")
@IndexSecondary(Primary="volatile", Secondary="state")

The guardian's handlers.

The @i(background code).
@index(background)
This is code that the guardian executes independent of
any handler calls, for example, to
perform some periodic activity.

The @i(recover code).
@index(recover code)@index(recovery)
@IndexSecondary(Primary="crash", Secondary="recover code")
This is code that is executed after a crash
to restore the volatile objects.
@end(enumerate)
Guardians and guardian definitions are discussed in Section @ref(sect-guardian).


@Section(Actions)
@Label(sect-topaction)

The distributed data in an Argus application can be shared by
concurrent processes.
@index(process)@index(concurrency)
@SeeAlso(Primary="process", Other="action")
A process may attempt to examine
and transform some objects from their
current states to new states,
with any number of intermediate state changes.
Interactions among concurrent processes can leave
data in an inconsistent state.
Failures (for example, node crashes)@index(crash) can 
occur during the execution of a process,
raising the additional possibility that data will be left in an
inconsistent intermediate state.
To support applications that need consistent data,
Argus
permits the programmer to make
processes atomic.

We call an atomic process an @i(action).
@index(action)@index(atomic)
@IndexSecondary(Primary="atomic", Secondary="action")
@SeeAlso(Primary="action", Other="atomic")
Actions are @i(atomic) in that they are both serializable
and recoverable.
By @i(serializable)@index(serializable),
we mean that the overall effect of executing
multiple concurrent actions is as if they had been
executed in some sequential order, even though they actually
execute concurrently.
By @i(recoverable)@index(recoverable),
we mean that the overall effect of an action
is "all-or-nothing:"  either all changes made to the data by
the action happen, or none of these changes happen.
An action that completes all its changes successfully @i(commits);
@index(commit)
otherwise it @i(aborts)@index(abort), and objects that it modified are
restored to their previous states.

Before an action can commit, new states of all modified, stable
objects must be written to stable storage@Foot{
Lampson, B. W., "Atomic Transactions",
in @i(Distributed Systems--Architecture and Implementation),
Lecture Notes in Computer Science, volume 105, pages 246-265.
Springer-Verlag, New York, 1981.
}: 
storage that survives media crashes with high probability.
@IndexSecondary(Primary="stable", Secondary="storage")
Argus uses a two-phase commit
@index(two-phase commit)
@IndexSecondary(Primary="commit", Secondary="two phase commit protocol")
protocol@Foot{
Gray, J. N., "Notes on data base operating systems",
in
@i(Operating Systems, An Advanced Course),
Bayer, R., Graham, R. M., and Seegm@Ovp{@Umlaut()}uller, G. (editors),
Lecture Notes in Computer Science, volume 60, pages 393-481.
Springer-Verlag, New York, 1978.
}
to ensure
that either all of the changes made by an action occur or none of
them do.
If a crash occurs after an action modifies a stable object,
but before the new state has been written to stable storage,
the action will be aborted.


@SubSection(Nested Actions)
@Label(sect-nested-actions)

Actions in Argus can be nested: an action may be composed of several
@i(subactions).
@index(subaction)@IndexSecondary(Primary="action", Secondary="nested")
@index(nested action)@IndexSecondary(Primary="action", Secondary="subaction")
Subactions can be used to limit the
scope of failures and to introduce concurrency within an action.
@SeeAlso(Primary="failure", Other="crash")@index(concurrency)

An action may contain any number of subactions, some of which may
be performed sequentially, some concurrently.
This structure cannot be observed from outside the action;
the overall action is still atomic.
Subactions appear as atomic actions with respect to other
subactions of the same parent.
@index(parent)@IndexSecondary(Primary="action", Secondary="parent of")
Thus, subactions can be executed concurrently.

Subactions can commit and abort independently,
and a subaction can abort without forcing its parent action to abort.
@IndexSecondary(Primary="commit", Secondary="of a subaction")
@IndexSecondary(Primary="abort", Secondary="of a subaction")
However, the commit of a subaction is conditional:  even if a subaction
commits, aborting its parent action will abort it.

The root of a tree of nested actions is called a @i(topaction).
@index(topaction)@IndexSecondary(Primary="action", Secondary="topaction")
Topactions have no parent;
they cannot be aborted once they have committed.
Since the effects of a subaction can always be undone by aborting its parent,
the two-phase commit protocol is used only when topactions
attempt to commit.

In Argus,
an action (e.g., a handler call)
may return objects through either a normal return or an exception
and then abort.
The following rule should be followed to avoid violating serializability:
@index(serializable)
a subaction that aborts should
not return any information obtained from data shared with other
concurrent actions.


@SubSection(Atomic Objects and Atomic Types)
@Label(sect-atomic-obj)@Label(sect-atomic-objects)@Label(sect-lockrules)

Atomicity of actions is achieved via the data objects shared among those
actions.
Shared objects must be implemented so that actions
using them appear to be atomic.
@IndexSecondary(Primary="atomic", Secondary="object")
@IndexSecondary(Primary="atomic", Secondary="type")
@IndexSecondary(Primary="type", Secondary="atomic")
Objects that support atomicity are referred to as @i(atomic objects).
Atomic objects provide the synchronization and recovery needed
to ensure that actions are atomic.
An @i(atomic type) is a type whose objects are all atomic.
Some objects do not need to be atomic:
for example, objects that are local to a single process.
Since the synchronization and recovery needed to ensure atomicity may
be expensive,
we do not require that all types be atomic.
(For example, Argus provides all the built-in mutable types of CLU;
these types are not atomic.)
However, it is important to remember
that atomic actions must share only atomic objects.

Argus provides a number of built-in atomic types and type generators.
@IndexSecondary(Primary="atomic", Secondary="built-in atomic types")
@IndexSecondary(Primary="built-in", Secondary="atomic types")
@IndexSecondary(Primary="type", Secondary="built-in atomic types")
The built-in scalar types (@b[null], @b[node], @b[bool], @b[char], @b[int], @b[real],
and @b[string]) are atomic.
Parameterized types can also be atomic.
@IndexSecondary(Primary="type", Secondary="parameterized")
@IndexSecondary(Primary="type", Secondary="atomic")
Typically, an instance of a type generator will
be atomic only if any actual type parameters are also atomic.
The built-in immutable type generators (@b[sequence], @b[struct], and @b[oneof])
are atomic if their parameter types are atomic.
In addition, Argus provides three mutable atomic type generators:
@b[atomic@us<>array], @b[atomic@us<>record], and @b[atomic@us<>variant].
The operations on these types are nearly identical to the normal
@b[array], @b[record], and @b[variant] types of CLU.
Users may also define their own atomic types
(see Section @ref(sect-atomic)).

The implementation of the built-in mutable atomic type generators
is based on a simple locking model.
@index(locking)
@IndexSecondary(Primary="locking", Secondary="for built-in atomic types")
There are two kinds of locks:  read locks and write locks.
@index(read lock)@index(write lock)
When an action calls an operation on an atomic object,
the implementation acquires a lock on that object in the appropriate mode:
it acquires a write lock if it mutates the object,
or a read lock if it only examines the object.
The built-in types allow multiple concurrent readers,
but only a single writer.
If necessary, an action is forced to wait until it can obtain the appropriate
lock.
When a write lock on an object is first obtained by an action,
the system makes a copy of the object's state
in a new @i(version),
@IndexSecondary(Primary="version", Secondary="of an atomic object")
and the operations called by
the action work on this version@Foot{
This operational description (and others in this manual)
is not meant to constrain implementors.
However, this particular description does reflect our current implementation.
}.
If, ultimately, the action commits, this version will be retained,
and the old version discarded.
@index(commit)
A subaction's locks are given to its parent action when it commits.
When a topaction commits, its locks are discarded and its effects
become visible to other actions.
If the action aborts, the action's locks and the new version will be discarded,
and the old version retained
@index(abort)@index(locking)
@IndexSecondary(Primary="locking", Secondary="table of locking rules")
(see Figure @ref(fig-lockrules)).
@begin[Figure]
@Caption[Locking and Version Management Rules for a Subaction @i(S), on Object @i(X)]
@Tag(fig-lockrules)
@Bar()
@begin[Display]
@TabClear()
Acquiring a read lock:
       All holders of write locks on @i(X) must be ancestors of @i(S).
	
Acquiring a write lock:
       All holders of read and write locks on @i(X) must be ancestors of @i(S).
       If this is the first time @i(S) has acquired a write lock on @i(X),
                push a copy of @i(X) on the top of its version stack.
		
Commit:
       @i(S)'s parent acquires @i(S)'s lock on @i(X).
       If @i(S) holds a write lock on @i(X), then @i(S)'s version becomes @i(S)'s parent's version.

Abort:
       @i(S)'s lock and version (if any) are discarded.
@end[Display]
@Bar()
@end[Figure]

More precisely, an action can obtain a read lock on an object
if every action holding a write lock on that object is an ancestor
of the requesting action.
An action can obtain a write lock on an object if every action
holding a (read or write) lock on that object is an ancestor.
When a subaction commits, its locks are inherited
by its parent and its new versions replace those of its parent;
when a subaction aborts, its locks and versions are discarded
(see Figure @ref(fig-lockrules)).
Because Argus guarantees that parent actions never
run concurrently with their children,
these rules ensure that concurrent actions never hold write
locks on the same object simultaneously.

The @i(ancestors) of a subaction are itself,
its parent, its parent's parent, and so on;
@index(ancestor)@IndexSecondary(Primary="action", Secondary="ancestors")
a subaction is a @i(descendant) of its ancestors.
@index(subaction)@IndexSecondary(Primary="action", Secondary="descendants")
A subaction @i(commits to the top)
@IndexSecondary(Primary="commit", Secondary="to the top")
if it and all its ancestors, including the topaction, commit.
A subaction is a @i(committed descendant)
@IndexSecondary(Primary="commit", Secondary="committed descendant")
of an ancestor action if the
subaction and all intervening ancestors have committed.
When a topaction attempts to commit,
the two-phase commit protocol is used to ensure that the new versions
of all objects modified by the action
and all its committed descendants
are copied to stable storage.
After the new versions have been recorded stably,
the old versions are thrown away.

User-defined atomic types can provide greater concurrency
than built-in atomic types@Foot{An example can be found in 
Weihl, W. and Liskov, B., "Implementation of Resilient, Atomic Data Types,"
@i(ACM Transactions on Programming Languages and Systems),
volume 7, number 2 (April 1985), pages 244-269.}.
An implementation of a user-defined atomic type must address several issues.
First, it must provide proper synchronization so
that concurrent calls of its operations
do not interfere with each other,
and so that the actions that call its operations are serialized.
Second, it must provide recovery for actions
using its objects so that aborted actions have no effect.
Finally, it must ensure that changes made to its objects
by actions that commit to the top
are recorded properly on stable storage.
The built-in atomic types and the @b[mutex] type generator
are useful in coping with these issues.
@index(mutex)
User-defined atomic types are discussed further in Section @ref(sect-atomic).

@SubSection(Nested Topactions)
@Label(sect-nested-topaction)

In addition to nesting subactions inside other actions,
it is sometimes useful to start a new topaction inside
another action.
Such a @i(nested topaction), unlike a subaction,
@index(nested topaction)@IndexSecondary(Primary="topaction", Secondary="nested")
@IndexSecondary(Primary="action", Secondary="nested topaction")
has no special privileges relative to its "parent";
for example, it is not able to read an atomic object modified by its "parent".
Furthermore, the commit of a nested topaction is not relative to its "parent";
its versions are written to stable storage, and its locks are released,
just as for normal topactions.

Nested topactions are useful for benevolent side effects that change the
representation of an object without affecting its abstract state.
For example, in a naming system a name look-up may cause information
to be copied from one location to another,
to speed up subsequent look-ups of that name.
Copying the data within a nested topaction that commits
ensures that the changes remain in effect even if the "parent" action aborts.

A nested topaction is used correctly if it is serializable before its "parent".
This is true if either the nested
topaction performs a benevolent side effect,
or if all communication between the nested topaction and its
parent is through atomic objects.

@Section(Remote Calls)
@Label(sect-remote-call)

An action running in one guardian can cause work to be performed
at another guardian by calling a handler provided by the latter guardian.
An action can cause a new guardian to be created by
calling a creator.
@index(creator)
Handler and creator calls are @i(remote calls).
@index(remote call)@IndexSecondary(Primary="call", Secondary="remote")
Remote calls are similar to local procedure calls;
for example, the calling process waits for the call to return.
Remote calls differ from local procedure calls in several
ways, however.

First, the arguments and results of a remote call are passed
by value (see below and also Section @ref(sect-transmit)) rather than by sharing.
This ensures that the local objects of one guardian remain local to that
guardian, even if their values are used as
arguments or results of remote calls to other guardians.
The only objects that are passed by sharing
in remote calls are the global objects:
guardians, handlers, creators, and nodes.

Second, any remote call can raise the exceptions @i(failure)
and @i(unavailable).
@index(failure)@index(unavailable)
(Unlike CLU@index(CLU), not all local calls can raise @i(failure),
see Appendix @ref(apdx-diffs).)
The occurrence of @i(failure) means that the call is
unlikely to ever succeed, so there is no point
in retrying the call in the future.
@i(Unavailable), on the other hand,
means that the call should succeed if retried in the future,
but is unlikely to succeed if retried immediately.
For example,
@i(failure) can arise because it is impossible
to transmit the arguments or
results of the call (see Section @ref(sect-transmit));
@i(unavailable) can arise if the guardian being called has crashed,
or if the network is partitioned.

Third, a handler or creator can be called only from inside an action,
and the call runs as a subaction of the
calling action.
This ensures that a remote call succeeds @i(at most once):
either a remote call completes successfully and commits,
or it aborts and all of its modifications are undone
(provided, of course, that the actions involved are truly atomic).
Although the effect of a remote call occurs at most once,
the system may need to attempt it several times;
this is why remote calls are made within actions.

@Section(Transmissible Types)
@Label(sect-transmissible)

Arguments and results of remote calls are passed by value.
@IndexSecondary(Primary="call", Secondary="by value")
This means that the argument and result objects must be copied to
produce distinct objects.
Not all objects can be copied like this;
those that can are called @i(transmissible objects),
@IndexSecondary(Primary="transmissible", Secondary="object")
@IndexSecondary(Primary="object", Secondary="transmissible")
and their types are called @i(transmissible types).
@IndexSecondary(Primary="type", Secondary="transmissible")
Only transmissible objects
may be used as arguments and results of a remote call.
In addition, @b[image]@index(image) objects 
(see Section @ref(sect-image)) can contain
only transmissible objects.
Parameterized types may be transmissible in some instances and not in others;
for example, instantiations of the built-in type generators are transmissible
only if their parameter types are transmissible.
@index(transmissible)
While guardians, creators, and handlers are always transmissible, procedures
and iterators are never transmissible.

Users can define new transmissible types.
For each transmissible type @i(T)
the @i(external representation type) of @i(T) must be defined;
@index(external representation type)
@IndexSecondary(Primary="representation", Secondary="external")
@IndexSecondary(Primary="type", Secondary="external representation")
this describes the format in which objects of type @i(T) are transmitted.
Each cluster that implements a transmissible type @i(T) must
contain two procedures, @i(encode) and @i(decode),
@index(encode)@index(decode)
to translate objects of type @i(T) to and from their external representation.
More information about defining transmissible types can
be found in Section @ref(sect-transmit).

@Section(Orphans)
@Label(sect-orphan)

An @i(orphan) is an action that has had some ancestor "perish" or has had
the pertinent results of some relative action lost in a crash.
@index(orphan)@IndexSecondary(Primary="action", Secondary="orphan")
Orphans can arise in Argus due to crashes and explicit aborts.
For example, when a parent action is aborted, the active descendents it
leaves behind become orphans.
Crashes also cause orphans: when a guardian crashes, all active actions with
an ancestor at the crashed guardian and all active actions with committed
descendants that ran at the crashed guardian become orphans@Foot{
Walker, E. F., "Orphan Detection in the Argus System",
Massachusetts Institute of Technology, Laboratory for Computer Science,
Technical Report MIT/LCS/TR-326, June 1984.
}.
However, having a descendent that is an orphan does not necessarily imply
that the parent is an orphan;  as previously described,
actions may commit or abort independently of their subactions.

Argus programmers can largely ignore orphans.  Argus guarantees
that orphans are aborted before they can view inconsistent data
(provided actions are written so that they only communicate
through atomic data).
Remote calls that fail for any reason may be retried by the system,
including some cases where the call action becomes
an orphan due to crashes (see Section @ref(sect-hcall)).

Orphans always abort.
They may abort voluntarily or they may be forced to
abort by the run-time system;
however,
an orphan that is in a critical section
@index(critical section)
(executing a @b[seize] statement,
see Section @ref(sect-seize))
may not be forcibly aborted by the run-time system,
except by crashing the guardian.
On the other hand,
the system may encourage
orphans (especially topactions that are orphans) to abort
themselves by having their remote calls signal @i(unavailable).


@Section(Deadlocks)

Actions in Argus programs may become deadlocked.
@index(deadlock)@IndexSecondary(Primary="action", Secondary="deadlock")
@index(locking)@IndexSecondary(Primary="locking", Secondary="deadlock")
For example, if action @i(A) is waiting for a lock that @i(B) holds
and @i(B) is waiting for a lock that @i(A) holds, then @i(A) and @i(B)
are deadlocked.
Although implementations may provide some form of deadlock detection or
avoidance, they are not required to do so.
This is because detecting deadlocks is difficult in a language with
user-defined atomic types, since it is not always clear when actions
are "waiting" for each other.

If an implementation of Argus chooses to do deadlock detection
(presumably for the built-in atomic types),
it may only break deadlocks by aborting actions or by crashing guardians.
