@Part(environment, Root "manual.mss")

@Chapter(Environment)
@Label(sect-environ)

The Argus environment ensures complete static type checking of programs.
@index(type checking)
It also supports separate compilation and the independence of guardians.

@Section(The Library)

Argus modules are compiled in the context of a library that gives
meaning to external identifiers and allows inter-module type checking.
@index(library)
The Argus library contains type information about abstractions;
for each abstraction, the library contains a
@i(description unit), or DU, describing that abstraction
and its implementations.
@SeeAlso(Primary="DU", Other="description unit")@index(description unit)
Each DU has a unique name and these names form the basis of type checking.
@index(type)


@Section(Independence of Guardian Images)

The code run by a guardian comes from some guardian image.
@SeeAlso(Primary="image", Other="guardian image")
@index(guardian)@IndexSecondary(Primary="guardian", Secondary="guardian image")
A guardian image contains
all the code needed to carry out any local activity of the guardian;
any procedure, iterator or cluster used by that guardian will be in its
guardian image.
Any handler calls made by the guardian, however, are carried out at the
called guardian, which contains the code that performs the call.
Thus a guardian is independent of the implementations of the guardians it calls
and the implementation of a guardian can be changed
without affecting the implementations of its clients.


@Section(Guardian Creation)
@Label(sect-select)

When a guardian is created, it is necessary to select the guardian
image that will supply the code run by the new guardian.
@index(guardian)@IndexSecondary(Primary="guardian", Secondary="creation")
To this end, each guardian has an associated @i(creation environment)
that specifies the guardian images for other guardians it may create.
The creation environment is a mapping from guardian types to information
that can be used to select a guardian image appropriate for each kind of node.
For greater flexibility, this information can be associated with particular
creator objects.

@Section(The Catalog)
@Label(sect-catalog)

Somehow, guardians must be able to find other guardians to call for services.
A guardian usually has a reference to any guardian it creates.
Also, if a guardian can call
some other server guardian,
it can learn about the guardians that the server "knows", because
guardians can be passed in remote calls.
In addition, Argus provides a built-in subsystem known by
all guardians.
This subsystem is called the @i(catalog).
@index(catalog)
The catalog
provides an atomic mapping from names to transmissible objects.
For example, when a new guardian is created, it can be catalogued under
some well-known name, so that other guardians can find it in the future.
Since we are currently experimenting with various interfaces to the catalog,
we do not include an interface specification here.
