@Part(lexical, Root "manual.mss")

@Chapter(Lexical Considerations)

A module is written as a sequence of tokens and separators.
A @i(token) is a sequence of "printing" ASCII characters
@index(token)
(values 40 octal through 176 octal) representing
a reserved word,
an identifier,
a literal,
an operator, or
a punctuation symbol.
A @i(separator) is a "blank" character
@index(separator)
(space, vertical tab, horizontal tab, carriage return, newline, form feed)
or a comment.
Any number of separators may appear between tokens.

@Section(Reserved Words)

The following character sequences
@index(reserved word)
are reserved word tokens:
@Begin(Table)
@Caption(Reserved Words)
@begin[Display]
@Bar()
@TabClear()
@TabDivide(4)
abort @\else @\leave @\signals
action @\elseif @\mutex @\stable
any @\end @\nil @\string
array @\enter @\node @\struct
atomic@us<>array @\equates @\null @\tag
atomic@us<>record @\except @\oneof @\tagcase
atomic@us<>variant @\exit @\others @\tagtest
background @\false @\own @\tagwait
begin @\for @\pause @\terminate
bind @\foreach @\proc @\then
bool @\fork @\process @\topaction
break @\guardian @\proctype @\transmit
cand @\handler @\real @\true
char @\handlertype @\record @\type
cluster @\handles @\recover @\up
coenter @\has @\rep @\variant
continue @\if @\resignal @\when
cor @\image @\return @\where
creator @\in @\returns @\while
creatortype @\int @\seize @\with
cvt @\is @\self @\wtag
do @\iter @\sequence @\yield
down @\itertype @\signal @\yields
@end[Display]
@Bar()
@end(Table)
Upper and lower case letters are not distinguished in reserved words.
For example, 'end', 'END', and 'eNd' are all the same reserved word.
Reserved words appear in @b(bold) face in this document.

@Section(Identifiers)

An @i(identifier) is a sequence of
@index(identifier)@SeeAlso(Primary="identifier", other="name")
@SeeAlso(Primary="identifier", other="idn")
letters,
digits,
and underscores (@us<>) that begins with a letter or underscore,
and that is not a reserved word.
Upper and lower case letters are not distinguished in identifiers.

In the syntax there are two different nonterminals for identifiers.
The nonterminal @i(idn) is used when the identifier has scope
(see Section @ref(sect-scope));
idns are used for variables, parameters, module names,
and as abbreviations for constants.
The nonterminal @i(name) is used when the identifier is not subject
to scope rules;
names are used for record and structure selectors,
oneof and variant tags,
operation names,
and exceptional condition names.

@Section(Literals)

There are literals for naming objects of the built-in types
@b[null], @b[bool], @b[int], @b[real], @b[char], and @b[string].
Their forms are described in Appendix @ref(apdx-syntax).
@index(literal)

@Section(Operators and Punctuation Tokens)

The following character sequences are used as operators and punctuation tokens.
@index(operator)@index(punctuation token)
@begin(Table)
@Caption(Operator and Punctuation Tokens)
@Bar()
@begin[Display]
@TabClear()
@TabDivide(8)
( @\[ @\. @\~ @\* @\< @\~< @\=
) @\] @\$ @\** @\|| @\<= @\~<= @\~=
{ @\: @\:= @\// @\+ @\>= @\~>= @\&
} @\, @\@@ @\/ @\@MinusSign() @\> @\~> @\|
@end[Display]
@Bar()
@end(Table)

@Section(Comments and Other Separators)

A @i(comment)
@index(comment)@IndexEntry(Key=" %", Entry="%", Number)
is a sequence of characters that
begins with a percent sign (%),
ends with a newline character,
and contains only printing ASCII characters (including blanks)
and horizontal tabs in between.
For example:
@begin[Display]
@TabClear()
z := a[i] + @^% a comment in an expression
@\b[i]
@end[Display]

A @i(separator) is a blank character
@index(separator)
(space, vertical tab, horizontal tab, carriage return, newline, form feed)
or a comment.
Zero or more separators may appear between any two tokens,
except that at least one separator is required between any two adjacent
non-self-terminating tokens:
reserved words,
identifiers,
integer literals,
and real literals.
This rule is necessary to avoid lexical ambiguities.
