@Part(modules, Root "manual.mss")

@Chapter(Modules)

Besides guardian modules,
@index(module)
Argus has procedure, iterator, cluster, and equate modules.
@begin[SyntaxDisplay]
@TabClear()
module @^@BNFequals() @BNFlcurly() equate @BNFrcurly() guardian
 @\@BNFor()   @BNFlcurly() equate @BNFrcurly() procedure
 @\@BNFor()   @BNFlcurly() equate @BNFrcurly() iterator
 @\@BNFor()   @BNFlcurly() equate @BNFrcurly() cluster
 @\@BNFor()   @BNFlcurly() equate @BNFrcurly() equates
@end[SyntaxDisplay]
Guardians are discussed in Section @ref(sect-guardian),
the rest are described below.


@Section(Procedures)
@Label(sect-procedure)

A procedure performs an action on zero or more
@i(arguments), and when it terminates it returns zero or more @i(results).
@index(procedure)@index(routine)@SeeAlso(Primary="routine", other="procedure")
@SeeAlso(Primary="routine", other="iterator")
A procedure implements a @i(procedural abstraction):  a mapping from
a set of argument objects to a set of result objects, with possible
modification of some of the argument objects.
A procedure may terminate in one of a number of @i(conditions);
one of these is the @i(normal condition), while others are
@i(exceptional conditions).
Differing numbers and types of results may be returned in the
different conditions.

The form of a procedure is:
@begin[SyntaxDisplay]
@TabClear()
idn = @^@b[proc] @BNFlbkt() parms @BNFrbkt() args @BNFlbkt() returns @BNFrbkt() @BNFlbkt() signals @BNFrbkt() @BNFlbkt() where @BNFrbkt()
 @\routine@us<>body
 @\@b[end] idn
@end[SyntaxDisplay]
where
@begin[SyntaxDisplay]
@TabClear()
@TabSet(1.2 inches)
args @\@BNFequals() ( @BNFlbkt() decl , @BNFdots() @BNFrbkt() )
returns @\@BNFequals() @b[returns] ( type@us<>spec , @BNFdots() )
signals @\@BNFequals() @b[signals] ( exception , @BNFdots() )
exception @\@BNFequals() name @BNFlbkt() ( type@us<>spec , @BNFdots() ) @BNFrbkt()
routine@us<>body @\@BNFequals() @^@BNFlcurly() equate @BNFrcurly()
 @\ @\@BNFlcurly() own@us<>var @BNFrcurly()
 @\ @\@BNFlcurly() statement @BNFrcurly()
@end[SyntaxDisplay]

In this section we discuss non-parameterized procedures,
in which the @i(parms) and @i(where) clauses are missing.
Parameterized modules are discussed in Section @ref(sect-parms).
Own variables are discussed in Section @ref(sect-owns).

The heading of a procedure describes the way in which the
procedure communicates with its caller.
The @i(args) clause specifies the number, order, and types of
arguments required to call the procedure, while the @i(returns)
clause specifies the number, order, and types of results returned
when the procedure terminates normally
(by executing a @b[return] statement or reaching the end of its body).
A missing @i(returns) clause indicates that no results are returned.

The @i(signals) clause names the exceptional conditions
in which the procedure can terminate, and specifies the
number, order, and types of result objects returned in each condition.
All names of exceptions in the @i(signals) clause must be distinct.
The @i(idn) following the @b[end] of the procedure
must be the same as the @i(idn) naming the procedure.

A procedure is an object of some procedure type.
For a non-parameterized procedure, this type
is derived from the procedure heading by removing the
procedure name, rewriting the formal argument declarations with
@IndexSecondary(Primary="formal", Secondary="argument")
one @i(idn) per @i(decl),
deleting the @i(idns) of all formal arguments,
and finally,
replacing @b[proc] by @b[proctype].

The call of a procedure causes the introduction of the formal
variables, and the actual arguments are assigned to these variables.
Then the procedure body is executed.
Execution terminates when a @b[return] statement or a @b[signal]
statement is executed, or when the textual end of the body is reached.
If a procedure that should return results reaches the textual end
of the body, the guardian crashes due to this error.
At termination the result objects, if any, are passed back
to the caller of the procedure.

@Section(Iterators)
@Label(sect-iterators)

An iterator computes a sequence of @i(items),
@index(iterator)@index(routine)
one item at a time, where an item is a group of zero or more objects.
In the generation of such a sequence, the computation of
each item of the sequence is usually controlled by information about
what previous items have been produced.
Such information and the way it controls the production of items
is local to the iterator.
The user of the iterator is not concerned with how the items
are produced, but simply uses them (through a @b[for] statement)
as they are produced.
Thus the iterator abstracts from the details of how the production
of the items is controlled;
for this reason, we consider an iterator to implement a control abstraction.
Iterators are particularly useful as operations of data abstractions
that are collections of objects (e.g., sets), since they may produce the
objects in a collection without revealing how the collection
is represented.

An iterator has the form:
@begin[SyntaxDisplay]
@TabClear()
idn = @^@b[iter] @BNFlbkt() parms @BNFrbkt() args @BNFlbkt() yields @BNFrbkt() @BNFlbkt() signals @BNFrbkt() @BNFlbkt() where @BNFrbkt()
 @\routine@us<>body
 @\@b[end] idn
@end[SyntaxDisplay]
where
@begin[SyntaxDisplay]
@TabClear()
yields @^@BNFequals() @b[yields] ( type@us<>spec , @BNFdots() )
@end[SyntaxDisplay]
In this section we discuss non-parameterized iterators,
in which the @i(parms) and @i(where) clauses are missing.
Parameterized modules are discussed in Section @ref(sect-parms).
Own variables are discussed in Section @ref(sect-owns).

The form of an iterator is similar to the form of a procedure.
There are only two differences:
@begin(enumerate)
An iterator has a @i(yields) clause in its heading in
place of the @i(returns) clause of a procedure.
The @i(yields) clause specifies the number, order, and types
of objects yielded each time the iterator produces the next item in the sequence.
If zero objects are yielded,
then the @i(yields) clause is omitted.
The @i(idn) following the @b[end] of the iterator
must be the same as the @i(idn) naming the iterator.

Within the iterator body, the @b[yield] statement is used to
present the caller with the next item in the sequence.
An iterator terminates in the same manner as a procedure,
but it may not return any results.
@end(enumerate)

An iterator is an object of some iterator type.
For a non-parameterized iterator,
this type is derived from the iterator heading by removing the
iterator name, rewriting the formal argument declarations with
one @i(idn) per @i(decl),
deleting the @i(idns) of all formal arguments,
and finally,
replacing @b[iter] by @b[itertype].

An iterator can be called only by a @b[for] statement or by a @b[foreach]
clause in a @b[coenter] statement.


@Section(Clusters)
@Label(sect-cluster)

A cluster is used to implement a new data type, distinct
@index(cluster)@index(data type)@index(data abstraction)@index(type)
@index(object)@IndexSecondary(Primary="object", Secondary="implementation of")
@IndexSecondary(Primary="type", Secondary="implementation of")
@IndexSecondary(Primary="type", Secondary="user-defined")
from any other built-in or user-defined data type.
A data type (or data abstraction) consists of a set of objects
and a set of primitive operations.
The primitive operations provide the most basic ways of manipulating the objects;
ultimately every computation that can be performed on the objects must be
expressed in terms of the primitive operations.
Thus the primitive operations define the lowest level
of observable object behavior@Foot{
Readers not familiar with the concept of data abstraction might read
Liskov, B. and Guttag, J.,
@i(Abstraction and Specification in Program Development),
MIT Press, Cambridge, 1986.}.

The form of a cluster is:
@begin[SyntaxDisplay]
@TabClear()
idn = @^@b[cluster] @BNFlbkt() parms @BNFrbkt() @b[is] opidn , @BNFdots() @BNFlbkt() where @BNFrbkt()
 @\cluster@us<>body
 @\@b[end] idn
@end[SyntaxDisplay]
where
@begin[SyntaxDisplay]
@TabClear()
opidn @^@BNFequals() idn
 @\@BNFor() @b[transmit]

@TabClear()
cluster@us<>body @^@BNFequals() @^@BNFlcurly() equate @BNFrcurly() @b[rep] = type@us<>spec @BNFlcurly() equate @BNFrcurly()
 @\ @\@BNFlcurly() own@us<>var @BNFrcurly()
 @\ @\routine @BNFlcurly() routine @BNFrcurly()

routine @^@BNFequals() procedure
 @\@BNFor() iterator
@end[SyntaxDisplay]
In this section we discuss non-parameterized clusters,
in which the @i(parms) and @i(where) clauses are missing.
Parameterized modules are discussed in Section @ref(sect-parms).
Own variables are discussed in Section @ref(sect-owns).

The primitive operations are named by
@index(operation)
the list of @i(opidns) following the reserved word @b[is].
All of the @i(opidns) in this list must be distinct.
The @i(idn) following the @b[end] of the cluster must be the same as
the @i(idn) naming the cluster.

To define a new data type, it is necessary to choose a
@i(concrete representation) for the objects of the type.
@index(representation)@IndexSecondary(Primary="object", Secondary="representation")
The special equate:
@begin[Display]
@b[rep] = type@us<>spec
@end[Display]
within the cluster body identifies the @i(type@us<>spec) as the concrete representation.
Within the cluster, @b[rep] may be used as an abbreviation for this @i(type@us<>spec).

The identifier naming the cluster is available for use in the cluster body.
Use of this identifier within the cluster body permits the
definition of recursive types.

In addition to giving the representation of objects,
the cluster must implement the primitive operations of the type.
One exception to this, however, is the @b[transmit] operation.
@index(transmit)
The transmit operation is not directly implemented by a cluster;
instead, the cluster must implement two operations:  @i(encode) and @i(decode)
(see Section @ref(sect-transmit) for details).
The primitive operations may be either procedural or control abstractions;
they are implemented by procedures and iterators, respectively.
Any additional routines implemented within the cluster are @i(hidden):
@index(hidden routine)@index(private routine)
they are private to the cluster and may not
be named directly by users of the abstract type.
All the routines must be named by distinct identifiers;
the scope of these identifiers is the entire cluster.
@index(scope)

Outside the cluster, the type's objects may only be treated
abstractly (i.e., manipulated by using the primitive operations).
@index(object)@IndexSecondary(Primary="object", Secondary="abstract")
@IndexSecondary(Primary="object", Secondary="concrete")
@IndexSecondary(Primary="representation", Secondary="concrete")
To implement the operations, however, it is usually necessary
to manipulate the objects in terms of their concrete
representation.
It is also convenient sometimes to manipulate the objects abstractly.
Therefore, inside the cluster it is possible to view the type's
objects either abstractly or in terms of their representation.
The syntax is defined to specify unambiguously, for each variable that
refers to one of the type's objects, which view is being taken.
Thus, inside a cluster named T, a declaration:
@index(declaration)
@begin[Display]
v: T
@end[Display]
indicates that the object referred to by @i(v) is to be
treated abstractly, while a declaration:
@begin[Display]
w: @b[rep]
@end[Display]
indicates that the object referred to by @i(w) is to be treated concretely.
@index(up)@index(down)
Two primitives, @b[up] and @b[down], are available for converting
between these two points of view.
The use of @b[up] permits a type @b[rep] object to be viewed abstractly,
while @b[down] permits an abstract object to be viewed concretely.
For example,
given the declarations above,
the following two assignments are legal:
@begin[Display]
v := @b[up](w)
w := @b[down](v)
@end[Display]
Only routines inside a cluster may use @b[up] and @b[down].
Note that @b[up] and @b[down] are used merely to inform the compiler that the
object is going to be viewed abstractly or concretely, respectively.

A common place where the view of an object changes is at the
interface to one of the type's operations:  the user, of course, views
the object abstractly, while inside the operation, the object is viewed concretely.
To facilitate this usage, a special type specification, @b[cvt], is provided.
@index(cvt)
The use of @b[cvt] is restricted to the @i(args), @i(returns), @i(yields) and
@i(signals) clauses of routines inside a cluster, and may be used at the
top level only (e.g., @b[array][@b[cvt]] is illegal).
When used inside the @i(args) clause, it means that the view of the
argument object changes from abstract to concrete when it is assigned
to the formal argument variable.
When @b[cvt] is used in the @i(returns), @i(yields), or @i(signals)
clause, it means the view of the result object changes from concrete to abstract
as it is returned (or yielded) to the caller.
Thus @b[cvt] means abstract outside, concrete inside:
when constructing the type of a routine,
@b[cvt] is equivalent to the abstract type,
but when type-checking the body of a routine,
@b[cvt] is equivalent to the representation type.
The type of each routine is derived from its heading in the usual manner,
except that each occurrence of @b[cvt] is replaced by the abstract type.
The @b[cvt] form does not introduce any new ability
over what is provided by @b[up] and @b[down].
It is merely a shorthand for a common case.

Inside the cluster, it is not necessary to use the compound form
(@i(type@us<>spec)$@i(op@us<>name)) for naming locally defined routines.
@IndexEntry(Key=" $", Entry="$", Number)
Furthermore, the compound form cannot be used for calling hidden routines.


@Section(Equate Modules)
@Label(sect-equate-module)@Label(sect-equmod)

An equate module provides a convenient way to define a set of equates
for later use by other modules.
@index(equate module)@index(equate)

The form of an equate module is:
@begin[SyntaxDisplay]
@TabClear()
idn = @^@b[equates] @BNFlbkt() parms @BNFlbkt() where @BNFrbkt() @BNFrbkt()
 @\equate @BNFlcurly() equate @BNFrcurly()
 @\@b[end] idn
@end[SyntaxDisplay]
The usual scope rules apply.
The @i(idn) following the @b[end] of the equate module
must be the same as the @i(idn) naming the equate module.

In this section we discuss non-parameterized equate modules.
Parameterized modules are discussed in Section @ref(sect-parms).

An equate module defines a set of equates,
that is, it defines a set of named constants.
The set of equates is also a constant, although it is not an object.
Thus the name of an equate module can be used in an equate,
but an equate module cannot be assigned to a variable.
The equates defined by an equate module @i(E) may be referenced
using the same syntax as for naming the operations of a cluster.
For example, an object or type named @i(n) in equate module @i(E) can be
referred to as @i(E)$@i(n).
If equate modules contain equates that give names to other equate modules,
compound names can be used.  For example:
@begin[Display]
A[@b[int]]$B$C$name
@end[Display]
where @i(A), @i(B), and @i(C) are equate modules is legal.

As always, equates to type specifications do not define new types but merely
abbreviations for types.
For example, in the following:
@begin[Display]
@TabClear()
my@us<>types@^ = @b[equates]
 @\ai = @b[array][@b[int]]
 @\float = @b[real]
 @\@b[end] my@us<>types
@end[Display]
the types @i(my@us<>types)$@i(ai) and @b[array][@b[int]] are equivalent.


@Section(Parameterized Modules)
@Label(sect-param)@Label(sect-parm)@Label(sect-parms)@Label(sect-typegen)

Procedures, iterators, clusters, guardians (see Section @ref(sect-guardian)),
and equate modules may all be parameterized.
@index(parameterization)@index(generator)
@IndexSecondary(Primary="type", Secondary="generator")
@IndexSecondary(Primary="type", Secondary="parameterized")
@IndexSecondary(Primary="module", Secondary="parameterized")
Parameterization permits a set of related abstractions to be defined
by a single module.
In each module heading there is an optional
@i(parms) clause and an optional @i(where) clause
(see Appendix @ref(apdx-syntax)).
The presence of the @i(parms) clause indicates that the module
is parameterized;
the @i(where) clause declares the types of any operation parameters
that are expected to accompany the formal type parameters.

The form of the @i(parms) clause is:
@begin[SyntaxDisplay]
[ parm , @BNFdots() ]
@end[SyntaxDisplay]
where
@begin[SyntaxDisplay]
parm @^@BNFequals() idn , @BNFdots() : type@us<>spec
 @\@BNFor() idn , @BNFdots() : @b[type]
@end[SyntaxDisplay]
Each parm declares some number of formal parameters.
@index(parameter)@index(actual parameter)
@IndexSecondary(Primary="formal", Secondary="parameter")
@IndexSecondary(Primary="parameter", Secondary="versus argument")
@IndexSecondary(Primary="argument", Secondary="versus parameter")
Only the following types of parameters can be declared in a @i(parms) clause:
@b[int], @b[real], @b[bool], @b[char], @b[string], @b[null], and @b[type].
The declaration of operation parameters associated with type parameters
is done in the @i(where) clause, as discussed below.
The actual values for parameters
are required to be constants that can be computed at compile-time.
This requirement ensures that all types are known at compile-time, and
permits complete compile-time type-checking.

In a parameterized module, the scope rules permit the parameters
to be used throughout the module.
Type parameters can be used freely as type specifications, and
all other parameters (including the operations parameters specified in
the @i[where] clause) can be used freely as expressions.

A parameterized module implements a set of related abstractions.
A program must @i(instantiate) a parameterized module before it can be used;
that is, it must provide actual, constant values for the parameters
(see Section @ref(sect-instantiations)).
@index(instantiate)
@IndexSecondary(Primary="module", Secondary="instantiation of")
The result of an instantiation is a procedure, iterator, type, guardian,
or equate module that may be used just like a non-parameterized module
of the same kind.
Each distinct list of actual parameters produces
a distinct procedure, iterator, type, guardian, or equate module
(see Section @ref(sect-instantiations) for details).

The meaning of a parameterized module is given by binding the actual parameters
to the formal parameter names
and deleting the @i(parms) clause and the @i[where] clause.
That is, in an an instantiation of a parameterized module,
each formal parameter name denotes the corresponding actual parameter.
The resulting module is a regular (non-parameterized) module.
In the case of a cluster some of the operations may have additional parameters;
further bindings take place when these operations are instantiated.

In the case of a type parameter,
one can also declare what operation parameters must accompany the type
by using a @i[where] clause.
@index(where clause)
The @i[where] clause also specifies the type of each required operation
parameter.
The @i[where] clause constrains the parameterized module as well:
the only operations of the type parameter that can be used
are those listed in the @i[where] clause.

The form of the @i[where] clause is:
@index(restriction)
@IndexSecondary(Primary="type", Secondary="set")
@begin[SyntaxDisplay]
@TabClear()@TabSet(1.1inches, 1.5inches, 2inches, 2.8inches, 4inches)
where @^@BNFequals() @b[where] restriction , @BNFdots()
restriction @BNFequals() idn @b[has] oper@us<>decl , @BNFdots()
 @\@BNFor() @\idn @b[in] type@us<>set
oper@us<>decl @BNFequals() name , @BNFdots() : type@us<>spec
 @\@BNFor() @\@b[transmit]
type@us<>set @BNFequals() { idn | idn @b[has] oper@us<>decl , @BNFdots() @BNFlcurly() equate @BNFrcurly() }
 @\@BNFor() @\idn
 @\@BNFor() @\reference $ name
@end[SyntaxDisplay]

There are two forms of restrictions.
@index(restriction)
In both forms,
the initial @i(idn) must be a type parameter.
The @b[has] form lists the set of required operation parameters directly,
by means of @i(oper@us<>decls).
The @i(type@us<>spec) in each @i(oper@us<>decl) must be a
@b[proctype], @b[itertype], or @b[creatortype] (see Appendix @ref(apdx-syntax)).
The @b[in] form requires that the actual type be a member of a @i(type@us<>set),
a set of types with the required operations.
@index(required operation)
The two identifiers in the @i(type@us<>set) must match,
and the notation is read like set notation; for example,
@begin[SyntaxDisplay]
{t | t @b[has] f: ... }
@end[SyntaxDisplay]
means "the set of all types @i(t) such that @i(t) @b[has] @i(f) ...".
The scope of the identifier is the type@us<>set.

The @b[in] form is useful because an abbreviation can be given
for a @i(type@us<>set) via an equate.
If it is helpful to introduce some abbreviations in defining the
@i(type@us<>set),
these are given in the optional equates within the @i(type@us<>set).
The scope of these equates is the entire @i(type@us<>set).

A routine in a parameterized cluster may have a @i[where] clause in
its heading, and can place further constraints on the cluster parameters.
For example, any type is permissible for the array element type,
but the array @i(similar) operation requires that the element type
have a @i(similar) operation.
This means that @b[array][@i(T)] exists for any type @i(T),
but that @b[array][@i(T)]$@i(similar)
exists only when an actual operation parameter
is provided for @i(T$similar) (see Section @ref(sect-instantiations)).
Note that a routine need not include in its @i[where] clause any of
the restrictions included in the cluster @i[where] clause.

@Section(Instantiations)
@Label(sect-instantiations)
@index(instantiation)@index(instance)@index(constant)@index(type)
@IndexSecondary(Primary="module", Secondary="instantiation of")
@IndexSecondary(Primary="parameterized type", Secondary="instantiation of")
@index(parameterized type)
@IndexSecondary(Primary="type", Secondary="generator")
@index(with)

To instantiate a parameterized module, constants or type specifications
are provided as actual parameters:
@index(actual parameter)
@index(opbinding)
@IndexSecondary(Primary="type", Secondary="actual")
@IndexSecondary(Primary="type", Secondary="parameter")
@IndexSecondary(Primary="generator", Secondary="instantiation")
@IndexSecondary(Primary="parameter", Secondary="actual")
@begin[SyntaxDisplay]
@TabClear()
actual@us()parm @^ @BNFequals() @^ constant
@\@BNFor()@\type@us<>actual

type@us<>actual @BNFequals() type@us()spec @BNFlbkt() @b[with] { opbinding , @BNFdots() } @BNFrbkt()

opbinding @BNFequals() name , @BNFdots() : primary
@end[SyntaxDisplay]
If the parameter is a type, the module's @i(where) clause
may require that some routines be passed as parameters.
These routines can be passed implicitly by omitting the @i(with) clause;
the routine selected as a default will be the operation
of the type that has the same name as that used in the @i(where) clause.
Routines may also be passed explicitly by using the @i(with) clause,
overriding the default.
In this case, the actual routine parameter
need not have the same name as is required
in the @i(where) clause, and need not even be one of the type's 
primitive operations.

The syntactic sugar that allows default routines to be selected implicitly
works as follows.
If a generator requires an operation named @i(op) from
a type parameter, and if the corresponding @i(type@us<>actual),
@i(TS) @b[with] { ... },
has no explicit binding for @i(op),
then Argus adds an @i(opbinding) of @i(op) to @i(TS$op).
(It will be an error if @i(TS$op) is not defined.)
Thus one only has to provide an explicit @i(opbinding)
if the default is unsatisfactory.

For example, suppose a procedure generator named @i(sort)
has the following heading:
@begin[Display]
sort = @b(proc)@^[t: @b[type]](a: @b[array][t]) @b[where] t @b[has] gt: @b[proctype](t,t) @b[returns](@b[bool])
@end[Display]
and consider the three instantiations:
@begin[Display]
sort[@b(int) @b(with) {gt: @b(int)$gt} ]
sort[@b(int)]
sort[@b(int) @b(with) {lt: @b(int)$lt} ]
@end[Display]
The first two instantiations are equivalent; in the first the routine
@b(int)$@i(gt) is passed explicitly,
while in the second it is passed implicitly as the default.
In the third instantiation, however,  @b[int]$@i(lt) is
passed in place of the default.
All three instantiations result in a routine of type:
@begin(display)
@b(proctype) (@b(array)[@b(int)])
@end(display)
and so each could be called by passing it an 
@b[array][@b[int]] as an argument.
@IndexSecondary(Primary="type", Secondary="versus type actual")
However a call of the third instantiation will sort
its array argument
in the opposite order from a call of either the first or second
instantiation.

Within an instantiation of a parameterized module,
an operation of a type parameter named @i(t)$@i(op)
denotes the actual routine parameter bound to @i(op) in the instantiation of
that module.
For example, suppose we make the call:
@begin[Display]
sort[@b(int) @b(with) {gt: @b(int)$lt} ] (my@us<>ints)
@end[Display]
where @i(my@us()ints) is an array of integers.
If, in the body of @i(sort), there is a recursive call:
@begin(Display)
sort[t @b[with] {gt: t$gt} ] (a, i, j)
@end(Display)
then @i(t) denotes the type @b[int], and @i(t)$@i(gt) denotes the routine
@b[int]$@i(lt), so that the recursive sort happens in the correct order.

A cluster generator may include routines with @i(where) clauses that place
additional requirements on the cluster's type parameters.
A common example is to require a @i(copy) operation only
within the cluster's @i(copy) implementation.
@begin[Display]
set = @b(cluster)[t: @b(type)] @b(is) ..., copy 
             @b(where) t @b(has) equal: @b(proctype)(t,t) @b(returns)(@b(bool))
@b(rep) = @b(array)[t]
...
copy = @b(proc)(s: @b(cvt)) @b(returns)(@b(cvt)) @b(where) t @b(has) copy: @b(proctype)(t) @b(returns)(t)
    @b(return)(@b(rep)$copy(s))
    @b(end) copy
@end[Display]
The intent of these subordinate @b(where) clauses is to allow more operations
to be
defined if the actual type parameter has the additional required operations,
but not to make the additional operations an absolute requirement
for obtaining an instance of the type generator.
@IndexSecondary(Primary="where clause", Secondary="subordinate")
@index(optional parameter)
@IndexSecondary(Primary="parameter", secondary="optional")
For example, with the above definition of @i(set), @i{set}[@b(any)]
would be defined, but @i(set)[@b(any)]$@i(copy) would not be defined
because @b(any) does not have a @i(copy) operation.
We shall call the routine parameters required by subordinate @i(where) clauses
@i(optional parameters).

Like regular required parameters, optional parameters can be provided when
the cluster as a whole is instantiated and can be provided
explicitly or by default.
For any optional parameter @i(op) that is not provided explicitly
by the @i(type@us()actual), @i(TS) @b[with] { ... },
we add an @i(opbinding) of @i(op) to
@i(TS)$@i(op) if @i(TS)$@i(op) exists; otherwise the @i(opbinding)
is not added.
The resulting cluster contains just those operations for which @i(opbindings)
exist for all the required routine parameters.
For example, as mentioned above, set[@b(any)] would not
have a @i(copy) operation because @b(any)$@i(copy) does not exist and therefore
the needed @i(opbinding) is not present.
On the other hand, set[@b(int)] does have a @i(copy) operation because
@b(int)$@i(copy) does exist.
Finally, @i(set)[@b(any) @b(with) {@i(copy): @i(foo)}],
where @i(foo) is a procedure
that takes an @b(any) as an argument and returns an @b(any) as a result,
would have a @i(copy) operation.

For an instantiation to be legal it must type check.
Type checking is done after the syntactic sugars are applied.
@index(type checking)
@IndexSecondary(Primary="type checking", Secondary="of an instantiation")
@IndexSecondary(Primary="instantiation", Secondary="type checking of")
The types of constant parameters must be included in the declared
type, type actuals must be types, and the
types of the actual routine parameters must be included in the
proctypes, itertypes, or creatortypes declared in the appropriate @i[where]
clauses.
Of course, the number of parameters declared must match the
number of actuals passed and
with each type actual parameter there must be an
@i(opbinding) for each required routine parameter.
If the generator is a cluster, then @i(opbindings) must be provided for
all operations required in the cluster's @i[where] clause;
@i(opbindings) can (but need not) be provided for optional parameters.
Extra actual routine parameters are illegal.

Because the meaning of an instantiation
may depend on the actual routine parameters,
type equality makes instances with different actual routine parameters
distinct types.
For example, consider the @i(set) type generator again;
the instance
@begin[Display]
set[ @b(array)[@b(int)] @b(with) {equal: @b(array)[@b(int)]$equal} ]
@end[Display]
is not equal to
@begin[Display]
set[ @b(array)[@b(int)] @b(with) {equal: @b(array)[@b(int)]$similar} ]
@end[Display]
Intuitively these instances should be unequal
because the two @i(equal) procedures define different equivalence classes
and therefore
the abstract behaviors of the two instances are different.
However, optional parameters do not effect type equality.
For example,
@begin(display)
set[@b(array)[@b(int)] @b(with) {copy: @b(int)$copy} ]
@end(display)
and
@begin(display)
set[@b(array)[@b(int)] @b(with) {copy: my@us<>copy} ]
@end(display)
are equal types.
This is intuitively justified because in each case set objects behave the
same way even though different sets are produced when sets are copied in
the two cases.

Thus we have the following type equality rule,
which defines when two @i(type@us()specs) denote equal types
(after syntactic sugars are applied).
@IndexSecondary(Primary="type", Secondary="equality")
@IndexSecondary(Primary="routine", Secondary="equality")
A similar notion is also needed for routine equality.
A formal type identifier is only equal to itself
for type checking purposes.
Otherwise, two type names denote equal types
if they denote the same Description Unit
@index(description unit)
(DU).@Foot{
This is name equality unless the type environment has synonyms for types.}
Similarly, Argus compares the names of routine formals or the DUs of routines,
or checks that they are the same operation in equal types.
To decide the equality of two type generator instantiations:
@begin[Display]
T[t@-{1} @b(with) {op@-{1}: act@-{1}, ... op@-{m}: act@-{m}} , ..., t@-{n} @b(with) {...} ]
and
T'[t@-{1}' @b(with) {op@-{1}: act@-{1}', ... op@-{m}: act@-{m}'} , ..., t@-{n}' @b(with) {...} ]
@end[Display]
Argus first checks whether:
@begin(enumerate)
@i(T) and @i(T)' denote the same DU, and whether

they have the same number of @i(type@us<>actuals),
and @i(t)@-{1} is equal to @i(t)@-{1}', etc.
@end(enumerate)
Second, any optional parameter @i(opbindings) in either instantiation
are deleted.
After this step, Argus checks that
for each corresponding @i(type@us<>actual) there is the same number of
@i(opbindings)
and that each corresponding @i(opbinding) is the same.
(That is, the corresponding actual routines are equal.)
The order of the actual routine parameters does not matter,
since Argus matches @i(opbindings) by operation names.
(The definition of routine equality for instantiations of routine generators
is similar.)
This definition, for example, tells us that
@begin[Display]
set[ @b(array)[@b(int)] @b(with) {equal: @b(array)[@b(int)]$equal} ]
@end[Display]
is different from
@begin[Display]
set[ @b(array)[@b(int)] @b(with) {equal: @b(array)[@b(int)]$similar} ]  ,
@end[Display]
(assuming @i(set) requires an @i(equal) operation from its type parameter).
It also tells us that:
@begin[Display]
set[ @b(int) @b(with) {equal: foo, copy: bar} ]
@end[Display]
and
@begin[Display]
set[ @b(int) @b(with) {equal: foo, copy: xerox} ]
@end[Display]
are equal (assuming @i(copy) is required only by the @i(set)[@b[int]]$@i(copy)
operation).

This type equality rule allows programmers to control
what requirements affect type equality by choosing whether
to put them on a cluster or on each operation.
@index(optional parameter)
A requirement on the cluster should be used whenever the actuals make
some difference in the abstraction.
For example, in the @i(set) cluster, the type parameter's @i(equal) operation
should be required by the cluster as a whole,
since using different equality tests for a set's objects causes the
set's behavior to change.

One can require that a type parameter, say @i(t),
be transmissible by stating the requirement:
@begin[Display]
t @b(has) @b(transmit)
@end[Display]
This requirement is regarded as a formal parameter declaration for
a special "transmit actual",
@index(transmit)@IndexSecondary(Primary="transmit", Secondary="actual")
but Argus does not provide syntax for passing it explicitly.
The "transmit actual" is passed implicitly just when the actual type parameter
is transmissible and the generator requires it.


@Section(Own Variables)
@Label(sect-owns)

Occasionally it is desirable to have a module that retains information
internally between calls.
@index(own variable)@index(own data)
@IndexSecondary(Primary="variable", Secondary="own variable")
Without such an ability,
the information would either have to be reconstructed at every call,
which can be expensive (and may even be impossible if the information
depends on previous calls),
or the information would have to be passed in through arguments,
which is undesirable because the information is then subject to
uncontrolled modification in other modules
(but see also the binding mechanism described in Section @ref(sect-bind)).

Procedures, iterators, handlers, creators, and clusters
may all retain information
through the use of @b[own] variables.
An @b[own] variable is similar to a normal variable,
except that it exists for the life of the program or guardian,
rather than being bound to the life of any particular routine activation.
Syntactically,
@b[own] variable declarations must appear immediately after the equates
in a routine or cluster body;
they cannot appear in bodies nested within statements.
Declarations of @b[own] variables have the form:
@begin[SyntaxDisplay]
@TabClear()
own@us<>var @^@BNFequals() @b[own] decl
 @\@BNFor() @b[own] idn : type@us<>spec := expression
 @\@BNFor() @b[own] decl , @BNFdots() := call @BNFlbkt() @@ primary @BNFrbkt()
@end[SyntaxDisplay]
Note that initialization is optional.

The @b[own] variables of a module are created when a guardian
begins execution or recovers from a crash,
and they always start out uninitialized.
@index(crash)
@IndexSecondary(Primary="own variable", Secondary="and crash recovery")
@IndexSecondary(Primary="crash", Secondary="and own variables")
The @b[own] variables of a routine (including cluster operations)
are initialized in textual order as part of the first call of an operation
of that routine (or the first such call after a crash),
before any statements in the body of the routine are executed.
Cluster @b[own] variables are initialized in textual order
as part of the first call of the first cluster operation to be called
(even if the operation does not use the @b[own] variables).
Cluster @b[own] variables are initialized
before any operation @b[own] variables are initialized.
Argus insures that only one process can execute a cluster's or a routine's
@b[own] variable initializations.

Aside from the placement of their declarations,
the time of their initialization,
and their lifetime,
@b[own] variables act just like normal variables
and can be used in all the same places.
As with normal variables,
an attempt to use an uninitialized @b[own] variable
(if not detected at compile-time)
will cause the guardian to crash.

Declarations of @b[own] variables in different modules always refer to
distinct own variables,
and distinct guardians never share @b[own] variables.
Furthermore,
@b[own] variable declarations within a parameterized module produce
distinct @b[own] variables for each distinct instantiation of the module.
For a given instantiation of a parameterized cluster,
all instantiations of the type's operations share the same set of
cluster @b[own] variables,
but distinct instantiations of parameterized operations have distinct
routine @b[own] variables.

Declarations of @b[own] variables cannot be enclosed by an @b[except] statement,
so care must be exercised when writing initialization expressions.
If an exception is raised by an initialization expression,
it will be treated as an exception raised,
but not handled,
in the body of the routine whose call caused the initialization
to be attempted.
Thus, the guardian will crash due to this error.
