@Part(guardians, Root "manual.mss")

@Chapter(Guardians)
@Label(sect-guardian)

This section is concerned with the form and
meaning of the modules used to define guardians.
@index(guardian)@IndexSecondary(Primary="guardian", Secondary="definition")
@index(module)
Such a module, called a @i(guardian definition),
declares the objects making up the guardian's
stable state and volatile state,
@IndexSecondary(Primary="volatile", Secondary="state")
and provides implementations for the guardian's handlers.
It also defines one or more @i(creators):
operations that produce new guardians that behave in accordance with the
guardian definition.
In addition, a guardian definition may provide @i(background code) to
carry out independent activities, and @i(recovery code) to restore the
volatile state when the guardian is restarted after a crash.

The syntactic form of a guardian definition is as follows:
@begin[SyntaxDisplay]
@TabClear()
idn = @^@b[guardian] @BNFlbkt() parms @BNFrbkt() @b[is] idn , @BNFdots() @BNFlbkt() @~
@b[handles] idn , @BNFdots() @BNFrbkt() @BNFlbkt() where @BNFrbkt()
 @\@BNFlcurly() equate @BNFrcurly()
 @\@BNFlcurly() state@us<>decl @BNFrcurly()
 @\@BNFlbkt() @b[recover] body @b[end] @BNFrbkt()
 @\@BNFlbkt() @b[background] body @b[end] @BNFrbkt()
 @\@BNFlcurly() operation @BNFrcurly() creator @BNFlcurly() operation @BNFrcurly()
 @\@b[end] idn
@end[SyntaxDisplay]
where
@begin[SyntaxDisplay]
@TabClear()
operation @^@BNFequals() @^ creator
 @\@BNFor()@\ handler
 @\@BNFor()@\ routine
@end[SyntaxDisplay]
The initial @i(idn) names the guardian type or type generator
(as explained in Section @ref(sect-gtype)) and must agree with the final @i(idn).
The guardian header contains two @i(idn) lists.
The first, following @b[is], gives the names of the @i(creators),
which can be called to create and initialize new guardians
(the objects belonging to the guardian type).
The second, following the @b[handles],
gives the names of the @i(handlers) that can be called on these guardian
objects.
The names of all operations must be distinct.
Creators may not be named @i(equal), @i(similar), @i(copy),
or @i(get@us<>h) where @i(h) is the name of a handler.
See Section @ref(sect-gtype) for the interface type defined by a
guardian definition.  See Section @ref(sect-parms) for the meaning of
guardians having @i(parms) and @i(where) clauses.

The remaining portions of the guardian definition are discussed in the
subsections below.

@Section(The Guardian State)
@Label(sect-guardian-state)

The @i(state@us<>decls) of the guardian definition
@IndexSecondary(Primary="stable", Secondary="variable")
@IndexSecondary(Primary="stable", Secondary="state")
@IndexSecondary(Primary="volatile", Secondary="variable")
@IndexSecondary(Primary="guardian", Secondary="state")
@IndexSecondary(Primary="guardian", Secondary="stable state")
declare a number of variables (with optional initialization):
@begin[SyntaxDisplay]
@TabClear()
state@us<>decl @^@BNFequals() @BNFlbkt() @b[stable] @BNFrbkt() decl
 @\@BNFor() @BNFlbkt() @b[stable] @BNFrbkt() idn : type@us<>spec := expression
 @\@BNFor() @BNFlbkt() @b[stable] @BNFrbkt() decl , @BNFdots() := call
@end[SyntaxDisplay]
The scope of these declarations is the entire guardian definition.
The objects reachable from variables declared to be
@b[stable] survive crashes of the guardian,
while other objects do not.

For example, if the @i(state@us<>decls) were:
@begin[Display]
@b[stable] buffer:  @b[atomic@us<>array][@b[int]] := @b[atomic@us<>array][@b[int]]$new ( )
cache:  @b[array][@b[int]] := @b[array][@b[int]]$new ( )
@end[Display]
then the atomic@us<>array object denoted by
@i(buffer) would survive a guardian crash,
but the array object denoted by @i(cache) would not.
See Section @ref(sect-crash) for more details of crash recovery.
Volatile variables can be assigned wherever an assignment statement is legal.
However, stable variables may only be assigned by an initialization when
declared or in the body of a creator.
The initializations of both stable and volatile variables are executed within
an action, as described below.
However, the stable variables are not reinitialized upon crash recovery,
whereas volatile variables are reinitialized upon crash recovery.

Stable variables should denote resilient objects (see Section @ref(sect-resilience)),
because only resilient data objects (reachable from the stable variables)
are written to stable storage when a topaction commits.
(This can be ensured by having stable variables only
denote objects of an atomic type or objects protected by @b(mutex).)
Non-resilient objects stored in stable variables
are only written to stable storage once, when the guardian is created.
Furthermore, the stable variables should usually denote atomic objects,
because the stable variables are potentially shared by all the actions
in a guardian.

@Section(Creators)

A guardian definition must provide one or more creators.
@index(creator)
The names of these creators must be listed in the
guardian header (internal creators are not allowed);
each such name must correspond to a single creator definition appearing
in the body of the guardian definition.

A creator definition has the same form as a procedure definition,
except that creators cannot be parameterized,
and the reserved word @b[creator] is used in place of @b[proc]:
@begin[Display]
@TabClear()
idn = @b[creator] (@BNFlbkt() args @BNFrbkt()) @BNFlbkt() returns @BNFrbkt() @BNFlbkt() signals @BNFrbkt()
      routine@us<>body
      @b[end] idn
@end[Display]
The initial @i(idn) names the creator
and must agree with the final @i(idn).
The types of all arguments and all results (normal and exceptional) must
be transmissible.

A creator is an object of some creator type.  This type is derived from the
creator heading by removing the creator name, rewriting the formal argument
declarations with one @i(idn) per @i(decl), deleting the idns of all
formal arguments, deleting any @i(failure) or @i(unavailable) signals,
and finally, replacing @b[creator] by @b[creatortype].
The signals @i(failure)(@b[string]) and @i(unavailable)(@b[string])
are implicit in every creator type
(since they can arise from any creator call).
However, if these signals are raised explicitly by a creator,
they must be listed in the @i(signals) clause with @b[string] result types.

The semantics of a creator call are explained in
Section @ref(sect-creatorcall).
Typically, the body of a creator will initialize some stable and volatile
variables.
It can also return the name of the guardian being created using the
expression @b[self].
@index(self)
Since the creator (and the state initialization) runs as an action,
the creator terminates by committing or aborting.
@index(commit)@index(abort)@index(action)
@IndexSecondary(Primary="guardian", Secondary="creation")
If it aborts, the guardian is destroyed.
If it commits,
the guardian begins to accept handler calls, and runs the background code,
if any (see below).
If an ancestor of the creator aborts, the guardian is destroyed.
If the creator and all its ancestors commit,
the guardian becomes permanent, and will survive subsequent crashes.

@Section(Crash Recovery)
@Label(sect-crash)

Once a guardian becomes permanent,
@index(recovery)@IndexSecondary(Primary="guardian", Secondary="recovery")
@index(crash)@IndexSecondary(Primary="crash", Secondary="recovery")
@IndexSecondary(Primary="stable", Secondary="storage recovery")
@index(recover code)
it will be recreated automatically
after a crash with its stable variables initialized to the same state
they were in at the last topaction commit before the crash.
The volatile variables are then initialized (in declaration order)
by a topaction.  To aid in this reinitialization,
the guardian definition can provide a @i(recover section):
@begin[SyntaxDisplay]
@b[recover] body @b[end]
@end[SyntaxDisplay]
to be run, as part of this topaction, after the initializations
attached to the volatile variable declarations are performed.
The recover section commits when control reaches the end of the body,
or when a @b[return] statement is executed.
The recover section may abort by executing an @b[abort] @b[return] statement
or as a result of an unhandled exception.
The guardian crashes if the recover section aborts.

@Section(Background Tasks)

Tasks that must be performed periodically,
independent of handler calls, can be defined by a @i(background section):
@index(background)@IndexSecondary(Primary="guardian", Secondary="background code")
@begin[SyntaxDisplay]
@b[background] body @b[end]
@end[SyntaxDisplay]
The system creates a process to run this body as
soon as creation or recovery commits successfully.
The body of the background section does @i(not) run as an action;
typically it will perform a sequence of topactions.

If the background process finishes executing its @i(body)
(either by reaching the end of the block or by returning),
the process terminates,
but the guardian continues to execute incoming handler calls.

@Section(Handlers and Other Routines)
@Label(sect-handler-defs)

Typically, the principal purpose of a guardian is
to execute incoming handler calls.
@index(handler)@IndexSecondary(Primary="call", Secondary="remote")
@index(remote call)
A guardian accepts handler calls as soon as creation or recovery commits.

The guardian header lists the names of the externally available handlers.
Each handler listed must be defined by a handler definition.
Additional handler definitions may also be given,
but these handlers can be named only within the guardian to which they belong.

A handler definition has the same form as a procedure definition,
except that handlers cannot be parameterized,
and the reserved word @b[handler] is used in place of @b[proc]:
@begin[SyntaxDisplay]
@TabClear()
idn = @b[handler] (@BNFlbkt() args @BNFrbkt()) @BNFlbkt() returns @BNFrbkt() @BNFlbkt() signals @BNFrbkt()
      routine@us<>body
      @b[end] idn
@end[SyntaxDisplay]
The initial @i(idn) names the handler
and must agree with the final @i(idn).
The types of all arguments and all results (normal and exceptional) must
be transmissible.

A handler is an object of some handler type.  This type is derived from the
handler heading by removing the handler name, rewriting the formal argument
declarations with one @i(idn) per @i(decl), deleting the idns of all
formal arguments, deleting any @i(failure) or @i(unavailable) signals,
and finally, replacing @b[handler] by @b[handlertype].
The signals @i(failure)(@b[string]) and @i(unavailable)(@b[string])
are implicit in every handler type.
However, if these signals are raised explicitly by a handler,
they must be listed in the @i(signals) clause, with @b[string]
as their result type.

As explained in Section @ref(sect-handlercall),
a handler call runs as a subaction of its caller,
and arguments and results are passed by value.
A new process is created at the handler's guardian to run the handler
activation.
Since the handler activation is an action,
it terminates by committing or aborting.

A guardian definition may also contain procedure and iterator definitions.
@index(routine)@index(hidden routine)
These procedures and iterators may be called only within the guardian to which
they belong.

@Section(Guardian Lifetime and Destruction)

A guardian does not become permanent until its creating action
(the creator activation that initialized the guardian) commits to the top.
@IndexSecondary(Primary="guardian", Secondary="lifetime")
@IndexSecondary(Primary="guardian", Secondary="termination")
@IndexSecondary(Primary="guardian", Secondary="permanence")
@IndexSecondary(Primary="termination", Secondary="of a guardian")
If the creating action or any of its ancestors aborts,
the guardian will be destroyed.

Once a guardian becomes permanent,
it will survive node crashes (with high probability)
and thus may live forever.
However, a shorter lifetime may be appropriate for some guardians.
One guardian can never destroy another, but a guardian can destroy itself.
Destruction is accomplished by the @b[terminate] statement
(see Section @ref(sect-terminate)).
(A node's guardian manager may also destroy guardians.)

A short-lived guardian can be implemented by using background code of the form:
@begin[Display]
@b[background] @b[terminate] @b[end]
@end[Display]
The background code starts to run as soon as the creator returns.
The @b[terminate] is delayed, however,
until the creating action commits to the top,
so the creating action can make use of the new guardian before it is destroyed.
(If an ancestor of the creating action aborts,
the guardian will be destroyed automatically.)
@IndexSecondary(Primary="guardian", Secondary="temporary")

The following is an example of a handler for destroying a permanent guardian:
@begin[Display]
@TabClear()
finish = @b[handler] (...) @b[returns] (...) @b[signals] (not@us<>authorized)
      ...
      @b[terminate]
      @b[return](...)
      @b[end] finish
@end[Display]
Here, @i(finish) might check whether its caller is authorized
to make this request, and signal @i(not@us<>authorized) if not.
Otherwise it returns its vital state information to its caller
and destroys its guardian.

@Section(An Example)

To illustrate how most of the components of a guardian definition are used,
an example of a simple guardian is given in Figure @ref(fig-spooler).
An action can use a @i(spooler) guardian to store objects until after
@index(spooler guardian)
@IndexSecondary(Primary="guardian", Secondary="spooler example")
@IndexSecondary(Primary="example", Secondary="spooler guardian")
the action has committed to the top.  The spooler then passes
those objects to other guardians for consumption.
The spooler provides an operation for adding (object, consumer) pairs,
and for destroying the guardian.

@begin[Figure]
@Caption[Spooler Guardian]
@tag(fig-spooler)
@bar()
@begin[Display]
@TabClear()
spooler = @b[guardian] [t: @b[type]] @b[is] create @b[handles] enq, finish
                    @b[where] t @b[has] @b[transmit]

utype = @b[handlertype] (t)
entry = @b[struct][object: t,  consumer: utype]
queue = semiqueue[entry]

@b[stable] state: queue := queue$create()

@b[background]
      @b[while] @b[true] @b[do]
            @b[enter] @b[topaction]
                  e: entry := queue$deq(state)
                  e.consumer(e.object)
                     @b[except] @b[when] unavailable (*): @b[abort] @b[leave] @b[end]
                  @b[end] @b[except] @b[when] failure, unavailable (*): @b[end]
            @b[end]
      @b[end]

create = @b[creator] () @b[returns] (spooler[t])
      @b[return](@b[self])
      @b[end] create

enq = @b[handler] (item: t, user: utype)
      queue$enq(state, entry${object: item,  consumer: user})
      @b[end] enq

finish = @b[handler] ()
      @b[terminate]
      @b[end] finish

@b[end] spooler
@end[Display]
@bar()
@end[Figure]

The spooler guardian is parameterized by the type of object to be stored.
The @i(enq) handler takes an object of this type, and a handler
for sending the object to the consumer, and adds this information
to the stable state of the spooler.
This state is an object of the @i(semiqueue)
abstract data type@Foot{
See W. Weihl and B. Liskov, "Implementation of Resilient, Atomic Data Types",
in @i(ACM Transactions on Programming Languages and Systems),
volume 7, number 2, (April 1985), pages 244-269.
}.
Each entry in the semiqueue is a structure containing a stored object
and its corresponding consumer handler.
The background code of the guardian runs an infinite loop that
starts a topaction, removes an entry from the queue, and sends
the object using the associated handler.

Note that an @i(unavailable)
exception arising from this handler call is caught inside the topaction,
so that an explicit abort can be performed.  If the exception were
caught outside the topaction, it would cause the topaction to commit,
and the entry would be removed without being consumed.
Note also that @i(failure) is caught outside the topaction,
since if an @i(encode) were to fail, or if the guardian did not exist,
the background process might aimlessly loop forever,
because it would not be able to remove that entry.

A more extended example of a distributed system appears in the paper
Liskov, B. and Scheifler, R.,
"Guardians and Actions: Linguistic Support for Robust, Distributed Programs,"
@i(ACM Transactions on Programming Languages and Systems),
volume 5, number 3, (July 1983), pages 381-404.
