@Part(TypeDefinitions, Root "manual.mss")

@SpecialFont(F7="Math6")

@Comment{ Move stuff up (i.e., TP) a little, or down a little (i.e., TN) }
@Define[TP, Script +0.005in]
@Define[TP2,Script +0.010in]
@Define[TP3,Script +0.015in]
@Define[TP4,Script +0.020in]
@Define[TP5,Script +0.025in]
@Define[TP6,Script +0.030in]
@Define[TP7,Script +0.035in]
@Define[TP8,Script +0.040in]
@Define[TP9,Script +0.045in]
@Define[TP10,Script +0.050in]
@Define[TP11,Script +0.055in]
@Define[TP12,Script +0.060in]
@Define[TP13,Script +0.065in]
@Define[TP14,Script +0.070in]
@Define[TP15,Script +0.075in]
@Define[TM, Script -0.005in]
@Define[TM2,Script -0.010in]
@Define[TM3,Script -0.015in]
@Define[TM4,Script -0.020in]
@Define[TM5,Script -0.025in]
@Define[TM6,Script -0.030in]
@Define[TM7,Script -0.035in]
@Define[TM8,Script -0.040in]
@Define[TM9,Script -0.045in]
@Define[TM10,Script -0.050in]
@Define[TM11,Script -0.055in]
@Define[TM12,Script -0.060in]
@Comment{ TR's put a little (1..10) horizontal space to move things to the
	  right  a little if they overlap with the above or below line.   }
@Form[TR="@F7()"]
@Form[TR2="@TR()@TR()"]
@Form[TR3="@TR2()@TR()"]
@Form[TR4="@TR3()@TR()"]
@Form[TR5="@TR4()@TR()"]
@Form[TR6="@TR5()@TR()"]
@Form[TR7="@TR6()@TR()"]
@Form[TR8="@TR7()@TR()"]
@Form[TR9="@TR8()@TR()"]
@Form[TR10="@TR9()@TR()"]

@Textform{ZAdd = "+"}
@Comment{ This form moves the '*' down so that it is even with a '+'. }
@Form[ZMult="@TM6(*)"]

@Comment{Environment for type definitions}
@Define(TypeDef = Format, Need 7 lines, Group, BlankLines Hinge)

@Comment{Environment for Overview clause of definitions}
@Define(OverviewText = Text, spacing 1, leftmargin +6, indent 0, BlankLines Hinge)

@Comment{Indented Environment for operation definitions}
@Define(OpDef = Format, LeftMargin +0.4 inches, Group)

@Comment{Indented environment for requires, modifies, and effects clauses}
@Define[Itext = Text, LeftMargin +0.6inches, Indent -0.2inch, Group]


@Appendix(Built-in Types and Type Generators)
@Label(apdx-types)@Label(apdx-dtypes)
@IndexSecondary(Primary="type", Secondary="built-in")
@IndexSecondary(Primary="built-in", Secondary="type")

The following sections 
specify the built-in types and the types produced by the built-in type
generators of Argus.
For each type and for each instance of each type generator,
the objects of the type are characterized,
and all of the operations of the type are defined.
(An implementation may provide additional operations on the built in types,
as long as these are operations that could be implemented in terms of those
described in this section.)

All the built-in types (except for @b(any)) are transmissible.
All instances of the built-in type generators
(except for @b(proctype) and @b(itertype))
are transmissible if all their type parameters are transmissible.
Transmission of the built-in types preserves value equality,
except for objects of type @b[real].
However, in a homogeneous environment,
reals can be transmitted without approximations.
In a homogeneous environment, the only possible encode or decode
failures are exceeding the representation limits of an @b(image),
mutating the size of an @b(array) or @b(atomic@us()array) while it is being
encoded or decoded, and improper decoding of cyclic objects
(see Section @ref(sect-transmit-sharing)).

All operations are indivisible except at calls to subsidiary
operations (such as @b[int]$@i(similar) within @b(array)[@b[int]]$@i(similar)),
at yields, and while waiting for locks.
@IndexSecondary(Primary="operation", Secondary="indivisibility")

The specifications given below are informal and are adapted from the book
@i(Abstraction and Specification in Program Development)
(Liskov, B. and Guttag, J., MIT Press, 1986).
A specification starts out by giving a list of the operations and declarations
of any formal parameters for the type.
This is followed by an @b(overview),
which gives an introduction to the type and if necessary
defines a way of describing the type's objects and their values.
@index(overview)@index(effects)
Following this the individual operations are described.
For each operation there is a heading and a statement of the operation's
effects.
In the heading, the return values may be given names.
The @b(effects) section describes the normal and exceptional behavior of the
operation.
The effects given are abstract, that is they are described using the
vocabulary (or model) defined in the overview section.
For example, objects of type @b[int] are described using mathematical integers.
Thus arithmetic expressions and comparisons used in defining @b[int] operations
are to be computed over the domain of mathematical integers.

An operation that (abstractly) mutates one of its arguments lists the arguments
that it mutates in the clause following the word @b[modifies].
An operation is not allowed to mutate any objects, except for those listed
in the @b[modifies] clause.
@index(modifies)
(For the built-in mutable atomic type generators, modification only refers to
the sequential state;
it does not refer to changes in the locking information kept for each object.)
When an argument, say @i(a), is mutated, it is often necessary to describe
its state at the start of the call as well as its final state at the end of
the call.  We use the notation @i(a)@-(pre) for @i(a)'s state at the start
of the call and the notation @i(a)@-(post) for its state at the end of the
call.
@index(pre)@index(post)

Some operations of the built in type generators
are only defined if the type generator is passed
appropriate actual routine parameters (see Section @ref(sect-instantiations)).
For example, the @i(copy) operation of the @b[array] type generator,
is only defined if there is an actual parameter passed
(explicitly or implicitly)
for the type parameter's @i(copy) operation.
Thus @b[array][@b[int]]$copy is defined but @b[array][@b[any]]$copy is not
defined.
These requirements are stated in a @b[requires] clause that precedes the
description of the operation's effect.
The type of the expected routine is also described;
remember that the actual operation parameter can have fewer signals
(see Section @ref(sect-type-incl) and Section @ref(sect-instantiations)).

By convention,
the order in which exceptions are listed in the operation type
is the order in which the various conditions are checked.

Operations with the same semantics
(for example, @b[null]$@i(equal) and @b[null]$@i(similar))
or that can be described in the same way
(for example, @b[int]$@i(add) and @b[int]$@i(sub))
are grouped together to save space.

In defining the built-in types, we do not depend on users satisfying any
constraints beyond those that can be type-checked.
This decision leads to more complicated specifications.
For example,
the behavior of the @i(elements) iterator for arrays
is defined even when the loop modifies the array.

@Section(Null)
@label(null-type-def)

@index(null)@index(nil)
@begin(TypeDef)
@b[null] = @b(data type is) copy, equal, similar, @b[transmit]

@b(Overview)

@begin(OverviewText)
The type @b[null] has exactly one, immutable, atomic 
object, represented by the literal
@b(nil).
@b(Nil) is generally used as a place holder in type definitions
using oneofs or variants.
@end(OverviewText)

@b(Operations)

@begin(OpDef)
equal = @b(proc) (n1, n2: @b[null]) @b(returns) (@b[bool])
similar = @b(proc) (n1, n2: @b[null]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true].
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (n: @b[null]) @b(returns) (@b[null])
@b[transmit] = @b(proc) (n: @b[null]) @b(returns) (@b[null])
@begin(itext)
@b(effects)  Returns @b(nil).
@end(itext)
@end(OpDef)
@end(TypeDef)


@Section(Nodes)
@label(node-type-def)

@index(node)
@begin(TypeDef)
@b[node] = @b(data type is) here, copy, equal, similar, @b[transmit]

@b(Overview)

@begin(OverviewText)
Objects of type @b(node) are immutable and atomic, and stand for physical 
nodes.
Implementations should provide some mechanism for translating a node "address"
into a @b[node] object and vice versa.  (However, these do not have to be
operations of type @b[node].)
@end(OverviewText)

@b(Operations)

@begin(OpDef)
here = @b(proc) () @b(returns) (@b[node])
@begin(itext)
@b(effects)  Returns the @b[node] object for the caller's node.
@end(itext)
@end(OpDef)

@begin(OpDef)
equal = @b(proc) (n1, n2: @b[node]) @b(returns) (@b[bool])
similar = @b(proc) (n1, n2: @b[node]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns true if and only if n1 and n2 are the same node.
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (n: @b[node]) @b(returns) (@b[node])
@b[transmit] = @b(proc) (n: @b[node]) @b(returns) (@b[node])
@begin(itext)
@b(effects)  Returns @i(n).
@end(itext)
@end(OpDef)
@end(TypeDef)


@Section(Booleans)
@label(bool-type-def)
@index(bool)@index(true)@index(false)
@begin(TypeDef)
@b[bool] = @b(data type is) and, or, not, equal, similar, copy, @b[transmit]

@b(Overview)

@begin(OverviewText)
The two immutable, atomic 
objects of type @b[bool], with literals @b(true) and @b(false),
represent logical truth values.

The language also provides the operators @b(cand) and @b(cor)
for conditional evaluation of boolean expressions, see Section @ref(sect-cand).
@end(OverviewText)

@b(Operations)

@begin(OpDef)
and = @b(proc) (b1, b2: @b[bool]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if @i(b1) and @i(b2) are both @b[true];
returns @b[false] otherwise.
@end(itext)
@end(OpDef)

@begin(OpDef)
or = @b(proc) (b1, b2: @b[bool]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if either @i(b1) or @i(b2) is @b[true];
returns @b[false] otherwise.
@end(itext)
@end(OpDef)

@begin(OpDef)
not = @b(proc) (b: @b[bool]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[false] if @i(b) is @b[true];
returns @b[true] if @i(b) is @b[false].
@end(itext)
@end(OpDef)

@begin(OpDef)
equal = @b(proc) (b1, b2: @b[bool]) @b(returns) (@b[bool])
similar = @b(proc) (b1, b2: @b[bool]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if @i(b1) and @i(b2) are both @b[true]
or both @b[false]; otherwise returns @b[false].
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (b: @b[bool]) @b(returns) (@b[bool])
@b[transmit] = @b(proc) (b: @b[bool]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @i(b).
@end(itext)
@end(OpDef)
@end(TypeDef)


@Section(Integers)
@label(int-type-def)
@index(int)
@begin(TypeDef)
@b[int] = @b(data type is) add, sub, mul, minus, div, mod, power, abs, from@us()to@us()by, from@us()to, max, min,
				parse, unparse,	lt, le, ge, gt, equal, similar, copy, @b[transmit]

@b(Overview)

@begin(OverviewText)
Objects of type @b[int] are immutable and atomic, 
and are intended to model a subrange of
the mathematical integers.
The exact range is not part of the language definition and can vary somewhat
from implementation to implementation.
Each implementation is constrained to provide a closed interval
[@i(int@us()min), @i(int@us()max)],
with @i(int@us()min) < 0 and @i(int@us()max) @greaterequal()
@i(char@us()top) (the number of
characters @Dash() see section @ref(apdx-char)).
An @i(overflow) exception is signalled by an operation if the result
would lie outside this interval.
See Appendix @ref(apdx-syntax) for the syntax of integer literals.
@end(OverviewText)

@b(Operations)

@begin(OpDef)
add = @b(proc) (x, y: @b[int]) @b(returns) (@b[int]) @b(signals) (overflow)
sub = @b(proc) (x, y: @b[int]) @b(returns) (@b[int]) @b(signals) (overflow)
mul = @b(proc) (x, y: @b[int]) @b(returns) (@b[int]) @b(signals) (overflow)
@begin(itext)
@b(effects)  These are the standard integer addition, subtraction,
and multiplication operations.  They signal @i(overflow)
if the result would lie outside the represented interval.
@end(itext)
@end(OpDef)

@begin(OpDef)
minus = @b(proc) (x: @b[int]) @b(returns) (@b[int]) @b(signals) (overflow)
@begin(itext)
@b(effects)  Returns the negative of @i(x); signals @i(overflow) if the result
would lie outside the represented interval.
@end(itext)
@end(OpDef)

@begin(OpDef)
div = @b(proc) (x, y: @b[int]) @b(returns) (q: @b[int]) @b(signals) (zero@us()divide, overflow)
@begin(itext)
@b(effects)  Signals @i(zero@us()divide) if @i(y) = 0.  Otherwise returns
the integer quotient of dividing @i(x) by @i(y);
that is, @i(x) = @i(y)@ZMult()@i(q) + @i(r),
for some integer @i(r) such that 0 @LessEqual() @i(r) < |@i(y)|.
Signals @i(overflow) if @i(q) would lie outside the represented interval.
@end(itext)
@end(OpDef)

@begin(OpDef)
mod = @b(proc) (x, y: @b[int]) @b(returns) (r: @b[int]) @b(signals) (zero@us()divide, overflow)
@begin(itext)
@b(effects)  Signals @i(zero@us()divide) if @i(y) = 0.  Otherwise returns the
integer remainder of dividing @i(x) by @i(y);
that is, @i(r) is such that 0 @LessEqual() @i(r) < |@i(y)|,
for some integer @i(q): @i(x) = @i(y)@ZMult()@i(q) + @i(r).
Signals @i(overflow) if @i(r) would lie outside the represented interval.
@end(itext)
@end(OpDef)

@begin(OpDef)
power = @b(proc) (x, y: @b[int]) @b(returns) (@b[int]) @b(signals) (negative@us()exponent, overflow)
@begin(itext)
@b(effects)  Signals @i(negative@us()exponent) if @i(y) < 0.
Otherwise returns
@i(x@+(y)); signals @i(overflow) if the result would lie outside
the represented interval.  0@+(0)@ =@ 1 by definition.
@end(itext)
@end(OpDef)

@begin(OpDef)
abs = @b(proc) (x: @b[int]) @b(returns) (@b[int]) @b(signals) (overflow)
@begin(itext)
@b(effects)  Returns the absolute value of @i(x); signals @i(overflow) if the 
result would lie outside the represented interval.
@end(itext)
@end(OpDef)

@begin(OpDef)
from@us()to@us()by = @b(iter) (from, to, by: @b[int]) @b(yields) (@b[int])
@begin(itext)
@b(effects)  Yields the integers from @i(from) to @i(to), incrementing by
@i(by) each time, that is, yields @i(from), @i(from)@ZAdd()@i(by), ... ,
@i(from)@ZAdd()@i(n)@ZMult()@i(by), where
@i(n) is the largest positive integer
such that @w[@i(from)@ZAdd()@i(n)@ZMult()@i(by) @lessequal() @i(to).]
If @i(by) = 0, then yields @i(from) indefinitely.  Yields
nothing if @w[@i(from) > @i(to)] and @i(by) > 0, or 
if @i(from) < @i(to) and @i(by) < 0.  This iterator is divisible at yields.
@end(itext)
@end(OpDef)

@begin(OpDef)
from@us()to = @b(iter) (from, to: @b[int]) @b(yields) (@b[int])
@begin(itext)
@b(effects)  The effect is identical to @i(from@us()to@us()by)(@i(from), @i(to), 1).
@end(itext)
@end(OpDef)

@begin(OpDef)
max = @b(proc) (x, y: @b[int]) @b(returns) (@b[int])
@begin(itext)
@b(effects)  If @i(x) @greaterequal() @i(y), then returns @i(x),
otherwise returns @i(y).
@end(itext)
@end(OpDef)

@begin(OpDef)
min = @b(proc) (x, y: @b[int]) @b(returns) (@b[int])
@begin(itext)
@b(effects)  If @i(x) @lessequal() @i(y), then returns @i(x),
otherwise returns @i(y).
@end(itext)
@end(OpDef)

@begin(OpDef)
parse = @b(proc) (s: string) @b(returns) (@b[int]) @b(signals) (bad@us()format, overflow)
@begin(itext)
@b(effects)  @i(S) must be an integer literal (see Appendix @ref(apdx-syntax)),
with an optional leading plus or minus sign; if @i(s) is not of this
form, signals @i(bad@us()format).  Otherwise returns the
integer corresponding to @i(s); signals @i(overflow) if the result
would lie outside the represented interval.
@end(itext)
@end(OpDef)

@begin(OpDef)
unparse = @b(proc) (x: @b[int]) @b(returns) (string)
@begin(itext)
@b(effects)  Produces the string representing the integer value of @i(x)
in decimal notation,
preceded by a minus sign if @i(x) < 0.  Leading zeros are
suppressed, and there is no leading plus sign for positive integers.
@end(itext)
@end(OpDef)


@begin(OpDef)
lt = @b(proc) (x, y: @b[int]) @b(returns) (@b[bool])
gt = @b(proc) (x, y: @b[int]) @b(returns) (@b[bool])
le = @b(proc) (x, y: @b[int]) @b(returns) (@b[bool])
ge = @b(proc) (x, y: @b[int]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  These are the standard ordering relations.
@end(itext)
@end(OpDef)

@begin(OpDef)
equal = @b(proc) (x, y: @b[int]) @b(returns) (@b[bool])
similar = @b(proc) (x, y: @b[int]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if @i(x) and @i(y) are the same integer; returns
@b[false] otherwise.
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (x: @b[int]) @b(returns) (@b[int])
@begin(itext)
@b(effects)  Returns @i(x).
@end(itext)
@end(OpDef)

@begin(OpDef)
@b[transmit] = @b(proc) (x: @b[int]) @b(returns) (y: @b[int]) @b(signals)(failure(@b[string]))
@begin(itext)
@b(effects)  Returns @i(y) such that @i(x) = @i(y)
or signals @i(failure) if @i(x) cannot be represented in the implementation
on the receiving end.
@end(itext)
@end(OpDef)
@end(TypeDef)

@Section(Reals)
@label(real-type-def)
@label(apdx-reals)
@begin(TypeDef)
@b[real] = @b(data type is) add, sub, minus, mul, div, power, abs, max, min, exponent, mantissa, i2r, r2i,
				trunc, parse, unparse, lt, le, ge, gt, equal, similar, copy, @b[transmit]

@b(Overview)

@begin(OverviewText)

The type @b[real] models a subset of the mathematical numbers.
It is used for approximate or floating point arithmetic.
@index(real)
Reals are immutable and atomic, and are written as a @i(mantissa)
with an optional @i(exponent).
See Appendix @ref(apdx-syntax) for the format of real literals.

Each implementation represents a subset of the real numbers in:@*
@hsp(.5inch)D = {@MinusSign()real@us()max, @MinusSign()real@us()min} U {0} U {real@us()min, real@us()max}@*
where@*
@hsp(.5inch)0 < real@us()min < 1 < real@us()max@*
Numbers in D are approximated by the implementation
with a precision of @i(p) decimal digits such that:@*
@tabclear
@tabset(.5inch, 2inches)
@\@ForAll()r @ElementOf() D@\Approx(r) @ElementOf() Real@*
@\@ForAll()r @ElementOf() Real@\Approx(r) = r@*
@\@ForAll()r @ElementOf() D @MinusSign() {0}@\| (Approx(r) @MinusSign() r)/r| < 10@+(1@MinusSign()p)@*
@\@ForAll()r,s @ElementOf() D@\r @lessequal() s  @Implies()  Approx(r) @lessequal() Approx(s)@*
@\@ForAll()r @ElementOf() D@\Approx(@MinusSign()r) = @MinusSign()Approx(r)@*
We define
@i(Max@us()width) and @i(Exp@us()width) to be the smallest integers such that
every nonzero element of @b[real] can be represented in "standard" form
(exactly one digit, not zero, before the decimal point)
with no more than @i(Max@us()width) digits of mantissa
and no more than @i(Exp@us()width) digits of exponent.

Real operations signal an exception if the result of a computation lies outside
of D; @i(overflow) occurs if the magnitude exceeds @i(real@us()max), and
@i(underflow) occurs if the magnitude is less than @i(real@us()min).
@end(OverviewText)


@b(Operations)

@begin(OpDef)
add = @b(proc) (x, y: @b[real]) @b(returns) (@b[real]) @b(signals) (overflow, underflow)
@begin(itext)
@b(effects)  Computes the sum @i(z) of @i(x) and @i(y); signals @i(overflow)
or @i(underflow)
if @i(z) is outside of D, as explained earlier.  Otherwise
returns an approximation such that:@*
@hsp(.5inch)   (x,y @greaterequal() 0 @or() x,y @lessequal() 0)  @Implies()  add(x, y) = Approx(x @ZAdd() y)@*
@hsp(.5inch)   add(x, y) = (1 @ZAdd() @g(e))(x @ZAdd() y)@hsp(.5inch)|@g(e)| < 10@+(1@MinusSign()p)@*
@hsp(.5inch)   add(x, 0) = x@*
@hsp(.5inch)   add(x, y) = add(y, x)@*
@hsp(.5inch)   x @lessequal() x'  @Implies()  add(x, y) @lessequal() add(x', y)
@end(itext)
@end(OpDef)

@begin(OpDef)
sub = @b(proc) (x, y: @b[real]) @b(returns) (@b[real]) @b(signals) (overflow, underflow)
@begin(itext)
@b(effects)  Computes @i(x) @MinusSign() @i(y); the result is identical
to @i(add)(@i(x), @MinusSign()@i(y)).
@end(itext)
@end(OpDef)

@begin(OpDef)
minus = @b(proc) (x: @b[real]) @b(returns) (@b[real])
@begin(itext)
@b(effects)  Returns  @MinusSign()@i(x).
@end(itext)
@end(OpDef)

@begin(OpDef)
mul = @b(proc) (x, y: @b[real]) @b(returns) (@b[real]) @b(signals) (overflow, underflow)
@begin(itext)
@b(effects)  Returns @i(approx)(@i(x)@ZMult()@i(y));
signals @i(overflow) or @i(underflow) if
@i(x)@ZMult()@i(y) is outside of D.
@end(itext)
@end(OpDef)

@begin(OpDef)
div = @b(proc) (x, y: @b[real]) @b(returns) (@b[real]) @b(signals) (zero@us()divide, overflow, underflow)
@begin(itext)
@b(effects)  If @i(y) = 0, signals @i(zero@us()divide).  Otherwise returns
@i(approx)(@i(x)/@i(y)); signals @i(overflow) or @i(underflow)
if @i(x)/@i(y) is outside of D.
@end(itext)
@end(OpDef)

@begin(OpDef)
power = @b(proc) (x, y: @b[real]) @b(returns) (@b[real])
				@b(signals) (zero@us()divide, complex@us()result, overflow, underflow)
@begin(itext)
@b(effects)  If @i(x) = 0 and @i(y) < 0, signals @i(zero@us()divide).
If @i(x) < 0 and
@i(y) is nonintegral, signals @i(complex@us()result).  Otherwise returns
an
approximation to @i(x@+(y)), good to @i(p) significant digits;
signals @i(overflow) or @i(underflow) if @i(x@+(y)) is outside of D.
@end(itext)
@end(OpDef)

@begin(OpDef)
abs = @b(proc) (x: @b[real]) @b(returns) (@b[real])
@begin(itext)
@b(effects)  Returns the absolute value of @i(x).
@end(itext)
@end(OpDef)

@begin(OpDef)
max = @b(proc) (x, y: @b[real]) @b(returns) (@b[real])
@begin(itext)
@b(effects)  If @i(x) @greaterequal() @i(y), then returns @i(x),
otherwise returns @i(y).
@end(itext)
@end(OpDef)

@begin(OpDef)
min = @b(proc) (x, y: @b[real]) @b(returns) (@b[real])
@begin(itext)
@b(effects)  If @i(x) @lessequal() @i(y), then returns @i(x),
otherwise returns @i(y).
@end(itext)
@end(OpDef)

@begin(OpDef)
exponent = @b(proc) (x: @b[real]) @b(returns) (@b[int]) @b(signals) (undefined)
@begin(itext)
@b(effects)  If @i(x) = 0, signals @i(undefined).  Otherwise returns the
exponent that would be used in representing @i(x) as
a literal in standard form, that is, returns@*
@hsp(.5inch)max ({i | abs(x) @greaterequal() 10@+(i)})
@end(itext)
@end(OpDef)

@begin(OpDef)
mantissa = @b(proc) (x: @b[real]) @b(returns) (@b[real])
@begin(itext)
@b(effects)  Returns the mantissa of @i(x) when represented in standard
form, that is,
returns @i(approx)(@i(x)/10@+(@i(e))), where @i(e) = @i(exponent)(@i(x)).
If @i(x) = 0.0, returns 0.0.
@end(itext)
@end(OpDef)

@begin(OpDef)
i2r = @b(proc) (i: @b[int]) @b(returns) (@b[real]) @b(signals) (overflow)
@begin(itext)
@b(effects)  Returns @i(approx)(@i(i)); signals overflow if @i(i) is not in D.
@end(itext)
@end(OpDef)

@begin(OpDef)
r2i = @b(proc) (x: @b[real]) @b(returns) (@b[int]) @b(signals) (overflow)
@begin(itext)
@b(effects)  Rounds @i(x) to the nearest integer and toward zero in case of a tie.
Signals @i(overflow) if the result lies outside the represented range of integers.
@end(itext)
@end(OpDef)


@begin(OpDef)
trunc = @b(proc) (x: @b[real]) @b(returns) (@b[int]) @b(signals) (overflow)
@begin(itext)
@b(effects)  Truncates @i(x) toward zero; signals @i(overflow) if the result would
be outside the represented range of integers.
@end(itext)
@end(OpDef)

@begin(OpDef)
parse = @b(proc) (s: string) @b(returns) (@b[real]) @b(signals) (bad@us()format, overflow, underflow)
@begin(itext)
@b(effects)  Returns @i(approx)(@i(z)), where @i(z) is the value
represented by the string @i(s) (see Appendix @ref(apdx-syntax)).
@i(S) must represent a real or integer
literal with an optional leading plus or minus sign; otherwise
signals @i(bad@us()format).
Signals @i(underflow) or @i(overflow) if @i(z) is not in D.
@end(itext)
@end(OpDef)

@begin(OpDef)
unparse = @b(proc) (x: @b[real]) @b(returns) (string)
@begin(itext)
@b(effects)  Returns a real literal such that @i(parse)(@i(unparse)(@i(x))) = @i(x).
The general form of the literal is:@*
@hsp(.5inch)@BNFlbkt() @MinusSign() @BNFrbkt() @i(i@us()field)@b(.)@i(f@us()field) @BNFlbkt() @i(e) @plusminus() @i(x@us()field) @BNFrbkt()@*
Leading zeros in @i(i@us()field) and trailing zeros in @i(f@us()field)
are suppressed.
If @i(x) is integral and within the range of represented integers,
then @i(f@us()field) and the exponent are not present.
If @i(x) can be represented by a mantissa of no more than @i(Max@us()width) digits
and no exponent (that is, if @MinusSign()1 @lessequal() @i(exponent)(@i(arg)1)
< @i(Max@us()width)),
then the exponent is not present.
Otherwise
the literal is in standard form, with @i(Exp@us()width) digits of exponent.
@end(itext)
@end(OpDef)

@begin(OpDef)
lt = @b(proc) (x, y: @b[real]) @b(returns) (@b[bool])
le = @b(proc) (x, y: @b[real]) @b(returns) (@b[bool])
ge = @b(proc) (x, y: @b[real]) @b(returns) (@b[bool])
gt = @b(proc) (x, y: @b[real]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  These are the standard ordering relations.
@end(itext)
@end(OpDef)

@begin(OpDef)
equal = @b(proc) (x, y: @b[real]) @b(returns) (@b[bool])
similar = @b(proc) (x, y: @b[real]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if @i(x) and @i(y) are the same number;
returns @b[false] otherwise.
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (x: @b[real]) @b(returns) (@b[real])
@begin(itext)
@b(effects)  Returns @i(x).
@end(itext)
@end(OpDef)

@begin(OpDef)
@b[transmit] = @b(proc) (x: @b[real]) @b(returns) (@b[real]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(effects)  Returns @i(approx')(@i(x)) where @i(approx') is the approximation
function for the receiving implementation of Argus
or signals @i(failure) if this cannot be represented on the receiving end.
@end(itext)
@end(OpDef)
@end(TypeDef)


@Section(Characters)
@label(char-type-def)
@label(apdx-char)
@index(char)
@begin(TypeDef)
@b[char] = @b(data type is) i2c, c2i, lt, le, ge, gt, equal, similar, copy, @b(transmit)

@b(Overview)

@begin(OverviewText)
Type @b[char] provides the alphabet for text manipulation.
Characters are immutable and atomic, and form an ordered set.
Every implementation must provide at least 128, but no more than 512,
characters; the first 128 characters are the ASCII
characters in their standard order.

Operations @i(i2c) and @i(c2i) convert between @b[int]s and @b[char]s
(using the ASCII coding for the first 128 characters).
The smallest character corresponds to zero, and characters are numbered
sequentially
up to @i(char@us()top), the integer corresponding to
the largest character.
This numbering determines the ordering of the characters.

Printing ASCII characters (octal 40 through octal 176),
other than single quote or backslash, can be written as that
character enclosed in single quotes.
See Appendix @ref(apdx-syntax) for the syntax of character literals
and tables of character escape sequences.
@end(OverviewText)


@b(Operations)

@begin(OpDef)
i2c = @b(proc) (x: @b[int]) @b(returns) (@b[char]) @b(signals) (illegal@us()char)
@begin(itext)
@b(effects)  Returns the character corresponding to @i(x); signals
@i(illegal@us()char) if @i(x) is not in the range [0, @i(char@us()top)].
@end(itext)
@end(OpDef)

@begin(OpDef)
c2i = @b(proc) (c: @b[char]) @b(returns) (@b[int])
@begin(itext)
@b(effects)  Returns the integer corresponding to @i(c)
(using the ASCII coding if @i(c) is an ASCII character).
@end(itext)
@end(OpDef)

@begin(OpDef)
lt = @b(proc) (c1, c2: @b[char]) @b(returns) (@b[bool])
le = @b(proc) (c1, c2: @b[char]) @b(returns) (@b[bool])
ge = @b(proc) (c1, c2: @b[char]) @b(returns) (@b[bool])
gt = @b(proc) (c1, c2: @b[char]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  These are the standard ordering relations, where the
order is consistent with the numbering of characters.
That is, @i(c1) < @i(c2) just when @i(c2i)(@i(c1)) < @i(c2i)(@i(c2)).
@end(itext)
@end(OpDef)

@begin(OpDef)
equal = @b(proc) (c1, c2: @b[char]) @b(returns) (@b[bool])
similar = @b(proc) (c1, c2: @b[char]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if @i(c1) and @i(c2) are the same character,
i.e., returns (@i(c2i)(@i(c1)) = @i(c2i)(@i(c2))).
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (c1: @b[char]) @b(returns) (@b[char])
@begin(itext)
@b(effects)  Returns @i(c1).
@end(itext)
@end(OpDef)

@begin(OpDef)
@b[transmit] = @b(proc) (c1: @b[char]) @b(returns) (@b[char]) @b(signals) (failure(@b[string])
@begin(itext)
@b(effects)  Returns @i(c1).
Signals @i(failure) only if @i(c1) is not representable
by the implementation on the receiving end.
@end(itext)
@end(OpDef)
@end(TypeDef)


@Section(Strings)
@label(string-type-def)
@index(string)
@begin(TypeDef)
@b[string] = @b(data type is) c2s, concat, append, substr, rest, size, empty, fetch, chars, indexs, indexc,
				s2ac, ac2s, s2sc, sc2s, lt, le, ge, gt,	equal, similar, copy, @b[transmit]

@b(Overview)

@begin(OverviewText)
Type @b[string] is used for representing text.
A string is an immutable and atomic tuple of zero or more characters.
The characters of a string are indexed sequentially starting from one.
Strings are lexicographically ordered based on the ordering for characters.
@index(lexicographic order)

A string literal is written as a sequence
of zero or more character representations enclosed in double quotes.
See Appendix @ref(apdx-syntax) for a description of the character escape
sequences that can be used within string literals.
No string can have a size greater than @i(int@us()max); however, an 
implementation may restrict string lengths to a value less than 
@i(int@us()max).
If the result of a string operation would be a string containing more than
the maximum number of
characters, the operation signals @i(limits).
@end(OverviewText)

@b(Operations)

@begin(OpDef)
c2s = @b(proc) (c: @b[char]) @b(returns) (@b[string])
@begin(itext)
@b(effects)  Returns a string containing @i(c) as its only character.
@end(itext)
@end(OpDef)

@begin(OpDef)
concat = @b(proc) (s1, s2: @b[string]) @b(returns) (r: @b[string]) @b(signals) (limits)
@begin(itext)
@b(effects)  Returns the concatenation of @i(s1) and @i(s2).
That is, @i(r)[@i(i)]=@i(s1)[@i(i)] for @i(i) an index of @i(s1)
and @i(r)[@i[size(s1)]+@i(i)]=@i(s2)[@i(i)] for @i(i) an index of @i(s2).
Signals @i(limits) if @i(r) would be too large for the implementation.
@end(itext)
@end(OpDef)

@begin(OpDef)
append = @b(proc) (s: @b[string], c: @b[char]) @b(returns) (r: @b[string]) @b(signals) (limits)
@begin(itext)
@b(effects)  Returns a new string having the characters of @i(s) in order
followed by @i(c).  That is, @i(r)[@i(size)(@i(s))+1] = @i(c).
Signals @i(limits) if the new string would be too large for the implementation.
@end(itext)
@end(OpDef)

@begin(OpDef)
substr = @b(proc) (s: @b[string], at: @b[int], cnt: @b[int]) @b(returns) (@b[string]) @b(signals) (bounds, negative@us()size)
@begin(itext)
@b(effects)  If @i(cnt) < 0, signals @i(negative@us()size).  If @i(at) < 1 or
@i(at) > @i(size)(@i(s))@ZAdd()1, signals @i(bounds).  Otherwise returns
a string having the characters @i(s)[@i(at)], @i(s)[@i(at)@ZAdd()1], ...
in that order;
the new string contains
@i(min)(@i(cnt), @i(size)@MinusSign()@i(at)@ZAdd()1) characters.
For example,@*
@hsp(.5inch)substr ("abcdef", 2, 3) = "bcd"@*
@hsp(.5inch)substr ("abcdef", 2, 7) = "bcdef"@*
@hsp(.5inch)substr ("abcdef", 7, 1) = ""@*
Note that if @i(min)(@i(cnt), @i(size)@MinusSign()@i(at)@ZAdd()1) = 0,
@i(substr) returns the empty string.
@end(itext)
@end(OpDef)

@begin(OpDef)
rest = @b(proc) (s: @b[string], i: @b[int]) @b(returns) (r: @b[string]) @b(signals) (bounds)
@begin(itext)
@b(effects)  Signals @i(bounds) if @i(i) < 0 or @i(i) > @i(size)(@i(s)) @ZAdd() @i(1); otherwise
returns a string whose first character is @i(s)[@i(i)],
whose second is @i(s)[@i(i@ZAdd()1)], ..., and whose @i(size)(@i(r))th character
is @i(s)[@i(size)(@i(s))].
Note that if @i(i) = @i(size)(@i(s))@ZAdd()1, @i(rest) returns the empty string.
@end(itext)
@end(OpDef)

@begin(OpDef)
size = @b(proc) (s: @b[string]) @b(returns) (@b[int])
@begin(itext)
@b(effects)  Returns the number of characters in @i(s).
@end(itext)
@end(OpDef)

@begin(OpDef)
empty = @b(proc) (s: @b[string]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if @i(s) is empty (contains no characters);
otherwise returns @b[false].
@end(itext)
@end(OpDef)

@begin(OpDef)
fetch = @b(proc) (s: @b[string], i: @b[int]) @b(returns) (@b[char]) @b(signals) (bounds)
@begin(itext)
@b(effects)  Signals @i(bounds) if @i(i) < 0 or @i(i) > @i(size)(@i(s));
otherwise returns the @i(i)th character of @i(s).
@end(itext)
@end(OpDef)

@begin(OpDef)
chars = @b(iter) (s: @b[string]) @b[yields] (@b[char])
@begin(itext)
@b(effects)  Yields, in order, each character of @i(s) (i.e., s[1], s[2], ...).
@end(itext)
@end(OpDef)

@begin(OpDef)
indexs = @b(proc) (s1, s2: @b[string]) @b(returns) (@b[int])
@begin(itext)
@b(effects)  If @i(s1) occurs as a substring in @i(s2), returns the
least index at which @i(s1) occurs.
Returns 0 if @i(s1) does
not occur in @i(s2),
and 1 if @i(s1) is the empty string.
For example,@*
@hsp(.5inch)indexs("abc", "abcbc") = 1@*
@hsp(.5inch)indexs("bc", "abcbc") = 2@*
@hsp(.5inch)indexs("", "abcde") = 1@*
@hsp(.5inch)indexs("bcb", "abcde") = 0
@end(itext)
@end(OpDef)

@begin(OpDef)
indexc = @b(proc) (c: @b[char], s: @b[string]) @b(returns) (@b[int])
@begin(itext)
@b(effects)  If @i(c) occurs in @i(s), returns the least index at which @i(c) occurs;
returns 0 if @i(c) does not occur in @i(s).
@end(itext)
@end(OpDef)

@begin(OpDef)
s2ac = @b(proc) (s: @b[string]) @b(returns) (@b[array][@b[char]])
@begin(itext)
@b(effects)  Stores the characters of @i(s)
as elements of a new array of characters, @i(a).
The low bound of the array is 1,
the size is @i(size)(@i(s)), and
the @i(i)th element of the array is the @i(i)th character
of @i(s), for 1 @lessequal() @i(i) @lessequal() @i(size)(@i(s)).
@end(itext)
@end(OpDef)

@begin(OpDef)
ac2s = @b(proc) (a: @b[array][@b[char]]) @b(returns) (@b[string])
@begin(itext)
@b(effects)  This is the inverse of @i(s2ac).
The result is a string with characters in the same order as in @i(a).
That is,
the @i(i)th character of the string is the
(@i(i)@ZAdd()@b[array][@b[char]]$@i(low)(@i(a))@MinusSign()1)th element of @i(a).
@end(itext)
@end(OpDef)

@begin(OpDef)
s2sc = @b(proc) (s: @b[string]) @b(returns) (@b[sequence][@b[char]])
@begin(itext)
@b(effects)  Transforms a string into a sequence of characters.
The size of the sequence is @i(size)(@i(s)).
The @i(i)th element of the sequence is the @i(i)th character
of @i(s), for 1 @lessequal() @i(i) @lessequal() @i(size)(@i(s)).
@end(itext)
@end(OpDef)

@begin(OpDef)
sc2s = @b(proc) (s: @b[sequence][@b[char]]) @b(returns) (@b[string])
@begin(itext)
@b(effects)  This is the inverse of @i(s2sc).
The result is a string with characters in the same order as in @i(s).
That is,
the @i(i)th character of the string is the
@i(i)th element of @i(s).
@end(itext)
@end(OpDef)


@begin(OpDef)
lt = @b(proc) (s1, s2: @b[string]) @b(returns) (@b[bool])
le = @b(proc) (s1, s2: @b[string]) @b(returns) (@b[bool])
ge = @b(proc) (s1, s2: @b[string]) @b(returns) (@b[bool])
gt = @b(proc) (s1, s2: @b[string]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  These are the usual lexicographic ordering relations
on strings, based on the ordering of characters.  For example,@*
@hsp(.5inch)"abc" < "aca"@*
@hsp(.5inch)"abc" < "abca"
@end(itext)
@end(OpDef)

@begin(OpDef)
equal = @b(proc) (s1, s2: @b[string]) @b(returns) (@b[bool])
similar = @b(proc) (s1, s2: @b[string]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if @i(s1) and @i(s2) are the same string;
otherwise returns @b[false].
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (s1: @b[string]) @b(returns) (@b[string])
@begin(itext)
@b(effects)  Returns @i(s1).
@end(itext)
@end(OpDef)

@begin(OpDef)
@b[transmit] = @b(proc) (s1: @b[string]) @b(returns) (@b[string]) @b[signals] (failure(@b[string]))
@begin(itext)
@b(effects)  Returns @i(s1).
Signals @i(failure) only if @i(s1) is not representable on the receiving end.
@end(itext)
@end(OpDef)
@end(TypeDef)

@NewPage()

@Section(Sequences)
@label(sequence-type-def)
@index(sequence)
@begin(TypeDef)
@b[sequence] = @b(data type) [t: @b[type]] @b(is) new, e2s, fill, fill@us()copy, replace, addh, addl, remh, reml, concat,
				subseq, size, empty, fetch, bottom, top, elements, indexes, a2s, s2a,
				equal, similar, copy, @b[transmit]

@b(Overview)

@begin(OverviewText)
Sequences represent immutable tuples of objects of type @i(t).
The elements of the sequence can be indexed sequentially from 1 up to the
size of the sequence.  
Although a sequence is immutable, the elements of the sequence can be mutable
objects.   The state of such mutable elements may change; thus, a sequence
object is atomic only if its elements are also atomic.

Sequences can be created by calling sequence operations and by means of
the sequence constructor, see Section @ref(sect-sequence-constructor).

Any operation call that attempts to access a sequence with an index that
is not within the defined range terminates with the @i(bounds) exception.
The size of a sequence can be no larger than the
largest positive @b(int) (@i(int@us()max)), but an implementation may restrict
sequences to a smaller upper bound.  An attempt to construct a sequence which
is too large results in a @i(limits) exception.
@end(OverviewText)

@b(Operations)

@begin(OpDef)
new = @b(proc) ( ) @b(returns) (@b[sequence][t])
@begin(itext)
@b(effects)  Returns the empty sequence.
@end(itext)
@end(OpDef)

@begin(OpDef)
e2s = @b(proc) (elem: t) @b(returns) (@b[sequence][t])
@begin(itext)
@b(effects)  Returns a one-element sequence having @i(elem)
as its only element.
@end(itext)
@end(OpDef)

@begin(OpDef)
fill = @b(proc) (cnt: @b[int], elem: t) @b(returns) (@b[sequence][t]) @b(signals) (negative@us()size, limits)
@begin(itext)
@b(effects)  If @i(cnt) < 0, signals @i(negative@us()size).  If @i(cnt) is
larger than the maximum sequence size supported by the implementation, signals
@i(limits).
Otherwise
returns a sequence having @i(cnt) elements each of which is @i(elem).
@end(itext)
@end(OpDef)


@begin(OpDef)
fill@us()copy = @b(proc) (cnt: @b[int], elem: t) @b(returns) (@b[sequence][t])
@hsp(1.2inches)@b(signals) (negative@us()size, limits, failure(@b[string]))
@begin(itext)
@b(requires) t has copy: @b(proctype) (t) @b(returns) (t) @b(signals) (failure(@b[string])
@end(itext)
@begin(itext)
@b(effects)  If @i(cnt) < 0, signals @i(negative@us()size).
If @i(cnt) is bigger than the maximum size of sequences that the implementation
supports, signals @i(limits).
Otherwise returns a new sequence having @i(cnt) elements each of
which is a copy of @i(elem), as made by @i(t)$@i(copy).
Note that @i(t)$@i(copy) is called @i(cnt) times.
Any @i(failure) signal raised by @i(t)$@i(copy) is immediately resignalled.
This operation does not originate any @i(failure) signals by itself.
@end(itext)
@end(OpDef)

@begin(OpDef)
replace = @b(proc) (s: @b[sequence][t], i: @b[int], elem: t) @b(returns) (@b[sequence][t]) @b(signals) (bounds)
@begin(itext)
@b(effects)  If @i(i) < 1 or @i(i) > @i(high)(@i(s)), signals @i(bounds).  Otherwise
returns a sequence with the same elements as @i(s),
except that @i(elem) is in the @i(i)th position.  For example,@*
@hsp(.5inch)replace(@b[sequence][@b[int]]$[2,5], 1, 6) = @b[sequence][@b[int]]$[6, 5]
@end(itext)
@end(OpDef)

@begin(OpDef)
addh = @b(proc) (s: @b[sequence][t], elem: t) @b(returns) (r: @b[sequence][t]) @b(signals) (limits)
@begin(itext)
@b(effects)  Returns a sequence with the same elements
as @i(s) followed by one additional element, @i(elem).
That is, @i(r)[i]=@i(s)[i] for @i(i) an index of @i(s),
and @i(r)[@i[size(s)]+1]=@i(elem).
If the resulting sequence would be larger than the implementation supports,
signals @i(limits).
@end(itext)
@end(OpDef)

@begin(OpDef)
addl = @b(proc) (s: @b[sequence][t], elem: t) @b(returns) (r: @b[sequence][t]) @b(signals) (limits)
@begin(itext)
@b(effects)  Returns a sequence having @i(elem) as the first element
followed by the elements of @i(s) in order.
That is, @i(r)[1]=@i(elem) and @i(r)[@i(i)]=@i(s)[@i(i)@MinusSign()1]
for @i(i) = 2, ..., @i[size(r)].
If the resulting sequence would be larger than the implementation supports,
signals @i(limits).
@end(itext)
@end(OpDef)

@begin(OpDef)
remh = @b(proc) (s: @b[sequence][t]) @b(returns) (r: @b[sequence][t]) @b(signals) (bounds)
@begin(itext)
@b(effects)  If @i(s) is empty, signals @i(bounds).  Otherwise returns
a sequence having all elements of @i(s) in order, except the last one.
That is, @i[size(r)]=@i[size(s)]@MinusSign()1 and
@i(r)[@i(i)]=@i(s)[@i(i)] for @i(i) = 1, ..., @i[size(s)]@MinusSign()1.
@end(itext)
@end(OpDef)

@begin(OpDef)
reml = @b(proc) (s: @b[sequence][t]) @b(returns) (r: @b[sequence][t]) @b(signals) (bounds)
@begin(itext)
@b(effects)  If @i(s) is empty, signals @i(bounds).  Otherwise returns
a sequence containing all elements of @i(s) in order, except the first one.
That is, @i(r)[@i(i)]=@i(s)[@i(i)+1] for @i(i) = 1, ..., @i(size)(@i(s))@MinusSign()1.
@end(itext)
@end(OpDef)

@begin(OpDef)
concat = @b(proc) (s1, s2: @b[sequence][t]) @b(returns) (r: @b[sequence][t]) @b(signals) (limits)
@begin(itext)
@b(effects)  Returns the concatenation of @i(s1) and @i(s2);
which is a sequence having the elements of @i(s1)
followed by the elements of @i(s2).  
That is, @i(r)[@i(i)]=@i(s1)[@i(i)] for @i(i) an index of @i(s1)
and @i(r)[@i[size(s1)]+@i(i)]=@i(s2)[@i(i)] for @i(i) an index of @i(s2).
Signals @i(limits) 
if the resulting sequence would be larger than the implementation supports.
@end(itext)
@end(OpDef)

@begin(OpDef)
subseq = @b(proc) (s: @b[sequence][t], at, cnt: @b[int]) @b(returns) (@b[sequence][t])
@hsp(1.2inch)@b(signals) (bounds, negative@us()size)
@begin(itext)
@b(effects)  If @i(cnt) < 0, signals @i(negative@us()size).
If @i(at) < 1 or @i(at) > @i(size)(@i(s))@ZAdd()1, signals @i(bounds).
Otherwise returns
a sequence having the elements @i(s)[@i(at)], @i(s)[@i(at)@ZAdd()1], ...
in that order;
the new sequence contains
@i(min)(@i(cnt), @i(size)@MinusSign()@i(at)@ZAdd()1) elements.
Note that if @i(min)(@i(cnt), @i(size)@MinusSign()@i(at)@ZAdd()1) = 0,
@i(subseq) returns the empty sequence.
@end(itext)
@end(OpDef)

@begin(OpDef)
size = @b(proc) (s: @b[sequence][t]) @b(returns) (@b[int])
@begin(itext)
@b(effects)  Returns the number of elements in @i(s).
@end(itext)
@end(OpDef)

@begin(OpDef)
empty = @b(proc) (s: @b[sequence][t]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if @i(s) contains no elements; otherwise returns @b[false].
@end(itext)
@end(OpDef)
	
@begin(OpDef)
fetch = @b(proc) (s: @b[sequence][t], i: @b[int]) @b(returns) (t) @b(signals) (bounds)
@begin(itext)
@b(effects)  If @i(i) < 1 or @i(i) > @i(size)(@i(s)),
signals @i(bounds).  Otherwise
returns the @i(i)th element of @i(s).
@end(itext)
@end(OpDef)

@begin(OpDef)
bottom = @b(proc) (s: @b[sequence][t]) @b(returns) (t) @b(signals) (bounds)
@begin(itext)
@b(effects)  If @i(s) is empty, signals @i(bounds).
Otherwise returns @i(s)[1].
@end(itext)
@end(OpDef)

@begin(OpDef)
top = @b(proc) (s: @b[sequence][t]) @b(returns) (t) @b(signals) (bounds)
@begin(itext)
@b(effects)  If @i(s) is empty, signals @i(bounds).  Otherwise returns @i(s)[@i(size)(@i(s))].
@end(itext)
@end(OpDef)

@begin(OpDef)
elements = @b(iter) (s: @b[sequence][t]) @b(yields) (t)
@begin(itext)
@b(effects)  Yields the elements of @i(s) in order (i.e., @i(s)[1], @i(s[2]), ...).
@end(itext)
@end(OpDef)

@begin(OpDef)
indexes = @b(iter) (s: @b[sequence][t]) @b(yields) (@b[int])
@begin(itext)
@b(effects)  Yields the indexes of @i(s) from 1 to @i(size)(@i(s)).
@end(itext)
@end(OpDef)

@begin(OpDef)
a2s = @b(proc) (a: @b[array][t]) @b(returns) (@b[sequence][t])
@begin(itext)
@b(effects)  Returns a sequence having the elements of @i(a) in the same order as in @i(a).
@end(itext)
@end(OpDef)

@begin(OpDef)
s2a = @b(proc) (s: @b[sequence][t]) @b(returns) (@b[array][t])
@begin(itext)
@b(effects)  Returns a new array with low bound 1 and having
the elements of @i(s) in the same order as in @i(s).
@end(itext)
@end(OpDef)

@begin(OpDef)
equal = @b(proc) (s1, s2: @b[sequence][t]) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires) @i(t) has equal: @b(proctype) (t, t) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns @b[true] if @i(s1) and @i(s2) have equal values
as determined by @i(t)$@i(equal).
The effect of this operation is equivalent to the following procedure body:@*
@hsp(.5inch)qt = @b[sequence] [t]@*
@hsp(.5inch)@b(if) qt$size(s1) ~= qt$size(s2) @b(then return (false)) @b(end)@*
@hsp(.5inch)@b(for) i: @b[int] @b(in) qt$indexes(s1) @b(do)@*
@hsp(.8inches)@b(if) s1[i] ~= s2[i] @b(then return(false)) @b(end)@*
@hsp(1.0inch)@b(resignal) failure@*
@hsp(.8inches)@b(end)@*
@hsp(.5inch)@b(return) (@b(true))@*
@end(itext)
@end(OpDef)

@begin(OpDef)
similar = @b(proc) (s1, s2: @b[sequence][t]) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires) @i(t) has similar: @b(proctype) (t, t) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns @b[true] if @i(s1) and @i(s2) have similar values
as determined by @i(t)$@i(similar).
@i(Similar) works in the same way as @i(equal), except that @i(t)$@i(similar) is
used instead of @i(t)$@i(equal).
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (s: @b[sequence][t]) @b(returns) (@b[sequence][t]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires) @i(t) has copy: @b(proctype) (t) @b(returns) (t) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns a sequence having as elements copies
of the elements of @i(s).
The effect is equivalent to that of the following procedure body:@*
@hsp(.5inch)qt = @b[sequence][t]@*
@hsp(.5inch)y: qt := qt$new()@*
@hsp(.5inch)@b(for) e: t @b(in) qt$elements(s) @b(do)@*
@hsp(.8inches)y := qt$addh(y, t$copy(e)) @b[resignal] failure@*
@hsp(.8inches)@b(end)@*
@hsp(.5inches)@b(return) (y)@*
@end(itext)
@end(OpDef)

@begin(OpDef)
@b[transmit] = @b(proc) (s: @b[sequence][t]) @b(returns) (@b[sequence][t]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires) @i(t) has @b[transmit]
@end(itext)
@begin(itext)
@b(effects)  Returns a sequence having as elements transmitted copies
of the elements of @i(s) in the same order.
Sharing among elements is preserved.
Signals failure if this cannot be represented on the receiving end
and also resignals any failures from @i(t)$@b[transmit].
@end(itext)
@end(OpDef)
@end(TypeDef)


@Section(Arrays)
@label(array-type-def)
@index(array)
@begin(TypeDef)
@b[array] = @b(data type) [t: type] @b(is) create, new, predict, fill, fill@us()copy, addh, addl, remh, reml,
				set@us()low, trim, store, fetch, bottom, top, empty, size, low, high, elements, indexes,
				equal, similar, similar1, copy, copy1, @b[transmit]

@b(Overview)

@begin(OverviewText)
Arrays are mutable objects that represent tuples of elements of type @i(t)
that can grow and shrink dynamically.
Each array's state consists of this tuple of elements and
a low bound (or index).
The elements are indexed sequentially, starting from the low bound.
Each array also has an identity as an object.

Arrays can be created by calling array operations @i(create), @i(new), @i(fill),
@i(fill@us()copy), and @i(predict).
They can also be created by means of the array constructor,
which specifies the array low bound,
and an arbitrary number of initial elements,
see Section @ref(sect-array-constructor).

Operations @i(low), @i(high), and @i(size) return
the current low and high bounds and size of the array.
For array @i(a), @i(size)(@i(a)) is the number of elements in
@i(a), which is zero if @i(a) is empty.
These are related by the equation:
@i(high)(@i(a)) = @i(low)(@i(a)) @ZAdd() @i(size)(@i(a)) @MinusSign() 1.

For any index @i(i) between the low and high bound of an array, there is a 
defined element, @i(a)[@i(i)].
The @i(bounds) exception is raised when an attempt is made to access an
element outside the defined range.
Any array must have a low bound, a high bound, and a size which are all 
legal integers.  An implementation may restrict these to some smaller range
of integers.
A call that would lead to an array whose low or high bound or size
is outside the defined range terminates with a @i(limits) exception.
@end(OverviewText)

@b(Operations)

@begin(OpDef)
create = @b(proc) (lb: @b[int]) @b(returns) (@b[array][t]) @b(signals) (limits)
@begin(itext)
@b(effects)  Returns a new, empty array with low bound @i(lb).
@i(Limits) occurs if the resulting array would not be supported by the 
implementation.
@end(itext)
@end(OpDef)

@begin(OpDef)
new = @b(proc) ( ) @b(returns) (@b[array][t])
@begin(itext)
@b(effects)  Returns a new, empty array with low bound 1.  
Equivalent to create(1).
@end(itext)
@end(OpDef)

@begin(OpDef)
predict = @b(proc) (lb, cnt: @b[int]) @b(returns) (@b[array][t]) @b(signals) (limits)
@begin(itext)
@b(effects)  Returns a new, empty array with low bound @i(lb).
The absolute value of @i(cnt) is a prediction of how many @i(addh)s or
@i(addl)s are likely to be performed on this new array.
If @i(cnt) > 0, @i(addh)s are expected; otherwise @i(addl)s are
expected.  These operations may execute faster than
if the array had been produced by calling @i(create).
@i(Limits) occurs if the resulting array would not be supported by the 
implementation because of its initial low bound
(not because of its predicted size or because of the predicted
high or low bound).
@end(itext)
@end(OpDef)

@begin(OpDef)
fill = @b(proc) (lb, cnt: @b[int], elem: t) @b(returns) (@b[array][t]) @b(signals) (negative@us()size, limits)
@begin(itext)
@b(effects)  If @i(cnt) < 0, signals @i(negative@us()size).
Returns a new array with low bound @i(lb) and size @i(cnt), and
with @i(elem) as each element;
if this new array would not be supported by the implementation,
signals @i(limits).
@end(itext)
@end(OpDef)

@begin(OpDef)
fill@us()copy = @b(proc) (lb, cnt: @b[int], elem: t) @b(returns) (@b[array][t]) 
@hsp(1.2inches)@b(signals) (negative@us()size, limits, failure(@b[string]))
@begin(itext)
@b(requires)  @i(t) has copy: @b(proctype) (t) @b(returns) (t) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  The effect is like @i(fill) except that @i(elem) is copied
@i(cnt) times.
If @i(cnt) < 0, signals @i(negative@us()size).
Normally returns a new array with
low bound @i(lb) and size @i(cnt) and with each element a copy
of @i(elem), as produced by @i(t)$@i(copy).
Any @i(failure) signal raised by @i(t)$@i(copy) is immediately resignalled.
This operation does not originate any @i(failure) signals by itself.
However, if the new array cannot be represented by the implementation,
signals @i(limits).
@end(itext)
@end(OpDef)

@begin(OpDef)
addh = @b(proc) (a: @b[array][t], elem: t) @b(signals) (limits)
@begin(itext)
@b(modifies) @i(a).
@end(itext)
@begin(itext)
@b(effects)  If extending @i(a) on the high end causes the high bound
or size of @i(a) to be outside the range supported by the implementation,
signals @i(limits).  Otherwise extends @i(a) by 1 in the
high direction and stores @i(elem) as the new element.
That is, @i(a)@-(post)[@i(high)(@i(a)@-(pre))+1] = @i(elem).
@end(itext)
@end(OpDef)

@begin(OpDef)
addl = @b(proc) (a: @b[array][t], elem: t) @b(signals) (limits)
@begin(itext)
@b(modifies) @i(a).
@end(itext)
@begin(itext)
@b(effects)  If extending @i(a) on the low end causes the low bound
or size of @i(a) to be outside the range supported by the implementation,
signals @i(limits).  Otherwise extends @i(a) by 1 in the
low direction and stores @i(elem) as the new element.
That is, @i(a)@-(post)[@i(low)(@i(a)@-(pre))@MinusSign()1] = @i(elem).
@end(itext)
@end(OpDef)

@begin(OpDef)
remh = @b(proc) (a: @b[array][t]) @b(returns) (t) @b(signals) (bounds)
@begin(itext)
@b(modifies) @i(a).
@end(itext)
@begin(itext)
@b(effects)  If @i(a) is empty, signals @i(bounds).  Otherwise shrinks
@i(a) by removing its high element and returning the removed element.
That is, high(@i(a)@-(post)) = high(@i(a)@-(pre)) @MinusSign() 1.
@end(itext)
@end(OpDef)

@begin(OpDef)
reml = @b(proc) (a: @b[array][t]) @b(returns) (t) @b(signals) (bounds)
@begin(itext)
@b(modifies) @i(a).
@end(itext)
@begin(itext)
@b(effects)  If @i(a) is empty, signals @i(bounds).  Otherwise shrinks
@i(a) by removing its low element and returning the removed element.
That is, low(@i(a)@-(post)) = low(@i(a)@-(pre)) @ZAdd() 1.
@end(itext)
@end(OpDef)

@begin(OpDef)
set@us()low = @b(proc) (a: @b[array][t], lb: @b[int]) @b(signals) (limits)
@begin(itext)
@b(modifies) @i(a).
@end(itext)
@begin(itext)
@b(effects)  Modifies the low and high bounds of @i(a); the new low bound
of @i(a) is @i(lb) and the new high bound is
@i(high)(@i(a)@-(post)) = 
@i(high)(@i(a)@-(pre))@ZAdd()@i(lb)@MinusSign()@i(low)(@i(a)@-(pre)).
If the new low (or high) bound
is not supported by the implementation,
signals @i(limits) and does not modify @i(a).
@end(itext)
@end(OpDef)

@begin(OpDef)
trim = @b(proc) (a: @b[array][t], lb, cnt: @b[int]) @b(signals) (negative@us()size, bounds)
@begin(itext)
@b(modifies) @i(a).
@end(itext)
@begin(itext)
@b(effects)  If @i(cnt) < 0, signals @i(negative@us()size).
If @i(lb) < @i(low)(@i(a)) or @i(lb) > @i(high)(@i(a))@ZAdd()1,
signals @i(bounds).
Otherwise modifies
@i(a) by removing all elements with index < @i(lb) or greater than or
equal to
@i(lb)@ZAdd()@i(cnt);
the new low bound is @i(lb).
For example, if @i(a)@ =@ @b[array][@b[int]]$[1,2,3,4,5], then:@*
@hsp(.5inch)trim(a, 2, 2) results in @i(a) having the value of @b[array][@b[int]]$[2: 2, 3]@*
@hsp(.5inch)trim(a, 4, 3) results in @i(a) having the value of @b[array][@b[int]]$[4: 4, 5]
@end(itext)
@end(OpDef)

@begin(OpDef)
store = @b(proc) (a: @b[array][t], i: @b[int], elem: t) @b(signals) (bounds)
@begin(itext)
@b(modifies) @i(a).
@end(itext)
@begin(itext)
@b(effects)  If @i(i) < @i(low)(@i(a)) or @i(i) > @i(high)(@i(a)), signals @i(bounds); otherwise
makes @i(elem) the element of @i(a) with index @i(i).
@end(itext)
@end(OpDef)

@begin(OpDef)
fetch = @b(proc) (a: @b[array][t], i: @b[int]) @b(returns) (t) @b(signals) (bounds)
@begin(itext)
@b(effects)  If @i(i) < @i(low)(@i(a)) or @i(i) > @i(high)(@i(a)), signals @i(bounds); otherwise
returns the element of @i(a) with index @i(i).
@end(itext)
@end(OpDef)

@begin(OpDef)
bottom = @b(proc) (a: @b[array][t]) @b(returns) (t) @b(signals) (bounds)
@begin(itext)
@b(effects)  If @i(a) is empty, signals @i(bounds); otherwise returns @i(a)[@i(low)(@i(a))].
@end(itext)
@end(OpDef)


@begin(OpDef)
top = @b(proc) (a: @b[array][t]) @b(returns) (t) @b(signals) (bounds)
@begin(itext)
@b(effects)  If @i(a) is empty, signals @i(bounds); otherwise returns @i(a)[@i(high)(@i(a))].
@end(itext)
@end(OpDef)

@begin(OpDef)
empty = @b(proc) (a: @b[array][t]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if @i(a) contains no elements; otherwise returns @b[false].
@end(itext)
@end(OpDef)

@begin(OpDef)
size = @b(proc) (a: @b[array][t]) @b(returns) (@b[int])
@begin(itext)
@b(effects)  Returns a count of the number of elements of @i(a).
@end(itext)
@end(OpDef)

@begin(OpDef)
low = @b(proc) (a: @b[array][t]) @b(returns) (@b[int])
@begin(itext)
@b(effects)  Returns the low bound of @i(a).
@end(itext)
@end(OpDef)

@begin(OpDef)
high = @b(proc) (a: @b[array][t]) @b(returns) (@b[int])
@begin(itext)
@b(effects)  Returns the high bound of @i(a).
@end(itext)
@end(OpDef)

@begin(OpDef)
elements = @b(iter) (a: @b[array][t]) @b(yields) (t) @b(signals) (failure(@b[string]))
@begin(itext)
@b(effects) Yields the elements of @i(a), exactly once for each index,
from the low bound to the high bound
(i.e., @i(bottom)(@i(a)@-(pre)), ..., @i(top)(@i(a)@-(pre))).
The elements are fetched one at a time, using the indexes that were legal
at the start of the call.
If, during the iteration, @i(a) is modified so that
fetching at a previously legal index signals @i(bounds),
then the iterator signals @i(failure) with the string "bounds".
The iterator is divisible at yields.
@end(itext)
@end(OpDef)

@begin(OpDef)
indexes = @b(iter) (a: @b[array][t]) @b(yields) (@b[int])
@begin(itext)
@b(effects)  Yields the indexes of @i(a) from the low bound
of @i(a)@-(pre) to the high bound of @i(a)@-(pre).
Note that @i(indexes) is unaffected
by any modifications done by the loop body.
It is divisible at yields.
@end(itext)
@end(OpDef)

@begin(OpDef)
equal = @b(proc) (a1, a2: @b[array][t]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if @i(a1) and @i(a2) refer to the same array
object; otherwise returns @b[false].
@end(itext)
@end(OpDef)

@begin(OpDef)
similar = @b(proc) (a1, a2: @b[array][t]) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  @i(t) has similar: @b(proctype) (t, t) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns @b[true] if @i(a1) and @i(a2) have the same low and high
bounds
and if their elements are pairwise similar as
determined by @i(t)$@i(similar).
This effect of this operation is equivalent to the following procedure body
(except that this operation is only divisible at calls to @i(t)$@i(similar)):@*
@hsp(.5inch)at = @b[array][t]@*
@hsp(.5inch)@b(if) at$low(a1) ~= at$low(a2)  @b(cor)  at$size(a1) ~= at$size(a2)@*
@hsp(.8inches)@b(then return (false))@*
@hsp(.8inches)@b(end)@*
@hsp(.5inch)@b(for) i: @b[int] @b(in) at$indexes(a1) @b(do)@*
@hsp(.8inches)@b(if) ~t$similar(a1[i], a2[i]) @b(then return) (@b[false]) @b(end)@*
@hsp(1inch)@b(resignal) failure@*
@hsp(1inch)@b(except when) bounds: @b(signal) failure("bounds") @b(end)@*
@hsp(.8inches)@b(end)@*
@hsp(.5inch)@b(return) (@b(true))
@end(itext)
@end(OpDef)

@begin(OpDef)
similar1 = @b(proc) (a1, a2: @b[array][t]) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  @i(t) has equal: @b(proctype) (t, t) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns @b[true] if @i(a1) and @i(a2) have the same low and high bounds
and if their elements are pairwise equal as
determined by @i(t)$@i(equal).  This operation works the same way as @i(similar),
except that @i(t)$@i(equal) is used instead of @i(t)$@i(similar).
@end(itext)
@end(OpDef)


@begin(OpDef)
copy = @b(proc) (a: @b[array][t]) @b(returns) (b: @b[array][t]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  @i(t) has copy: @b(proctype) (t) @b(returns) (t) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns a new array @i(b) with the same low and high bounds
as @i(a) and such that each element @i(b)[@i(i)] contains
@i(t)$@i(copy)(@i(a)[@i(i)]).
The effect of this operation is equivalent to the following body
(except that it is only divisible at calls to @i(t)$@i(copy)):@*
@hsp(.5inch)b: @b[array][t] := @b[array][t]$copy1(a)@*
@hsp(.5inch)@b(for) i: @b[int] @b(in) @b[array][t]$indexes(a) @b(do)@*
@hsp(.8inches)b[i] := t$copy(a[i])@*
@hsp(1.1inches)@b(resignal) failure@*
@hsp(1.1inches)@b(except when) bounds: @b(signal) failure("bounds") @b(end)@*
@hsp(.8inches)@b(end)@*
@hsp(.5inch)@b(return) (b)
@end(itext)
@end(OpDef)

@begin(OpDef)
copy1 = @b(proc) (a: @b[array][t]) @b(returns) (b: @b[array][t])
@begin(itext)
@b(effects)  Returns a new array @i(b) with the same low and high bounds
as @i(a) and such that each element @i(b)[@i(i)] contains
the same element as @i(a)[@i(i)].
@end(itext)
@end(OpDef)

@begin(OpDef)
@b[transmit] = @b(proc) (a: @b[array][t]) @b(returns) (b: @b[array][t]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  @i(t) has @b[transmit]
@end(itext)
@begin(itext)
@b(effects)  Returns a new array @i(b) with the same low and high bounds
as @i(a) and such that each element @i(b)[@i(i)] contains a transmitted copy
of @i(a)[@i(i)].
Sharing among the elements of @i(a) is preserved in @i(b).
Signals @i(failure) if @i(b) cannot be represented on the receiving end
or if fetching an element at a legal index of @i(a)@-(pre) causes a bounds
exception
and resignals any @i(failure) signals raised by @i(t)$@b[transmit].
@end(itext)
@end(OpDef)
@end(TypeDef)


@Section(Atomic Arrays)
@label(atomic_array-type-def)
@IndexEntry(Key="atomic_array", Entry="atomic@us<>array", Number)
@IndexSecondary(Primary="atomic", Secondary="built-in atomic types")
@begin(TypeDef)
@b[atomic@us<>array] = @b(data type) [t: type] @b(is) create, new, predict, fill, fill@us()copy, addh, addl, remh, reml, 
			set@us()low, trim, store, fetch, bottom, top, empty, size, low, high,	elements, indexes,
			aa2a, a2aa, equal, similar, similar1, copy, copy1, @b[transmit],
			test@us()and@us()read, test@us()and@us()write,	can@us()read, can@us()write, read@us()lock, write@us()lock

@b(Overview)

@begin(OverviewText)
Atomic@us()arrays are mutable atomic objects that represent tuples of elements
of type @i(t) that can grow and shrink dynamically.
Each atomic@us()array's (sequential) state consists of this tuple of elements
and a low bound (or index).
The elements are indexed sequentially, starting from the low bound.
Each atomic@us<>array also has an identity as an object.

Atomic@us()arrays can be created by calling 
atomic@us()array operations @i(create), @i(new), @i(fill),
@i(fill@us()copy), and @i(predict).
They can also be created by means of the atomic@us<>array constructor,
which specifies the array low bound,
and an arbitrary number of initial elements,
see Section @ref(sect-array-constructor).

Operations @i(low), @i(high), and @i(size) return
the current low and high bounds and size of the atomic@us<>array.
For an atomic@us<>array @i(a), @i(size)(@i(a)) is the number of elements in
@i(a), which is zero if @i(a) is empty.
These are related by the equation:
@i(high)(@i(a)) = @i(low)(@i(a)) @ZAdd() @i(size)(@i(a)) @MinusSign() 1.

For any index @i(i) between the low 
and high bound of an atomic@us<>array, there is a 
defined element, @i(a)[@i(i)].
The @i(bounds) exception is raised when an attempt is made to access an
element outside the defined range.
Any atomic@us<>array must have a low bound, a high bound, and a size which are all 
legal integers.
An implementation may restrict these to some smaller range of integers.
A call that would lead to an atomic@us<>array whose low or high bound or size
is outside the defined range terminates with a @i(limits) exception.
@i(limits) exception.

Atomic@us<>arrays use read/write locking to achieve atomicity.
The locking rules are described in Section @ref(sect-atomic-obj).
It is an error if a process that is not in an action attempts to test or obtain
a lock;
when this happens the guardian running the process will crash.
As defined below, the only operation that (in the normal case)
does not attempt to test or obtain a lock is the @i(equal) operation.
@end(OverviewText)

@b(Operations)

@begin(OpDef)
create = @b(proc) (lb: @b[int]) @b(returns) (a:@b[atomic@us<>array][t]) @b(signals) (limits)
@begin(itext)
@b(effects)  Returns a new, empty atomic@us()array @i(a) with low bound @i(lb).
@i(Limits) occurs if the resulting atomic@us()array would not be supported by the 
implementation.
The caller obtains a read lock on @i(a).
@end(itext)
@end(OpDef)

@begin(OpDef)
new = @b(proc) ( ) @b(returns) (@b[atomic@us<>array][t])
@begin(itext)
@b(effects)  Equivalent to @i(create)(1).
@end(itext)
@end(OpDef)

@begin(OpDef)
predict = @b(proc) (lb, cnt: @b[int]) @b(returns) (a: @b[atomic@us<>array][t]) @b(signals) (limits)
@begin(itext)
@b(effects)  Returns a new, empty atomic@us()array @i(a) with low bound @i(lb).
The caller obtains a read lock on @i(a).
This is essentially the same as create(@i(lb)), except that
the absolute value of @i(cnt) is a prediction of how many @i(addh)s or
@i(addl)s are likely to be performed on this new atomic@us()array.
If @i(cnt) > 0, @i(addh)s are expected; otherwise @i(addl)s are
expected.  These operations may execute faster than
if the atomic@us()array had been produced by calling @i(create).
@i(Limits) occurs if the resulting atomic@us()array would not be supported by the 
implementation because of its initial low bound
(not because of its predicted size or because of the predicted
high or low bound).
@end(itext)
@end(OpDef)

@begin(OpDef)
fill = @b(proc) (lb, cnt: @b[int], elem: t) @b(returns) (@b[atomic@us<>array][t]) @b(signals) (negative@us()size, limits)
@begin(itext)
@b(effects)  If @i(cnt) < 0, signals @i(negative@us()size).
Returns a new atomic@us()array with low bound @i(lb) and size @i(cnt), and
with @i(elem) as each element;
if this new atomic@us()array would not be supported by the implementation,
signals @i(limits).
The caller obtains a read lock on the result.
@end(itext)
@end(OpDef)

@begin(OpDef)
fill@us()copy = @b(proc) (lb, cnt: @b[int], elem: t) @b(returns) (@b[atomic@us<>array][t]) 
@hsp(1.2inches)@b(signals) (negative@us()size, limits, failure(@b[string]))
@begin(itext)
@b(requires)  @i(t) has copy: @b(proctype) (t) @b(returns) (t) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  The effect is like @i(fill) except that @i(elem) is copied
@i(cnt) times.
If @i(cnt) < 0, signals @i(negative@us()size).
Normally returns a new array with
low bound @i(lb) and size @i(cnt) and with each element a copy
of @i(elem), as produced by @i(t)$@i(copy).
The caller obtains a read lock on the result.
Any @i(failure) signal raised by @i(t)$@i(copy) is immediately resignalled.
This operation does not originate any @i(failure) signals by itself.
If the new array cannot be represented by the implementation,
signals @i(limits).
@end(itext)
@end(OpDef)

@begin(OpDef)
addh = @b(proc) (a: @b[atomic@us<>array][t], elem: t) @b(signals) (limits)
@begin(itext)
@b(modifies) @i(a).
@end(itext)
@begin(itext)
@b(effects)  Obtains a write lock on @i(a).
If extending @i(a) on the high end would cause the high bound
or size of @i(a) to be outside the range supported by the implementation,
then signals @i(limits).
Otherwise extends @i(a) by 1 in the
high direction, and stores @i(elem) as the new element.
That is, @i(a)@-(post)[@i(high)(@i(a)@-(pre))+1] = @i(elem).
@end(itext)
@end(OpDef)

@begin(OpDef)
addl = @b(proc) (a: @b[atomic@us<>array][t], elem: t) @b(signals) (limits)
@begin(itext)
@b(modifies) @i(a).
@end(itext)
@begin(itext)
@b(effects)  Obtains a write lock on @i(a).
If extending @i(a) on the low end would causes the low bound
or size of @i(a) to be outside the range supported by the implementation,
then signals @i(limits).
Otherwise extends @i(a) by 1 in the low direction,
and stores @i(elem) as the new element.
That is, @i(a)@-(post)[@i(low)(@i(a)@-(pre))@MinusSign()1] = @i(elem).
@end(itext)
@end(OpDef)

@begin(OpDef)
remh = @b(proc) (a: @b[atomic@us<>array][t]) @b(returns) (t) @b(signals) (bounds)
@begin(itext)
@b(modifies) @i(a).
@end(itext)
@begin(itext)
@b(effects)  Obtains a write lock on @i(a).
If @i(a) is empty, signals @i(bounds).
Otherwise shrinks @i(a) by removing its high element,
and returns the removed element.
That is, high(@i(a)@-(post)) = high(@i(a)@-(pre)) @MinusSign() 1.
@end(itext)
@end(OpDef)

@begin(OpDef)
reml = @b(proc) (a: @b[atomic@us<>array][t]) @b(returns) (t) @b(signals) (bounds)
@begin(itext)
@b(modifies) @i(a).
@end(itext)
@begin(itext)
@b(effects)  Obtains a write lock on @i(a).
If @i(a) is empty, signals @i(bounds).
Otherwise shrinks @i(a) by removing its low element,
and returns the removed element.
That is, low(@i(a)@-(post)) = low(@i(a)@-(pre)) @ZAdd() 1.
@end(itext)
@end(OpDef)

@begin(OpDef)
set@us()low = @b(proc) (a: @b[atomic@us<>array][t], lb: @b[int]) @b(signals) (limits)
@begin(itext)
@b(modifies) @i(a).
@end(itext)
@begin(itext)
@b(effects)  Obtains a write lock on @i(a).
If the new low (or high) bound would not be supported by the implementation,
then signals @i(limits).
Otherwise, modifies the low and high bounds of @i(a); the new low bound
of @i(a) is @i(lb) and the new high bound is
@i(high)(@i(a)@-(post)) = 
@i(high)(@i(a)@-(pre))@ZAdd()@i(lb)@MinusSign()@i(low)(@i(a)@-(pre)).
@end(itext)
@end(OpDef)

@begin(OpDef)
trim = @b(proc) (a: @b[atomic@us<>array][t], lb, cnt: @b[int]) @b(signals) (negative@us()size, bounds)
@begin(itext)
@b(modifies) @i(a).
@end(itext)
@begin(itext)
@b(effects)  If @i(cnt)@ <@ 0, signals @i(negative@us()size) and 
does not obtain any locks.
Otherwise obtains a write lock on @i(a).
If @i(lb) < @i(low)(@i(a)) or @i(lb) > @i(high)(@i(a))@ZAdd()1,
signals @i(bounds).
Otherwise, modifies @i(a) by removing all elements with index@ <@ @i(lb)
or greater than or equal to @i(lb)@ZAdd()@i(cnt);
the new low bound is @i(lb).
For example, if @i(a)@ =@ @b[atomic@us<>array][@b[int]]$[1,2,3,4,5], then:@*
@hsp(.5inch)trim(a, 2, 2) results in @i(a) having value @b[atomic@us<>array][@b[int]]$[2: 2, 3]@*
@hsp(.5inch)trim(a, 4, 3) results in @i(a) having value @b[atomic@us<>array][@b[int]]$[4: 4, 5]
@end(itext)
@end(OpDef)

@begin(OpDef)
store = @b(proc) (a: @b[atomic@us<>array][t], i: @b[int], elem: t) @b(signals) (bounds)
@begin(itext)
@b(modifies) @i(a).
@end(itext)
@begin(itext)
@b(effects)  Obtains a write lock on @i(a).
If @i(i) < @i(low)(@i(a)) or @i(i) > @i(high)(@i(a)),
signals @i(bounds);
otherwise makes @i(elem) the element of @i(a) with index @i(i).
@end(itext)
@end(OpDef)

@begin(OpDef)
fetch = @b(proc) (a: @b[atomic@us<>array][t], i: @b[int]) @b(returns) (t) @b(signals) (bounds)
@begin(itext)
@b(effects)  If @i(i)@ <@ @i(low)(@i(a)) or @i(i)@ >@ @i(high)(@i(a)), 
signals @i(bounds); otherwise
returns the element of @i(a) with index @i(i).  Always obtains a 
read lock on @i(a).
@end(itext)
@end(OpDef)

@begin(OpDef)
bottom = @b(proc) (a: @b[atomic@us<>array][t]) @b(returns) (t) @b(signals) (bounds)
@begin(itext)
@b(effects)  If @i(a) is empty, signals @i(bounds); otherwise returns @i(a)[@i(low)(@i(a))].
Always obtains a read lock on @i(a).
@end(itext)
@end(OpDef)

@begin(OpDef)
top = @b(proc) (a: @b[atomic@us<>array][t]) @b(returns) (t) @b(signals) (bounds)
@begin(itext)
@b(effects)  If @i(a) is empty, signals @i(bounds); otherwise returns @i(a)[@i(high)(@i(a))].
Always obtains a read lock on @i(a).
@end(itext)
@end(OpDef)

@begin(OpDef)
empty = @b(proc) (a: @b[atomic@us<>array][t]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if @i(a) contains no elements,
returns @b[false] otherwise.
In either case obtains a read lock on @i(a).
@end(itext)
@end(OpDef)

@begin(OpDef)
size = @b(proc) (a: @b[atomic@us<>array][t]) @b(returns) (@b[int])
@begin(itext)
@b(effects)  Returns a count of the number of elements of @i(a), obtains a 
read lock on @i(a).
@end(itext)
@end(OpDef)

@begin(OpDef)
low = @b(proc) (a: @b[atomic@us<>array][t]) @b(returns) (@b[int])
@begin(itext)
@b(effects)  Returns the low bound of @i(a), obtains a read lock on @i(a)
@end(itext)
@end(OpDef)

@begin(OpDef)
high = @b(proc) (a: @b[atomic@us<>array][t]) @b(returns) (@b[int])
@begin(itext)
@b(effects)  Returns the high bound of @i(a), obtains a read lock on @i(a).
@end(itext)
@end(OpDef)

@begin(OpDef)
elements = @b(iter) (a: @b[atomic@us<>array][t]) @b(yields) (t) @b(signals) (failure(@b[string]))
@begin(itext)
@b(effects) Obtains a read lock on @i(a) and
yields the elements of @i(a), each exactly once for each index,
from the low bound to the high bound
(i.e., @i(bottom)(@i(a)@-(pre)), ..., @i(top)(@i(a)@-(pre))).
The elements are fetched one at a time, using the indexes that were legal
at the start of the call.
If, during the iteration, @i(a) is modified so that
fetching at a previously legal index signals @i(bounds),
then the iterator signals @i(failure) with the string "bounds".
The iterator is divisible at yields.
@end(itext)
@end(OpDef)

@begin(OpDef)
indexes = @b(iter) (a: @b[atomic@us<>array][t]) @b(yields) (@b[int])
@begin(itext)
@b(effects)  Obtains a read lock on a, then 
yields the indexes of @i(a) from the low bound
of @i(a)@-(pre) to the high
bound of @i(a)@-(pre).  Note that @i(indexes) is unaffected
by any modifications done by the loop body.  It is divisible at yields.
@end(itext)
@end(OpDef)

@begin(OpDef)
aa2a = @b(proc) (aa: @b[atomic@us<>array][t]) @b(returns) (@b[array][t])
@begin(itext)
@b(effects) Obtains a read lock on @i(aa) and 
returns an array @i(a) with the same (sequential) state.
@end(itext)
@end(OpDef)

@begin(OpDef)
a2aa = @b(proc) (@b[array][t]) @b(returns) (aa: @b[atomic@us<>array][t]) 
@begin(itext)
@b(effects)  Returns an @b[atomic@us<>array] @i(aa) with the same state
as @i(a).
Obtains a read lock on @i(aa).
@end(itext)
@end(OpDef)

@begin(OpDef)
equal = @b(proc) (a1, a2: @b[atomic@us<>array][t]) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if @i(a1) and @i(a2) refer to the same atomic@us()array
object; otherwise returns @b[false].  No locks are obtained.
@end(itext)
@end(OpDef)

@begin(OpDef)
similar = @b(proc) (a1, a2: @b[atomic@us<>array][t]) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  @i(t) has similar: @b(proctype) (t, t) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns @b[true] if @i(a1) and @i(a2) have the same low and high
bounds
and if their elements are pairwise similar as
determined by @i(t)$@i(similar).
See the description of the @i(similar) operation of @b[array] for an equivalent
body of code.
This operation is divisible at calls to @i(t)$@i(similar).
Read locks are obtained on @i(a1) and @i(a2), in that order.
@end(itext)
@end(OpDef)

@begin(OpDef)
similar1 = @b(proc) (a1, a2: @b[atomic@us<>array][t]) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  @i(t) has equal: @b(proctype) (t, t) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns @b[true] if @i(a1) and @i(a2) have the same low and high bounds
and if their elements are pairwise equal as
determined by @i(t)$@i(equal).  This operation works the same way as @i(similar),
except that @i(t)$@i(equal) is used instead of @i(t)$@i(similar).
Read locks are obtained on @i(a1) and @i(a2), in that order.
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (a: @b[atomic@us<>array][t]) @b(returns) (b: @b[atomic@us<>array][t]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  @i(t) has copy: @b(proctype) (t) @b(returns) (t) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns a new atomic@us()array @i(b) with the same low and high bounds
as @i(a) and such that each element @i(b)[@i(i)] contains
@i(t)$@i(copy)(@i(a)[@i(i)]).
See the description of the @i(copy) operation of @b[array] for an equivalent
body of code.
This operation is divisible at calls to @i(t)$@i(copy), and obtains 
read locks on @i(a) and @i(b).
@end(itext)
@end(OpDef)

@begin(OpDef)
copy1 = @b(proc) (a: @b[atomic@us<>array][t]) @b(returns) (b: @b[atomic@us<>array][t])
@begin(itext)
@b(effects)  Returns a new atomic@us()array @i(b) with the same low and high bounds
as @i(a) and such that each element @i(b)[@i(i)] contains
the same element as @i(a)[@i(i)].  Read locks are obtained on @i(a) and @i(b).
@end(itext)
@end(OpDef)

@begin(OpDef)
@b[transmit] = @b(proc) (a: @b[atomic@us<>array][t]) @b(returns) (b: @b[atomic@us<>array][t]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  @i(t) has @b[transmit]
@end(itext)
@begin(itext)
@b(effects)  Returns a new array @i(b) with the same low and high bounds
as @i(a) and such that each element @i(b)[@i(i)] contains a transmitted copy
of @i(a)[@i(i)].
Read locks are obtained on @i(a) and @i(b).
Sharing among the elements of @i(a) is preserved in @i(b).
Signals @i(failure) if @i(b) cannot be represented on the receiving end
or if fetching an element at a legal index of @i(a)@-(pre) causes a bounds
exception
and resignals any @i(failure) signals raised by @i(t)$@b[transmit].
@end(itext)
@end(OpDef)

@begin(OpDef)
test@us()and@us()read = @b(proc) (aa: @b[atomic@us<>array][t]) @b(returns) (@b[bool])
@begin(itext)
@b(effects) Tries to obtain a read lock on @i(aa).  If the lock is obtained,
returns @b(true); otherwise no lock is obtained and the operation returns 
@b(false).  The operation does not "wait" for a lock.  Even if the executing
action "knows" that a lock could be obtained, @b(false) may be returned.
Even if @b(false) is returned, a subsequent attempt to obtain a read lock 
might succeed without waiting.
@end(itext)
@end(OpDef)

@begin(OpDef)
test@us()and@us()write = @b(proc) (aa: @b[atomic@us<>array][t]) @b(returns) (@b[bool])
@begin(itext)
@b(effects) Tries to obtain a write lock on @i(aa).  If the lock is obtained,
returns @b(true); otherwise no lock is obtained and the operation returns 
@b(false).  The operation does not "wait" for a lock.  Even if the executing
action "knows" that a lock could be obtained, @b(false) may be returned.
Even if @b(false) is returned, a subsequent attempt to obtain a write lock 
might succeed without waiting.
@end(itext)
@end(OpDef)

@begin(OpDef)
can@us()read = @b(proc) (aa: @b[atomic@us<>array][t]) @b(returns) (@b[bool])
@begin(itext)
@b(effects) Returns @b(true) if a read lock could be obtained on @i(aa)
without waiting,
otherwise returns @b(false).  No lock is actually obtained.  
Even if the executing
action "knows" that a lock could be obtained, @b(false) may be returned.
Since some concurrent action may obtain or release a lock on an 
atomic@us()array
at any time, the information returned is unreliable:
even if @b(true) is returned, a subsequent attempt to obtain the lock may
require waiting; and even if @b(false) is returned, a 
subsequent attempt to obtain a read lock 
might succeed without waiting.
@end(itext)
@end(OpDef)

@begin(OpDef)
can@us()write = @b(proc) (aa: @b[atomic@us<>array][t]) @b(returns) (@b[bool])
@begin(itext)
@b(effects) Returns @b(true) if a write lock could be obtained on @i(aa)
without waiting,
otherwise returns @b(false).  No lock is actually obtained.  
Even if the executing
action "knows" that a lock could be obtained, @b(false) may be returned.
Since some concurrent action may obtain or release a lock on an 
atomic@us()array
at any time, the information returned is unreliable:
even if @b(true) is returned, a subsequent attempt to obtain the lock may
require waiting; and even if @b(false) is returned, a 
subsequent attempt to obtain a write lock 
might succeed without waiting.
@end(itext)
@end(OpDef)

@begin(OpDef)
read@us()lock = @b(proc) (aa: @b[atomic@us<>array][t])
@begin(itext)
@b(effects)  Obtains a read lock on @i(aa).
@end(itext)
@end(OpDef)

@begin(OpDef)
write@us()lock = @b(proc) (aa: @b[atomic@us<>array][t])
@begin(itext)
@b(effects)  Obtains a write lock on @i(aa).
@end(itext)
@end(OpDef)
@end(TypeDef)

@NewPage()

@Section(Structs)
@label(struct-type-def)
@index(struct)
@label(apdx-struct)
@begin(TypeDef)
@b[struct] = @b(data type) [n@-(1): t@-(1), ..., n@-(k): t@-(k)] @b(is) replace@us()n@-(1), ..., replace@us()n@-(k), get@us()n@-(1), ..., get@us()n@-(k), s2r, r2s,
			equal, similar, copy, @b[transmit]

@b(Overview)

@begin(OverviewText)
A @b[struct] (short for "structure") 
is an immutable collection of one or more named objects.
The names are called @i(selectors), and the objects are called @i(components).
Different components may have different types.

An instantiation of @b[struct] has the form:@*
@hsp(.5inch)@b(struct) [ field@us()spec, @BNFdots() ]@*
where@*
@hsp(.5inch)field@us()spec @BNFequals() name, @BNFdots() : type@us()actual@*
(see Appendix @ref(apdx-syntax)).
Selectors must be unique within an instantiation (ignoring capitalization),
but the ordering and grouping of selectors is unimportant.
For example, the following name the same type:@*
@hsp(.5inch)@b[struct][last, first, middle: @b[string],   age: @b[int]]@*
@hsp(.5inch)@b[struct][last: @b[string],   age: @b[int],   first, middle: @b[string]]@*
A struct is created using a struct constructor,
see Section @ref(sect-struct-constructor).

For purposes of the certain operations, the the names
of the selectors are ordered lexicographically.
@index(lexicographic order)
Lexicographic ordering of the selectors is the alphabetic ordering
of the selector names written in lower case
(based on the ASCII ordering of characters).

Much as with sequences, a struct is immutable but may contain mutable objects;
therefore, a struct is atomic only if all its components are atomic.

In the following operation descriptions,
let st = @b[struct][n@-(1): t@-(1), @BNFdots(), n@-(k): t@-(k)].
@end(OverviewText)

@b(Operations)

@begin(OpDef)
replace@us()n@-(i) = @b(proc) (s: st, e: t@-(i)) @b(returns) (st)
@begin(itext)
@b(effects)  Returns a struct object whose components are those of @i(s)
except that component @i(n@-(i)) is @i(e).
There is a @i(replace)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
get@us()n@-(i) = @b(proc) (s: st) @b(returns) (t@-(i))
@begin(itext)
@b(effects)  Returns the component of @i(s) whose selector is @i(n@-(i)).
There is a @i(get)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
s2r = @b(proc) (s: st) @b(returns) (rt)
@begin(itext)
@b(effects)  Here @i(rt) is a record type whose components have the same
selectors and types as @i(st).  Returns a new record object
whose components are those of @i(s).
@end(itext)
@end(OpDef)

@begin(OpDef)
r2s = @b(proc) (r: rt) @b(returns) (st)
@begin(itext)
@b(effects)  Here @i(rt) is a record type whose components have the same
selectors and types as @i(st).  Returns a struct object
whose components are the corresponding components of @i(r).
@end(itext)
@end(OpDef)

@begin(OpDef)
equal = @b(proc) (s1, s2: st) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has equal: @b(proctype) (t@-(i), t@-(i)) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns @b[true] if @i(s1) and @i(s2) contain equal objects for
each component as determined by the @i(t@-(i))$@i(equal) operations.
Any @i(failure) signal is immediately resignalled.  This operation does not 
itself originate any @i(failure) signal.
The comparison is done in lexicographic order of the selectors;
if any comparison returns @b[false], @b[false] is returned immediately.
@end(itext)
@end(OpDef)

@begin(OpDef)
similar = @b(proc) (s1, s2: st) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has similar: @b(proctype) (t@-(i), t@-(i)) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns @b[true] if @i(s1) and @i(s2) contain similar objects for
each component as determined by the @i(t@-(i))$@i(similar) operations.
Any @i(failure) signal is immediately resignalled.  This operation does not 
itself originate any @i(failure) signal.
The comparison is done in lexicographic order of the selectors;
if any comparison returns @b[false], @b[false] is returned immediately.
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (s: st) @b(returns) (st) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has copy: @b(proctype) (t@-(i)) @b(returns) (t@-(i)) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns a struct containing a copy of each component
of @i(s); copies are obtained by calling the @i(t@-(i))$@i(copy) operations.
Any @i(failure) signal is immediately resignalled.  This operation does not 
itself originate any @i(failure) signal.
Copying is done in lexicographic order of the selectors.
@end(itext)
@end(OpDef)

@begin(OpDef)
@b[transmit] = @b(proc) (s: st) @b(returns) (st) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has @b[transmit]
@end(itext)
@begin(itext)
@b(effects)  Returns a struct containing a transmitted copy of each component
of @i(s).
Sharing is preserved among the components of @i(s).
Any @i(failure) signal from @i(t)@-(i)$@b[transmit] is immediately resignalled.
This operation does not itself originate any @i(failure) signal.
@end(itext)
@end(OpDef)
@end(TypeDef)


@Section(Records)
@label(record-type-def)
@index(record)
@begin(TypeDef)
@b[record] = @b(data type) [n@-(1): t@-(1), ..., n@-(k): t@-(k)] @b(is) r@us()gets@us()r, r@us()gets@us()s, set@us()n@-(1), ..., set@us()n@-(k), get@us()n@-(1), ..., get@us()n@-(k),
				equal,similar, similar1, copy, copy1, @b[transmit]

@b(Overview)

@begin(OverviewText)
A @b[record] is a mutable collection of one or more named objects.
The names are called @i(selectors), and the objects are called @i(components).
Different components may have different types.
A record also has an identity as an object.

An instantiation of @b[record] has the form:@*
@hsp(.5inch)@b[record] [ field@us()spec , @BNFdots() ]@*
where@*
@hsp(.5inch)field@us()spec @BNFequals() name, @BNFdots() : type@us()actual@*
(see Appendix @ref(apdx-syntax)).
Selectors must be unique within an instantiation (ignoring capitalization),
but the ordering and grouping of selectors is unimportant.
For example, the following name the same type:@*
@hsp(.5inch)@b[record][last, first, middle: @b[string],   age: @b[int]]@*
@hsp(.5inch)@b[record][last: @b[string],   age: @b[int],   first, middle: @b[string]]
@blankspace(1)
A record is created using a record constructor,
see Section @ref(sect-record-constructor).

For purposes of the certain operations, the the names
of the selectors are ordered lexicographically.
@index(lexicographic order)
Lexicographic ordering of the selectors is the alphabetic ordering
of the selector names written in lower case
(based on the ASCII ordering of characters).

In the following definitions of record operations, let@
rt = @b[record][n@-(1): t@-(1), ..., n@-(k): t@-(k)].
@end(OverviewText)

@b(Operations)

@begin(OpDef)
r@us()gets@us()r = @b(proc) (r1, r2: rt)
@begin(itext)
@b(modifies) @i(r1).
@end(itext)
@begin(itext)
@b(effects)  Sets each component of @i(r1) to be the corresponding 
component of @i(r2).
@end(itext)
@end(OpDef)

@begin(OpDef)
r@us()gets@us()s = @b(proc) (r: rt, s: st)
@begin(itext)
@b(modifies) @i(r).
@end(itext)
@begin(itext)
@b(effects)  Here @i(st) is a struct type whose components have the
same selectors and types as @i(rt).  Sets each component
of @i(r) to be the corresponding component of @i(s).
@end(itext)
@end(OpDef)

@begin(OpDef)
set@us()n@-(i) = @b(proc) (r: rt, e: t@-(i))
@begin(itext)
@b(modifies) @i(r).
@end(itext)
@begin(itext)
@b(effects)  Modifies @i(r) by making the component whose selector is
@i(n@-(i)) be @i(e).  There is a @i(set)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
get@us()n@-(i) = @b(proc) (r: rt) @b(returns) (t@-(i))
@begin(itext)
@b(effects)  Returns the component of @i(r) whose selector is @i(n@-(i)).
There is a @i(get)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
equal = @b(proc) (r1, r2: rt) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if @i(r1) and @i(r2) are the same record object;
otherwise returns @b[false].
@end(itext)
@end(OpDef)

@begin(OpDef)
similar = @b(proc) (r1, r2: rt) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has similar: @b(proctype) (t@-(i), t@-(i)) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns @b[true] if @i(r1) and @i(r2) contain similar objects for
each component as determined by the @i(t@-(i))$@i(similar) operations.
Any @i(failure) signal is immediately resignalled.  This operation does not 
itself originate any @i(failure) signal.
The comparison is done in lexicographic order of the selectors;
if any comparison returns @b[false], @b[false] is returned immediately.
@end(itext)
@end(OpDef)

@begin(OpDef)
similar1 = @b(proc) (r1, r2: rt) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has equal: @b(proctype) (t@-(i), t@-(i)) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns @b[true] if @i(r1) and @i(r2) contain equal objects for
each component as determined by the @i(t@-(i))$@i(equal) operations.
Any @i(failure) signal is immediately resignalled.  This operation does not 
itself originate any @i(failure) signal.
The comparison is done in lexicographic order of the selectors;
if any comparison returns @b[false], @b[false] is returned immediately.
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (r: rt) @b(returns) (rt) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has copy: @b(proctype) (t@-(i)) @b(returns) (t@-(i)) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns a new record obtained by performing @i(copy1)(@i(r))
and then replacing each component with
a copy of the corresponding component
of @i(r).
Copies are obtained by calling the @i(t@-(i))$@i(copy) operations.
Any @i(failure) signal is immediately resignalled.  This operation does not 
itself originate any @i(failure) signal.
Copying is done in lexicographic order of the selectors.
@end(itext)
@end(OpDef)

@begin(OpDef)
copy1 = @b(proc) (r: rt) @b(returns) (rt)
@begin(itext)
@b(effects)  Returns a new record containing the components
of @i(r) as its components.
@end(itext)
@end(OpDef)

@begin(OpDef)
@b[transmit] = @b(proc) (r: rt) @b(returns) (rt) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has @b[transmit]
@end(itext)
@begin(itext)
@b(effects)  Returns a new record containing a transmitted
copy of each component of @i(r).
Sharing is preserved among the components of @i(r).
Any @i(failure) signal from @i(t)@-(i)$@b[transmit] is immediately resignalled.
This operation does not itself originate any @i(failure) signal.
@end(itext)
@end(OpDef)
@end(TypeDef)

@NewPage()

@Section(Atomic Records)
@label(atomic_record-type-def)
@IndexEntry(Key="atomic_record", Entry="atomic@us<>record", Number)
@IndexSecondary(Primary="atomic", Secondary="built-in atomic types")
@begin(TypeDef)
@b[atomic@us<>record] = @b(data type) [n@-(1) : t@-(1), ..., n@-(k): t@-(k)] @b(is) ar@us()gets@us()ar, set@us()n@-(1), ..., set@us()n@-(k), get@us()n@-(1), ..., get@us()n@-(k),
				ar2r, r2ar, equal,similar, similar1, copy, copy1, @b[transmit],
				test@us()and@us()read, test@us()and@us()write, can@us()read, can@us()write, read@us()lock, write@us()lock

@b(Overview)

@begin(OverviewText)
An @b[atomic@us<>record] is a mutable atomic collection
of one or more named objects.
The names are called @i(selectors), and the objects are called @i(components).
Different components may have different types.
An atomic@us<>record also has an identity as an object.

An instantiation of @b[atomic@us()record] has the form:@*
@hsp(.5inch)@b[atomic@us()record] [ field@us()spec , @BNFdots() ]@*
where@*
@hsp(.5inch)field@us()spec @BNFequals() name, @BNFdots() : type@us()spec@*
(see Appendix @ref(apdx-syntax)).
Selectors must be unique within an instantiation (ignoring capitalization),
but the ordering and grouping of selectors is unimportant.
For example, the following name the same type:@*
@hsp(.5inch)@b[atomic@us<>record][last, first, middle: @b[string],   age: @b[int]]@*
@hsp(.5inch)@b[atomic@us<>record][last: @b[string],   age: @b[int],   first, middle: @b[string]]
@blankspace(1)
An atomic@us()record is created using a atomic@us()record constructor,
see Section @ref(sect-record-constructor).

For purposes of the certain operations, the the names
of the selectors are ordered lexicographically.
@index(lexicographic order)
Lexicographic ordering of the selectors is the alphabetic ordering
of the selector names written in lower case
(based on the ASCII ordering of characters).

Atomic@us<>records use read/write locking to achieve atomicity.
The locking rules are described in Section @ref(sect-atomic-obj).
It is an error if a process that is not in an action attempts to test or obtain
a lock;
when this happens the guardian running the process will crash.
As defined below, the only operation that (in the normal case)
does not attempt to test or obtain a lock is the @i(equal) operation.

In the following, let
art = @b[atomic@us<>record][n@-(1): t@-(1), ..., n@-(k): t@-(k)].
@end(OverviewText)

@b(Operations)

@begin(OpDef)
ar@us()gets@us()ar = @b(proc) (r1, r2: art)
@begin(itext)
@b(modifies) @i(r1).
@end(itext)
@begin(itext)
@b(effects)  Obtains a write lock on @i(r1) and a read lock on @i(r2), then
sets each component of @i(r1) to be the corresponding component of @i(r2).
@end(itext)
@end(OpDef)

@begin(OpDef)
get@us()n@-(i) = @b(proc) (r: art) @b(returns) (t@-(i))
@begin(itext)
@b(effects)  Obtains a read lock on @i(r) and 
returns the component of @i(r) whose selector is @i(n@-(i)).
There is a @i(get)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
set@us()n@-(i) = @b(proc) (r: art, e: t@-(i))
@begin(itext)
@b(modifies) @i(r).
@end(itext)
@begin(itext)
@b(effects)  Obtains a write lock on @i(r) and
modifies @i(r) by making the component whose selector is
@i(n@-(i)) be @i(e).  There is a @i(set)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
ar2r = @b(proc) (ar: art) @b(returns) (r: art)
@begin(itext)
@b(effects) Obtains a read lock on @i(ar) and 
returns a record @i(r) with the same state.
@end(itext)
@end(OpDef)

@begin(OpDef)
r2ar = @b(proc) (r: art) @b(returns) (ar: art) 
@begin(itext)
@b(effects) returns an atomic@us()record @i(ar) with the same state as @i(r).
Obtains a read lock on @i(ar).
@end(itext)
@end(OpDef)

@begin(OpDef)
equal = @b(proc) (r1, r2: art) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if @i(r1) and @i(r2) are the very same atomic@us()record 
object; otherwise returns @b[false].  No locks are obtained.
@end(itext)
@end(OpDef)

@begin(OpDef)
similar = @b(proc) (r1, r2: art) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has similar: @b(proctype) (t@-(i), t@-(i)) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Obtains a read lock on @i(r1), then a read lock on @i(r2);
then compares corresponding components from @i(r1) and @i(r2) using the
@i(t@-(i))$@i(similar) operations.
Any @i(failure) signal is immediately resignalled.  This operation does not 
itself originate any @i(failure) signal.
The comparison is done in lexicographic order of the selectors;
if any comparison returns @b[false], @b[false] is returned immediately.
If all comparisons return @b(true), returns @b(true).
@end(itext)
@end(OpDef)

@begin(OpDef)
similar1 = @b(proc) (r1, r2: art) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has equal: @b(proctype) (t@-(i), t@-(i)) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  This operation is the same as similar, except that 
@i(t@-(i))$@i(equal) 
is used instead of
@i(t@-(i))$@i(similar).
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (r: art) @b(returns) (res: art) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has copy: @b(proctype) (t@-(i)) @b(returns) (t@-(i)) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Obtains a read lock on @i(r), then 
returns a new atomic@us()record @i(res) 
obtained by performing @i(copy1)(@i(r))
and then replacing each component with
a copy of the corresponding component
of @i(r).
Copies are obtained by calling the @i(t@-(i))$@i(copy) operations.
Any @i(failure) signal is immediately resignalled.  This operation does not 
itself originate any @i(failure) signal.
Copying is done in lexicographic order of the selectors.
A read lock is also obtained on the new atomic@us()record @i(res).
@end(itext)
@end(OpDef)

@begin(OpDef)
copy1 = @b(proc) (r: art) @b(returns) (res: art)
@begin(itext)
@b(effects)  Obtains a read lock on @i(r), then 
returns a new atomic@us()record @i(res)
containing the components
of @i(r) as its components.  A read lock is also obtained on the new 
atomic@us()record @i(res).
@end(itext)
@end(OpDef)

@begin(OpDef)
@b[transmit] = @b(proc) (ar: art) @b(returns) (art) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has @b[transmit]
@end(itext)
@begin(itext)
@b(effects)  Returns a new atomic@us<>record containing a transmitted
copy of each component of @i(ar).
Sharing is preserved among the components of @i(ar).
A read lock is obtained on @i(ar) and the new atomic@us<>array.
Any @i(failure) signal from @i(t)@-(i)$@b[transmit] is immediately resignalled.
This operation does not itself originate any @i(failure) signal.
@end(itext)
@end(OpDef)

@begin(OpDef)
test@us()and@us()read = @b(proc) (ar: art) @b(returns) (@b[bool])
@begin(itext)
@b(effects) Tries to obtain a read lock on @i(ar).  If the lock is obtained,
returns @b(true); otherwise no lock is obtained and the operation returns 
@b(false).  The operation does not "wait" for a lock.  Even if the executing
action "knows" that a lock could be obtained, @b(false) may be returned.
Even if @b(false) is returned, a subsequent attempt to obtain a read lock 
might succeed without waiting.
@end(itext)
@end(OpDef)

@begin(OpDef)
test@us()and@us()write = @b(proc) (ar: art) @b(returns) (@b[bool])
@begin(itext)
@b(effects) Tries to obtain a write lock on @i(ar).  If the lock is obtained,
returns @b(true); otherwise no lock is obtained and the operation returns 
@b(false).  The operation does not "wait" for a lock.  Even if the executing
action "knows" that a lock could be obtained, @b(false) may be returned.
Even if @b(false) is returned, a subsequent attempt to obtain a write lock 
might succeed without waiting.
@end(itext)
@end(OpDef)

@begin(OpDef)
can@us()read = @b(proc) (ar: art) @b(returns) (@b[bool])
@begin(itext)
@b(effects) Returns @b(true) if a read lock could be obtained on @i(ar)
without waiting,
otherwise returns @b(false).  No lock is actually obtained.  
Even if the executing
action "knows" that a lock could be obtained, @b(false) may be returned.
Since some concurrent action may obtain or release a lock on an 
atomic@us()record
at any time, the information returned is unreliable:
even if @b(true) is returned, a subsequent attempt to obtain the lock may
require waiting; and even if @b(false) is returned, a 
subsequent attempt to obtain a read lock 
might succeed without waiting.
@end(itext)
@end(OpDef)

@begin(OpDef)
can@us()write = @b(proc) (ar: art) @b(returns) (@b[bool])
@begin(itext)
@b(effects) Returns @b(true) if a write lock could be obtained on @i(ar)
without waiting,
otherwise returns @b(false).  No lock is actually obtained.  
Even if the executing
action "knows" that a lock could be obtained, @b(false) may be returned.
Since some concurrent action may obtain or release a lock on an 
atomic@us()record
at any time, the information returned is unreliable:
even if @b(true) is returned, a subsequent attempt to obtain the lock may
require waiting; and even if @b(false) is returned, a 
subsequent attempt to obtain a write lock 
might succeed without waiting.
@end(itext)
@end(OpDef)

@begin(OpDef)
read@us()lock = @b(proc) (ar: art)
@begin(itext)
@b(effects)  Obtains a read lock on @i(ar).
@end(itext)
@end(OpDef)

@begin(OpDef)
write@us()lock = @b(proc) (ar: art)
@begin(itext)
@b(effects)  Obtains a write lock on @i(ar).
@end(itext)
@end(OpDef)
@end(TypeDef)


@Section(Oneofs)
@label(oneof-type-def)
@index(oneof)
@begin(TypeDef)
@b[oneof] = @b(data type)[n@-(1): t@-(1), ..., n@-(k): t@-(k)] @b(is) make@us()n@-(1), ..., make@us<>n@-(k), is@us()n@-(1), ..., is@us<>n@-(k), value@us()n@-(1), ..., value@us<>n@-(k),
			o2v, v2o, equal, similar, copy, @b[transmit]

@b(Overview)

@begin(OverviewText)
A oneof is a tagged, discriminated union;
that is, a labeled object, to be thought of as "one of"
a set of alternatives.
The label is called the @i(tag part),
and the object is called the @i(value) (or data part).

An instantiation of @b[oneof] has the form:@*
@hsp(.5inch)@b(oneof) [field@us()spec , @BNFdots() ]@*
where (as for records)@*
@hsp(.5inch)field@us()spec @BNFequals() name, @BNFdots() : type@us()actual@*
(see Appendix @ref(apdx-syntax)).
Tags must be unique within an instantiation (ignoring capitalization),
but the ordering and grouping of tags is unimportant.

Although there are oneof operations for decomposing oneof objects, they
are usually decomposed via the @b(tagcase) statement, which is discussed
in Section @ref(sect-tagcase).

A oneof is immutable but may contain a mutable object;
therefore, a oneof is atomic only if all of the types of its data parts are
atomic.

In the following, let
ot = @b[oneof][n@-(1): t@-(1), ..., n@-(k): t@-(k)].
@end(OverviewText)

@b(Operations)

@begin(OpDef)
make@us()n@-(i) = @b(proc) (e: t@-(i)) @b(returns) (ot)
@begin(itext)
@b(effects)  Returns a oneof object with tag @i(n@-(i)) and value @i(e).
There is a @i(make)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
is@us()n@-(i) = @b(proc) (o: ot) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if the
tag of @i(o) is @i(n@-(i)), else returns @b[false].
There is an @i(is)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
value@us()n@-(i) = @b(proc) (o: ot) @b(returns) (t@-(i)) @b(signals) (wrong@us()tag)
@begin(itext)
@b(effects)  If the tag of @i(o) is @i(n@-(i)), returns the value of @i(o); otherwise
signals @i(wrong@us()tag).
There is a @i(value)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
o2v = @b(proc) (o: ot) @b(returns) (vt)
@begin(itext)
@b(effects)  Here @i(vt) is a variant type with the same selectors and
types as @i(ot).  Returns a new variant object with the
same tag and value as @i(o).
@end(itext)
@end(OpDef)

@begin(OpDef)
v2o = @b(proc) (v: vt) @b(returns) (ot)
@begin(itext)
@b(effects)  Here @i(vt) is a variant type with the same selectors and
types as @i(ot).  Returns a oneof object with the
same tag and value as @i(v).
@end(itext)
@end(OpDef)

@begin(OpDef)
equal = @b(proc) (o1, o2: ot) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has equal: @b(proctype) (t@-(i), t@-(i)) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns @b[true] if @i(o1) and @i(o2) have the same tag and equal
values as determined by the @i(equal) operation of their data part's type.
Any @i(failure) signal is immediately resignalled.  This operation does not itself
originate any @i(failure) signal.  This operation is divisible at the
call of @i(t@-(i))$@i(equal).
@end(itext)
@end(OpDef)

@begin(OpDef)
similar = @b(proc) (o1, o2: ot) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has similar: @b(proctype) (t@-(i), t@-(i)) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns @b[true] if @i(o1) and @i(o2) have the same tag and similar
values as determined by the @i(similar) operation of their value's type.
Any @i(failure) signal is immediately resignalled.  This operation does not itself
originate any @i(failure) signal.  This operation is divisible at the 
call of @i(t@-(i))$@i(similar).
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (o: ot) @b(returns) (ot) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has copy: @b(proctype) (t@-(i)) @b(returns) (t@-(i)) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns a oneof object with the same tag as @i(o) and containing
as a value a copy of @i(o)'s value; the copy is made using
the @i(copy) operation of the value's type.
Any @i(failure) signal is immediately resignalled.  This operation does not itself
originate any @i(failure) signal.  This operation is divisible at the 
call of @i(t@-(i))$@i(copy).
@end(itext)
@end(OpDef)

@begin(OpDef)
@b[transmit] = @b(proc) (o: ot) @b(returns) (ot) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has @b[transmit]
@end(itext)
@begin(itext)
@b(effects)  Returns a oneof object with the same tag as @i(o) and containing
as a value a transmitted copy of @i(o)'s value.
Any @i(failure) signal is immediately resignalled.  This operation does not itself
originate any @i(failure) signal.
@end(itext)
@end(OpDef)
@end(TypeDef)


@Section(Variants)
@label(variant-type-def)
@index(variant)
@begin(TypeDef)
@b[variant] = @b(data type) [n@-(1): t@-(1), ..., n@-(k): t@-(k)] @b(is) make@us()n@-(1), ..., make@us<>n@-(k), change@us()n@-(1), ..., change@us<>n@-(k),
			is@us()n@-(1), ..., is@us<>n@-(k), value@us()n@-(1), ..., value@us<>n@-(k), v@us()gets@us()v, v@us()gets@us()o,
			equal, similar, similar1, copy, copy1, @b[transmit]

@b(Overview)

@begin(OverviewText)
A variant is a mutable, tagged, discriminated union.
Its state is a oneof, that is,
a labeled object, to be thought of as "one of" a set of alternatives.
The label is called the @i(tag part),
and the object is called the @i(value) (or data part).
A variant also has an identity as an object.

An instantiation of @b[variant] has the form:@*
@hsp(.5inch)@b(variant) [ field@us()spec , @BNFdots() ]@*
where@*
@hsp(.5inch)field@us()spec @BNFequals() name, @BNFdots() : type@us()actual@*
(see Appendix @ref(apdx-syntax)).
Tags must be unique within an instantiation (ignoring capitalization),
but the ordering and grouping of tags is unimportant.

Although there are variant operations for decomposing variant objects, they
are usually decomposed via the @b(tagcase) statement, which is discussed
in Section @ref(sect-tagcase).

In the following let
vt = @b[variant][n@-(1): t@-(1), ..., n@-(k): t@-(k)].
@end(OverviewText)

@b(Operations)

@begin(OpDef)
make@us()n@-(i) = @b(proc) (e: t@-(i)) @b(returns) (vt)
@begin(itext)
@b(effects)  Returns a new variant object with tag @i(n@-(i)) and value
@i(e).
There is a @i(make)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
change@us()n@-(i) = @b(proc) (v: vt, e: t@-(i))
@begin(itext)
@b(modifies) @i(v).
@end(itext)
@begin(itext)
@b(effects)  Modifies @i(v) to have tag @i(n@-(i)) and value @i(e).
There is a @i(change)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
is@us()n@-(i) = @b(proc) (v: vt) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if the tag of @i(v) is @i(n@-(i)); otherwise returns @b[false].
There is an @i(is)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
value@us()n@-(i) = @b(proc) (v: vt) @b(returns) (t@-(i)) @b(signals) (wrong@us()tag)
@begin(itext)
@b(effects)  If the tag of @i(v) is @i(n@-(i)), returns the value of @i(v);
otherwise
signals @i(wrong@us()tag).
There is a @i(value)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
v@us()gets@us()v = @b(proc) (v1, v2: vt)
@begin(itext)
@b(modifies)  @i(v1).
@end(itext)
@begin(itext)
@b(effects)  Modifies @i(v1) to contain the same tag and value as @i(v2).
@end(itext)
@end(OpDef)

@begin(OpDef)
v@us()gets@us()o = @b(proc) (v: vt, o: ot)
@begin(itext)
@b(modifies)  @i(v).
@end(itext)
@begin(itext)
@b(effects)  Here @i(ot) is the oneof type with the same
selectors and types as @i(vt).
Modifies @i(v) to contain the same tag and value as @i(o).
@end(itext)
@end(OpDef)

@begin(OpDef)
equal = @b(proc) (v1, v2: vt) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if @i(v1) and @i(v2) are the same variant object.
@end(itext)
@end(OpDef)

@begin(OpDef)
similar = @b(proc) (v1, v2: vt) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has similar: @b(proctype) (t@-(i), t@-(i)) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns @b[true] if @i(v1) and @i(v2) have the same tag and similar
values as determined by the @i(similar) operation of their value's type.
Any @i(failure) signal is immediately resignalled.  This operation does not itself
originate any @i(failure) signal.  This operation is divisible at the 
call of @i(t@-(i))$@i(similar).
@end(itext)
@end(OpDef)

@begin(OpDef)
similar1 = @b(proc) (v1, v2: vt) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has equal: @b(proctype) (t@-(i), t@-(i)) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Same as similar, except that @i(t@-(i))$@i(equal) is used instead
of @i(t@-(i))$@i(similar).
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (v: vt) @b(returns) (vt) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has copy: @b(proctype) (t@-(i)) @b(returns) (t@-(i)) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Returns a variant object with the same tag as @i(v) and containing
as a value a copy of @i(v)'s value; the copy is made using
the @i(copy) operation of the value's type.
Any @i(failure) signal is immediately resignalled.  This operation does not itself
originate any @i(failure) signal.  This operation is divisible at the 
call of @i(t@-(i))$@i(copy).
@end(itext)
@end(OpDef)

@begin(OpDef)
copy1 = @b(proc) (v: vt) @b(returns) (vt)
@begin(itext)
@b(effects)  Returns a new variant object with the same tag as @i(v) and
containing @i(v)'s value as its value.
@end(itext)
@end(OpDef)

@begin(OpDef)
@b[transmit] = @b(proc) (v: vt) @b(returns) (vt) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has @b[transmit]
@end(itext)
@begin(itext)
@b(effects)  Returns a variant object with the same tag as @i(v) and containing
as a value a transmitted copy of @i(v)'s value.
Any @i(failure) signal is immediately resignalled.  This operation does not itself
originate any @i(failure) signal.
@end(itext)
@end(OpDef)
@end(TypeDef)


@Section(Atomic Variants)
@label(atomic_variant-type-def)
@IndexEntry(Key="atomic_variant", Entry="atomic@us<>variant", Number)
@IndexSecondary(Primary="atomic", Secondary="built-in atomic types")
@begin(TypeDef)
@b[atomic@us<>variant] = @b(data type) [n@-(1): t@-(1), ..., n@-(k): t@-(k)] @b(is) make@us()n@-(1), ..., make@us<>n@-(k), change@us()n@-(1), ..., change@us<>n@-(k),
			av@us()gets@us()av, is@us()n@-(1), ..., is@us<>n@-(k), value@us()n@-(1), ..., value@us<>n@-(k), av2v, v2av, 
			equal, similar, similar1, copy, copy1, @b[transmit],
			test@us()and@us()read, test@us()and@us()write, can@us()read, can@us()write, read@us()lock, write@us()lock

@b(Overview)

@begin(OverviewText)
An atomic@us<>variant is a mutable, atomic, tagged, discriminated union.
Its state is a oneof, that is,
a labeled object, to be thought of as "one of" a set of alternatives.
The label is called the @i(tag part),
and the object is called the @i(value) (or data part).
An atomic@us<>variant also has an identity as an object.

An instantiation of @b[atomic@us<>variant] has the form:@*
@hsp(.5inch)@b(atomic@us()variant) [ field@us()spec , @BNFdots() ]@*
where@*
@hsp(.5inch)field@us()spec @BNFequals() name, @BNFdots() : type@us()actual@*
(see Appendix @ref(apdx-syntax)).
Tags must be unique within an instantiation (ignoring capitalization),
but the ordering and grouping of tags is unimportant.

Although there are atomic@us()variant operations for decomposing 
atomic@us()variant objects, they
are usually decomposed via the @b(tagtest) statement or @b(tagwait) statement, 
which are discussed
in Section @ref(sect-tagtest).

In the following, let
avt = @b[atomic@us<>variant][n@-(1): t@-(1), ..., n@-(k): t@-(k)].
@end(OverviewText)

@b(Operations)

@begin(OpDef)
make@us()n@-(i) = @b(proc) (e: t@-(i)) @b(returns) (av: avt)
@begin(itext)
@b(effects)  Returns a new atomic@us()variant object @i(av) 
with tag @i(n@-(i)) 
and value
@i(e).  Obtains a read lock on @i(av).
There is a @i(make)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
change@us()n@-(i) = @b(proc) (v: avt, e: t@-(i))
@begin(itext)
@b(modifies) @i(v).
@end(itext)
@begin(itext)
@b(effects)  Obtains a write lock on @i(v), then 
modifies @i(v) to have tag @i(n@-(i)) and value @i(e).
There is a @i(change)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
av@us()gets@us()av = @b(proc) (v1, v2: avt)
@begin(itext)
@b(modifies)  @i(v1).
@end(itext)
@begin(itext)
@b(effects)  Obtains a read lock on @i(v2) and then a write lock on @i(v1), then
modifies @i(v1) to contain the same tag and value as @i(v2).
@end(itext)
@end(OpDef)

@begin(OpDef)
is@us()n@-(i) = @b(proc) (v: avt) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Obtains a read lock on @i(v), then
returns @b[true] if the tag of @i(v) is @i(n@-(i)); otherwise returns @b[false].
There is an @i(is)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
value@us()n@-(i) = @b(proc) (v: avt) @b(returns) (t@-(i)) @b(signals) (wrong@us()tag)
@begin(itext)
@b(effects)  Obtains a read lock on @i(v).
Then, if the tag of @i(v) is @i(n@-(i)), returns the value of @i(v);
otherwise
signals @i(wrong@us()tag).
There is a @i(value)@us() operation for each selector.
@end(itext)
@end(OpDef)

@begin(OpDef)
av2v = @b(proc) (av: avt) @b(returns) (v: vt)
@begin(itext)
@b(effects) Here vt is a variant type with the same selectors and types as avt.
Obtains a read lock on @i(av) and 
returns a variant @i(v) with the same state.
@end(itext)
@end(OpDef)

@begin(OpDef)
v2av = @b(proc) (v: vt) @b(returns) (av: avt) 
@begin(itext)
@b(effects) Here vt is a variant type with the same selectors and types as avt.
Returns an atomic@us()variant @i(av) with the same state as @i(v).
Obtains a read lock on @i(av).
@end(itext)
@end(OpDef)

@begin(OpDef)
equal = @b(proc) (v1, v2: avt) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  Returns @b[true] if @i(v1) and @i(v2) are the same 
atomic@us()variant object.  No locks are obtained.
@end(itext)
@end(OpDef)

@begin(OpDef)
similar = @b(proc) (v1, v2: avt) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has similar: @b(proctype) (t@-(i), t@-(i)) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Obtains read locks on @i(v1) and @i(v2), in order, and then
compares the objects;
returns @b[true] if @i(v1) and @i(v2) have the same tag and similar
values as determined by the @i(similar) operation of their type.
Any @i(failure) signal is immediately resignalled.  This operation does not itself
originate any @i(failure) signal.  This operation is divisible at the 
call of @i(t@-(i))$@i(similar).
@end(itext)
@end(OpDef)

@begin(OpDef)
similar1 = @b(proc) (v1, v2: avt) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has equal: @b(proctype) (t@-(i), t@-(i)) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Same as similar, except that @i(t@-(i))$@i(equal) is used instead
of @i(t@-(i))$@i(similar).
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (v: avt) @b(returns) (avt) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has copy: @b(proctype) (t@-(i)) @b(returns) (t@-(i)) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Obtains a read lock on @i(v), then
returns an atomic@us()variant object with the same tag as @i(v) and containing
as a value a copy of @i(v)'s value; the copy is made using
the @i(copy) operation of the value's type.
Any @i(failure) signal is immediately resignalled.  This operation does not itself
originate any @i(failure) signal.  This operation is divisible at the 
call of @i(t@-(i))$@i(copy).  A read lock is obtained on the result.
@end(itext)
@end(OpDef)

@begin(OpDef)
copy1 = @b(proc) (v: avt) @b(returns) (avt)
@begin(itext)
@b(effects)  Obtains a read lock on @i(v), then
returns a new atomic@us()variant object with the same tag as @i(v) and
containing @i(v)'s value as its value.  A read lock is obtained on the result.
@end(itext)
@end(OpDef)

@begin(OpDef)
@b[transmit] = @b(proc) (v: avt) @b(returns) (avt) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires)  each @i(t@-(i)) has @b[transmit]
@end(itext)
@begin(itext)
@b(effects)  Returns an atomic@us<>variant object
with the same tag as @i(v) and containing
as a value a transmitted copy of @i(v)'s value.
Obtains a read lock on @i(v).
Any @i(failure) signal is immediately resignalled.  This operation does not itself
originate any @i(failure) signal.
@end(itext)
@end(OpDef)

@begin(OpDef)
test@us()and@us()read = @b(proc) (av: avt) @b(returns) (@b[bool])
@begin(itext)
@b(effects) Tries to obtain a read lock on @i(av).  If the lock is obtained,
returns @b(true); otherwise no lock is obtained and the operation returns 
@b(false).  The operation does not "wait" for a lock.  Even if the executing
action "knows" that a lock could be obtained, @b(false) may be returned.
Even if @b(false) is returned, a subsequent attempt to obtain a read lock 
might succeed without waiting.
@end(itext)
@end(OpDef)

@begin(OpDef)
test@us()and@us()write = @b(proc) (av: avt) @b(returns) (@b[bool])
@begin(itext)
@b(effects) Tries to obtain a write lock on @i(av).  If the lock is obtained,
returns @b(true); otherwise no lock is obtained and the operation returns 
@b(false).  The operation does not "wait" for a lock.  Even if the executing
action "knows" that a lock could be obtained, @b(false) may be returned.
Even if @b(false) is returned, a subsequent attempt to obtain a write lock 
might succeed without waiting.
@end(itext)
@end(OpDef)

@begin(OpDef)
can@us()read = @b(proc) (av: avt) @b(returns) (@b[bool])
@begin(itext)
@b(effects) Returns @b(true) if a read lock could be obtained on @i(av)
without waiting,
otherwise returns @b(false).  No lock is actually obtained.  
Even if the executing
action "knows" that a lock could be obtained, @b(false) may be returned.
Since some concurrent action may obtain or release a lock on an 
atomic@us()variant
at any time, the information returned is unreliable:
even if @b(true) is returned, a subsequent attempt to obtain the lock may
require waiting; and even if @b(false) is returned, a 
subsequent attempt to obtain a read lock 
might succeed without waiting.
@end(itext)
@end(OpDef)

@begin(OpDef)
can@us()write = @b(proc) (av: avt) @b(returns) (@b[bool])
@begin(itext)
@b(effects) Returns @b(true) if a write lock could be obtained on @i(av)
without waiting,
otherwise returns @b(false).  No lock is actually obtained.  
Even if the executing
action "knows" that a lock could be obtained, @b(false) may be returned.
Since some concurrent action may obtain or release a lock on an 
atomic@us()variant
at any time, the information returned is unreliable:
even if @b(true) is returned, a subsequent attempt to obtain the lock may
require waiting; and even if @b(false) is returned, a 
subsequent attempt to obtain a write lock 
might succeed without waiting.
@end(itext)
@end(OpDef)

@begin(OpDef)
read@us()lock = @b(proc) (av: avt)
@begin(itext)
@b(effects)  Obtains a read lock on @i(av).
@end(itext)
@end(OpDef)

@begin(OpDef)
write@us()lock = @b(proc) (av: avt)
@begin(itext)
@b(effects)  Obtains a write lock on @i(av).
@end(itext)
@end(OpDef)
@end(TypeDef)

@Section(Procedures and Iterators)
@label(prociter-type-def)
@index(procedure)@index(iterator)@index(proctype)@index(itertype)
@IndexSecondary(Primary="procedure", Secondary="type")
@IndexSecondary(Primary="iterator", Secondary="type")
@IndexSecondary(Primary="type", Secondary="of a procedure")
@IndexSecondary(Primary="type", Secondary="of a iterator")
@begin(TypeDef)
@b[proctype] = @b(data type) @b(is) equal, similar, copy
@b[itertype] = @b(data type) @b(is) equal, similar, copy

@b(Overview)

@begin(OverviewText)
Procedures and iterators are objects created by the Argus system.
The type specification for a procedure or iterator contains
most of the information stated in a
procedure or iterator heading;
a procedure type specification has the form:
@begin(format)
	@b(proctype) ( @BNFlbkt() type@us()spec , @BNFdots() @BNFrbkt() )  @BNFlbkt() returns @BNFrbkt() @BNFlbkt() signals @BNFrbkt()
@end(format)
and an iterator type specification has the form:
@begin(format)
	@b(itertype) ( @BNFlbkt() type@us()spec , @BNFdots() @BNFrbkt() )  @BNFlbkt() yields @BNFrbkt() @BNFlbkt() signals @BNFrbkt()
@end(format)
where
@begin(format, group)
@tabclear
@tabset(1.2inches, 1.6inches)
	returns@\@BNFequals()@\@b(returns) (type@us()spec , @BNFdots())

	yields@\@BNFequals()@\@b(yields) (type@us()spec , @BNFdots())

	signals@\@BNFequals()@\@b(signals) (exception , @BNFdots())

	exception@\@BNFequals()@\name @BNFlbkt() (type@us()spec , @BNFdots()) @BNFrbkt()
@end(format)
(see Appendix @ref(apdx-syntax)).
The first list of type specifications describes the
number, types, and order of arguments.
The @i(returns) or @i(yields) clause gives the
number, types, and order of the objects to be returned or yielded.
The @i(signals) clause lists the exceptions raised by the procedure
or iterator;
for each exception name, the number, types, and order of the
objects to be returned are also given.
All names used in a @b(signals) clause must be unique.
The ordering of exceptions is not important.
For example, both of the following type specifications
name the procedure type for @b[string]$@i(substr):
@begin(format, group)
	@b(proctype) (@b[string], @b[int], @b[int]) @b(returns) (@b[string]) @b(signals) (bounds, negative@us()size)
	@b(proctype) (@b[string], @b[int], @b[int]) @b(returns) (@b[string]) @b(signals) (negative@us()size, bounds)
@end(format)

Procedure and Iterator objects are created by compiling modules
(and by the @b(bind) expression, see Section @ref(sect-bind)).
Procedure and iterator types are not transmissible
and are considered to be immutable and atomic in normal use.
However, some uses of @b(own) data (see Section @ref(sect-owns))
in procedures and iterators can violate this assumption.

In the following operation descriptions,
@i(t) stands for a proctype or itertype.
@end(OverviewText)

@b(Operations)

@begin(OpDef)
equal = @b(proc) (x, y: t) @b(returns) (@b[bool])
similar = @b(proc) (x, y: t) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  These operations return @b[true] if and only if @i(x) and @i(y) are 
the same
implementation of the same abstraction, with the same parameters
(see Section @ref(sect-instantiations)).
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (x: t) @b(returns) (t)
@begin(itext)
@b(effects)  Returns @i(x).
@end(itext)
@end(OpDef)
@end(TypeDef)


@Section(Handlers and Creators)
@label(proccreator-type-def)
@index(handler)@index(creator)@index(handlertype)@index(creatortype)
@IndexSecondary(Primary="handler", Secondary="type")
@IndexSecondary(Primary="creator", Secondary="type")
@IndexSecondary(Primary="type", Secondary="of a creator")
@IndexSecondary(Primary="type", Secondary="of a handler")
@begin(TypeDef)
handlertype = @b(data type is) equal, similar, copy, @b[transmit]
creatortype = @b(data type is) equal, similar, copy, @b[transmit]

@b(Overview)

@begin(OverviewText)
Handlers and creators are created by the Argus system.
The type specification for a handler or creator contains
most of the information stated in a
handler or creator heading;
a handler type specification has the form:
@begin(format)
	@b(handlertype) ( @BNFlbkt() type@us()spec , @BNFdots() @BNFrbkt() ) @BNFlbkt() returns @big(] [) signals @BNFrbkt()
@end(format)
and a creator type specification has the form:
@begin(format)
	@b(creatortype) ( @BNFlbkt() type@us()spec , @BNFdots() @BNFrbkt() ) @BNFlbkt() returns @big (] [) signals @BNFrbkt()
@end(format)
where
@begin(format, group)
@tabclear
@tabset(1.2inches, 1.6inches)
	returns@\@BNFequals()@\@b(returns) (type@us()spec , @BNFdots())

	signals@\@BNFequals()@\@b(signals) (exception , @BNFdots())

	exception@\@BNFequals()@\name @BNFlbkt() (type@us()spec , @BNFdots()) @BNFrbkt()
@end(format)
(see Appendix @ref(apdx-syntax)).
The first list of type specifications describes the
number, types, and order of arguments.
The @b(returns) clause gives the
number, types, and order of the objects to be returned.
The @b(signals) clause lists the exceptions raised by the handler
or creator;
for each exception name, the number, types, and order of the
objects to be returned are also given.
All names used in a @b(signals) clause must be unique; none can be 
@i(unavailable) or @i(failure), which have a pre-defined meaning for remote
calls (see Section @ref(sect-handlercall)).
The ordering of exceptions is not important.

Creators are created by compiling modules,
and handlers are created as a side-effect of guardian creation.
Handlers and creators are transmissible
and are considered to be immutable and atomic in normal use.
Certain uses of @b(own) data in handlers can violate this assumption.

In the following operation descriptions, @i(t) stands for a 
handlertype or creatortype.
@end(OverviewText)

@b(Operations)

@begin(OpDef)
equal = @b(proc) (x, y: t) @b(returns) (@b[bool])
similar = @b(proc) (x, y: t) @b(returns) (@b[bool])
@begin(itext)
@b(effects)  These operations return @b[true] if and only if @i(x) and @i(y) are 
the same object
(see Section @ref(sect-instantiations) for an exact definition
for the case of creators in guardian generators).
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (x: t) @b(returns) (t)
@b[transmit] = @b(proc) (x: t) @b(returns) (t)
@begin(itext)
@b(effects)  Returns @i(x).
@end(itext)
@end(OpDef)
@end(TypeDef)


@Section(Anys)
@label(any-type-def)
@index(any)
@begin(TypeDef)
@b[any] = @b(data type) @b(is) create, force, is@us()type

@b(Overview)

@begin(OverviewText)
An object of type @b[any] contains a type @i(T) and an object of type @i(T).
Anys are immutable and are not transmissible.
Anys are atomic only if their contained object is atomic.
@end(OverviewText)

@b(Operations)

@begin(OpDef)
create = @b(proc)[T: @b[type]] (contents: T) @b(returns) (@b[any])
@begin(itext)
@b(effects)  Returns an any object containing @i(contents) and the type @i(T).
@end(itext)
@end(OpDef)

@begin(OpDef)
force = @b(proc)[T: @b[type]] (thing: @b[any]) @b(returns) (T) @b(signals) (wrong@us()type)
@begin(itext)
@b(effects)  If @i(thing) contains an object of a type included in type @i(T),
then that object is returned; otherwise @i(wrong@us()type) is signalled.
@end(itext)
@end(OpDef)

@begin(OpDef)
is@us()type = @b(proc)[T: @b[type]] (thing: @b[any]) @b(returns) (@b[bool]) 
@begin(itext)
@b(effects)  If @i(thing) contains an object of a type included in type @i(T),
then @b(true) is returned; otherwise, @b(false) is returned.
@end(itext)
@end(OpDef)
@end(TypeDef)


@Section(Images)
@label(image-type-def)
@index(image)
@begin(TypeDef)
@b[image] = @b(data type) @b(is) create, force, is@us()type, copy, @b[transmit]

@b(Overview)

@begin(OverviewText)
An object of type @b[image] is the @i(value) of an arbitrary transmissible
type.  See Section @ref(sect-transmit) for more details.
Images are immutable, atomic, and transmissible.
@end(OverviewText)

@b(Operations)

@begin(OpDef)
create = @b(proc)[T: @b[type]] (contents: T) @b(returns) (@b[image]) @b(signals) (failure@b[string])
@begin(itext)
@b(requires) T @b(has transmit)
@end(itext)
@begin(itext)
@b(effects) Returns an image object obtained from @i(contents)
via the @i(encode) operation of @i(T).
Resignals any @i(failure) signal raised by @i(T)'s @i(encode) operation.
@end(itext)
@end(OpDef)

@begin(OpDef)
force = @b(proc)[T: @b[type]] (thing: @b[image]) @b(returns) (T) @b(signals) (wrong@us()type, failure(@b[string]))
@begin(itext)
@b(requires) T @b(has transmit)
@end(itext)
@begin(itext)
@b(effects)  If @i(thing)
encodes an object of a type included in type @i(T), then that object is extracted
using the @i(decode) operation of @i(T) and returned.
Otherwise @i(wrong@us()type) is signalled.
Resignals any @i(failure) signal raised by @i(T)'s @i(decode) operation.
@end(itext)
@end(OpDef)

@begin(OpDef)
is@us()type = @b(proc)[T: @b[type]] (thing: @b[image]) @b(returns) (@b[bool]) 
@begin(itext)
@b(requires) T @b(has transmit)
@end(itext)
@begin(itext)
@b(effects)  If @i(thing)
encodes an object of a type included in type @i(T), then @b(true) is returned;
otherwise, @b(false) is returned.
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (thing: @b[image]) @b(returns) (@b[image]) 
@b[transmit] = @b(proc) (thing: @b[image]) @b(returns) (@b[image]) 
@begin(itext)
@b(effects)  Returns @i(thing).
@end(itext)
@end(OpDef)
@end(TypeDef)


@Section(Mutexes)
@label(mutex-type-def)
@index(mutex)
@begin(TypeDef)
@b[mutex] = @b(data type)[t: @b[type]] @b(is) create, set@us()value, get@us()value, changed, equal, similar, copy, @b[transmit]

@b(Overview)

@begin(OverviewText)
A mutex is a mutable container for an object of type @i(t).
A mutex also has an identity as an object.

An object of type @b[mutex][t] provides @i(mut)ual @i(ex)clusion for process
synchronization, and allows explicit control over how information contained
in the mutex is written to stable storage
(see Section @ref(sect-synch)).

The @b(seize) statement is used in order to gain possession of a mutex.
See section @ref(sect-mutex).

Although mutex objects are mutable,
sharing among mutex objects is usually wrong,
because the contained object should only be accessible through the mutex.
Hence there is no @i(copy1) operation, since this would introduce
sharing,
and there is no @i(similar1) operation to check for sharing
(see Section @ref(sect-mutex)).
@end(OverviewText)

@b(Operations)

@begin(OpDef)
create = @b(proc) (thing: t) @b(returns) (@b[mutex][t]) 
@begin(itext)
@b(effects)  Returns a new mutex object containing @i(thing).
@end(itext)
@end(OpDef)

@begin(OpDef)
set@us()value = @b(proc) (container: @b[mutex][t], contents: t)  
@begin(itext)
@b(modifies) @i(container).
@end(itext)
@begin(itext)
@b(effects)  Modifies @i(container) by replacing its contained object with
@i(contents).
@end(itext)
@end(OpDef)

@begin(OpDef)
get@us()value = @b(proc) (container: @b[mutex][t]) @b(returns) (t) 
@begin(itext)
@b(effects)  Returns the object contained in @i(container).
@end(itext)
@end(OpDef)

@begin(OpDef)
changed = @b(proc) (container: @b[mutex][t])  
@begin(itext)
@b(effects)  Informs the Argus system that the calling action requires the
contents of @i(container) to be copied to stable storage by the time the 
action commits, provided @i(container) is accessible from a stable variable.
It is a programming error if a process that is not running an action calls
this operations, and if this is done the guardian will crash.
@end(itext)
@end(OpDef)

@begin(OpDef)
equal = @b(proc) (m1, m2: @b[mutex][t]) @b(returns) (@b[bool]) 
@begin(itext)
@b(effects)  Returns @b(true) if and only if @i(m1) and @i(m2) are the same
object.
@end(itext)
@end(OpDef)

@begin(OpDef)
similar = @b(proc) (m1, m2: @b[mutex][t]) @b(returns) (@b[bool]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires) t has similar: @b(proctype)(t, t) @b(returns)(@b[bool]) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Seizes @i(m1), then seizes @i(m2), and calls @i(t)$@i(similar)
to determine its result; any @i(failure) signal is immediately resignalled.
Possession of both mutexes is retained until @i(t)$@i(similar) terminates.
@end(itext)
@end(OpDef)

@begin(OpDef)
copy = @b(proc) (m1: @b[mutex][t]) @b(returns) (m2: @b[mutex][t]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires) t has copy: @b(proctype)(t) @b(returns)(t) @b(signals) (failure(@b[string]))
@end(itext)
@begin(itext)
@b(effects)  Seizes @i(m1), then calls @i(t)$@i(copy)
to make a copy which it places in the new mutex object
@i(m2).
Any @i(failure) signal is immediately resignalled.
Possession of @i(m1) is retained until @i(t)$@i(copy) terminates.
@end(itext)
@end(OpDef)

@begin(OpDef)
@b[transmit] = @b(proc) (m1: @b[mutex][t]) @b(returns) (@b[mutex][t]) @b(signals) (failure(@b[string]))
@begin(itext)
@b(requires) t has @b[transmit]
@end(itext)
@begin(itext)
@b(effects)  Seizes @i(m1), and returns a new mutex containing
a transmitted copy of the contained object.
Any @i(failure) signal is immediately resignalled.
Possession of @i(m1) is retained until @i(t)$@b(transmit) terminates.
@end(itext)
@end(OpDef)
@end(TypeDef)
