@Part(changes, Root "manual.mss")

@Appendix(Changes from CLU)
@Label(apdx-diffs)

This appendix lists the changes made to Argus that are not upward
compatible with CLU,
@index(CLU)@indexsecondary(primary="CLU", Secondary="differences from")
that is, those which are not merely additions to CLU
and that would cause a CLU program to be illegal or to run differently.


@Section(Exception Handling)

Unlike CLU,
@IndexSecondary(Primary="exception", Secondary="versus CLU")
@IndexSecondary(Primary="failure", Secondary="versus CLU")
which propagated unhandled exceptions
(by turning them into @i(failure) exceptions)
and gave the @i(failure) exception special status,
unhandled exceptions in Argus are considered errors and
always cause a crash of the guardian,
@IndexSecondary(Primary="unhandled exception", Secondary="versus CLU")
and @i(failure) is not given special status.
All exceptions signalled in a procedure, iterator, handler, or creator
must be declared in the routine's header,
and there are no implicit resignals of @i(failure) exceptions.
See Section @ref(sect-failure-exceptions) for details.


@Section(Type Any)

The type @b[any] is now a type like any other type,
@IndexSecondary(Primary="any", Secondary="versus CLU")
with parameterized routines @i(force),
@i(create), and @i(is@us<>type).
Thus the CLU manual's notion of "type inclusion" is no longer necessary
(but there is a new notion of type inclusion in Argus, see Section @ref(sect-tinclusion)).
The @b[any]$force routine only signals "wrong@us<>type" if the @b[any]
object's underlying type is not @i(included) in the type parameter given,
but the type of the result of @b[any]$force is its type parameter.
The @b[any]$is@us<>type routine returns @b[false]
if the @b[any] object's underlying type is not @i(included) in the type
parameter given.
The CLU reserved word "force" was eliminated from Argus,
and the creation of an @b[any] object is never implicit in an assignment in Argus.


@Section(Built-in Types)

Several changes to the interfaces of the built-in types were necessitated
by the changes to exception handling.
@IndexSecondary(Primary="built-in type", Secondary="versus CLU")
Specifically, the following changes were made to the built-in types.
@begin(enumerate)
The @b[string] operations @i(concat), @i(append), @i(s2ac), @i(ac2s),
@i(s2sc), and @i(sc2s),
can now all signal @i(limits).
A string literal that would be too large to represent will not be compiled.

The @b[sequence] operations
@i(fill), @i(fill@us<>copy), @i(addh), @i(addl), and @i(concat)
can now all signal @i(limits).
A sequence constructor that would be too large to represent
will not be compiled.

The @b[array] (and @b[atomic@us<>array]) operations
@i(create), @i(predict), @i(set@us<>low),
@i(fill), @i(fill@us<>copy), @i(addh),
and @i(addl)
can now all signal @i(limits).
An array constructor that cannot be legally represented
will either not be compiled (if this can be detected at compile time)
or will signal limits.

The @i(copy) operations of the structured built-in type generators,
and the @i(fill@us<>copy) operations
of @b[sequence] and @b[array] (and @b[atomic@us<>array]),
allow the @i(copy) operations of their type parameters
to have a @i(failure)(@b[string]) exception.
They will resignal such a @i(failure) exception.
(Note that the type inclusion rule allows a type parameter to be used
even if its @i(copy) operation does not have exceptions.)

The @i(similar) operations of the built-in structured type generators
allow the @i(similar) operations of their type parameters
to have a @i(failure)(@b[string]) exception.
They will resignal such a @i(failure) exception.

The @i(equal) operations of the type generators @b[sequence],
@b[struct], and @b[oneof], and the @i(similar1) operations of
the type generators @b[array], @b[record], and @b[variant]
(and their atomic counterparts),
allow the @i(equal) operation of their type parameters
to have a @i(failure)(@b[string]) exception.
They will resignal such a @i(failure) exception.

The @i(elements) iterator and the @i(similar) and @i(similar1) procedures
of the type generator @b[array]
(and @b[atomic@us<>array]) will raise a @i(failure)(@b[string]) exception
if the array argument
is mutated in such a way as to cause a bounds exception
when an element is fetched.
@end(enumerate)


@Section(Type Inclusion)

Type inclusion (the new notion, see Section @ref(sect-tinclusion))
@IndexSecondary(Primary="type inclusion", Secondary="versus CLU")
is used in all contexts,
including the @i(decls) of @b[except] and @b[tagcase] statements,
where CLU had previously required type equality.


@Section(Where Clauses)

CLU had syntax in the @i(where) clause
(specifically the production for @i(op@us<>name))
that allowed one to require an instantiation
of a type parameter's generator.
@index(where clause)@index(instantiation)
This little used feature has been superseded by the mechanism described
in Section @ref(sect-instantiations).


@Section(Uninitialized Variables)

An uninitialized variable reference error is defined to cause a crash of the
@IndexSecondary(Primary="uninitialized variable", Secondary="versus CLU")
guardian,
rather than raising a @i(failure) exception,
which could conceivably be caught.


@Section(Lexical Changes)

Several new reserved words were added.
In addition, the semicolon (;) was banished from the syntax.


@Section(Input/Output Changes)

The input/output data types (file@us<>name, stream, and istream)
@index(input/output)
@IndexSecondary(Primary="input/output", Secondary="versus CLU")
and the library procedures
described in appendix III of the CLU manual
are not furnished by the Argus system.
Our current implementation of Argus provides
a @i(keyboard)
@index(keyboard)
cluster for input and a @i(pstream)
@index(pstream)
cluster for output.
In addition, most of the built-in types currently have @i(print)
@index(print)
operations
defined, for pretty-printing objects onto pstreams.  These I/O mechanisms,
however, are still experimental, and so are not documented in this
reference manual.
