.so insert
.sr sect_typecheck Section`6.1
.sr sect_transmit Chapter`12
.sr sect_equmod Section`14.4
.sr sect_param Section`14.5
.chapter "Environment"
.para
The Argus environment, like the CLU environment, ensures
complete static type checking of programs.
It also supports independent configuration and linking of guardians,
sensible transmission, and the evolution of abstractions.
These are discussed below.
.
.section "The Library"
.para
Argus modules are compiled in the context of a library that gives
meaning to external identifiers and allows inter-module type checking.
The Argus library contains type information about abstractions;
for each abstraction, the library contains a
i(description unit), or DU, describing that abstraction
and its implementations.
The types used in type checking are either DUs or instances
of parameterized DUs. (See sect_typecheck for the type checking rules.)
.
.subsection "Independence of Guardian Images"
.para
The code run by a guardian comes from some guardian image.
A guardian image contains
all the code needed to carry out any local activity of the guardian;
any procedure, iterator or cluster used by that guardian will be in its
guardian image.
Any handler calls made by the guardian, however, are carried out at the
called guardian; the called guardian's guardian image contains
the code necessary to execute the handler call.
This independence of the guardian images of different guardians means that
the implementation of one guardian can be changed without affecting the
implementations of its clients.
.
.section "Guardian Creation"
.para
When a guardian is created, it is necessary to select the load
image that will supply the code run by the new guardian.
To this end, each guardian has an associated i(creation environment),
that specifies the load images for other guardians it may create.
The creation environment is a mapping from guardian types to some information
that can be used to select a load image appropriate for each kind of node.
More flexibility can be had by associating this information with particular
creator objects (creators of a given guardian type are objects).
.
.section "The Catalog"
.para
Another question concerning guardians is how do guardians get
to know about one another?
A guardian usually knows about any guardian it creates.
Also, once a guardian knows about some other guardian
it can learn about more guardians, because
guardians can be passed in remote calls.
However these two methods are inadequate for finding out about
guardians whose existence is independent of the using guardian.
To solve this problem, Argus provides a built-in subsystem known by
all guardians.
This subsystem is called the i(catalog).
The catalog
provides an atomic mapping from names to transmissible objects.
For example, when a new guardian is created, it can be cataloged under
some well-known name, so that other guardians can find it in the future.
We are currently experimenting with various interfaces to the catalog.
