@Device[Dover]
@Make[Manual]
@Style[Font NHelvetica11, Spacing 1]
@LibraryFile[Symbols]
@Comment{ An underscore (sigh)}
@TextForm<us={@ux[@ ]}>
@TextForm(DebugPrompt={3>@ })
@Comment{ Various unspread lists }
@Define[UnNumList=Enumerate, Numbered <>, Spread 1]
@Modify[Description, Spread 1]
@Modify[Enumerate, Spread 1]
@Modify[Itemize, Spread 1]
@Comment{ Program examples in normal type face }
@Define[CluProg=Example, Font BodyFont, FaceCode R, BlankLines Hinge]
@Define[SmallCluProg=Example, Font SmallBodyFont, FaceCode R, BlankLines Hinge]
@Comment{ Synonyms for title fonts }
@Define[Big, Font TitleFont3, FaceCode R]
@Define[Huge, Font TitleFont5, FaceCode R]
@Begin(TitlePage)
@Big[User's Manual for]

@Huge[The Argus System]

including

@Big[LIBBY, ARGUS, ALINK,]

and mainly the

@Big[DEBUGGER]







@Big[DRAFT]


@Center[@Value<RootFileDate>]


@b(Gary T. Leavens)
@End(TitlePage)
@PrefaceSection(Preface)

This manual describes the Argus system,
its library utility, compiler, linker, and debugger.
It concentrates on the commands available in the debugger.

Unlike the debuggers of the early era of computers,
the Argus debugger does not allow one to patch object code
(and thus literally "de-bug" it).
However, the debugger does assist the user in performing two essential tasks.
@begin(enumerate, spread=1)
Testing, by aiding the interactive development and running of test cases.

Locating bugs, by helping the user pinpoint the cause or causes of errors.
@end(enumerate)
Of course, the debugger is not able to test or locate bugs on its own,
it can only help its user test and locate bugs.
That is, you must supply the brainpower and creativity to
@begin(enumerate, spread=1)
design tests that will uncover bugs, and

locate the true cause or causes of any bugs uncovered,
based on the clues the debugger turns up.
@end(enumerate)
These are not easy tasks, but the debugger's features are designed to make them
less tedious.

This manual describes the debugger as of @value[RootFileDate].
This is not the ultimate version of the debugger,
and even the features described may not be completely implemented at this time.
The features described will be implemented as soon as is practical.

The Argus language is the creation of Barbara Liskov and her group at the
Massachusetts Institute of Technology.
The Argus system is primarily the responsibility of Paul Johnson and Robert
Scheifler, who have also implemented most of it.
There are several other people who have had a hand in the software
that makes up the Argus system, including Barbara Liskov and her group members
since the days of CLU.

The remainder of this manual is divided as follows.
Chapter one introduces you to the various parts of the Argus system,
the library utility, compiler, linker, and debugger,
and describes how to run each of them and how they interact.
Chapter two describes various details of the debugger's input and output.
The rest of the chapters describe the details of the debugger itself.
Chapter three describes the features of the debugger which support testing.
Chapter four describes the features of the debugger which help you locate the causes
of bugs.
The appendix contains a summary of the command syntaxes for the tools described
in chapter one.

@Chapter(The Argus System)

@String(ArgusDir="/usr/argus")
@String(ArgusBin="/usr/argus/exe")
@String(ArgusLib="/usr/argus/lib@.lb")
@String(ArgusLibSrc="/usr/argus/lib")

The Argus system lives in @value(ArgusDir).
@Index(Argus system)
In @value(ArgusDir) are several directories and files,
the most important of which are the following.
@begin(Itemize)
@value(ArgusBin), contains the executable files for the utility programs,
including the compiler, linker, and debugger.

@value(ArgusLibSrc), contains the source code for the low-level routines and
utilities in the system library @value(ArgusLib).

@value(ArgusLib), a library containing the run-time system and several
low-level and utility routines and clusters.
@end(Itemize)

A note about the Argus system on UNIX@+(@c(tm)).
@Index(UNIX)
The programs of the Argus system are not case-sensitive,
that is, you may type commands and data in upper or lower case
and the system does not care.
However, UNIX itself is quite fussy about case, and sees the world through
lower-case glasses.
Thus, since the compiler, for instance, is named "argus",
if you type
@begin(example)
Argus
@end(example)
you will get a response from UNIX such as
@begin(example)
Argus: Command not found.
@end(example)
since, to UNIX "argus" and "Argus" are quite different.
Because of this, the Argus system must make case distinctions count in file
names.  Thus if you have a file named "foo.arg" and you type it as "FOO.ARG"
to the compiler, it will be unable to find "foo.arg".

@Section(The Library Utility)

@Index(libby)@Index(library utility)@Index(libby, running it)
The library utility is called "libby", and its executable can be found in
@value(ArgusBin).
(Frequent users will find it convenient to put @value(ArgusBin) in their
shell's search path.)
It works with binary libraries containing type checking information
(not object files) having the suffix "lb".
Libraries help the compiler and user by providing
a repository for complete type-checking
information which can last between compilations.

@Index(library)
The Argus system works with two libraries at any one time:
the system library (@value(ArgusLib)), and a user-specified library.
You may have several libraries, but only one may be used by the compiler
during any particular compilation.
"libby" assists you in maintaining your own user libraries.

"libby" may be invoked interactively by giving it no command line
arguments, or non-interactively by giving it command line options
(each preceded by a sharp sign "#").
For example, if the user types to the shell the following
@begin(example)
libby #create lib.lb 17
@end(example)
the library file "lib@.lb" will be created with a hash size of 17
(if it does not already exist).
"libby" may also be invoked interactively, in which case it prompts with
the string "libby>@ ".
Commands may be abbreviated as long as no ambiguity arises,
suffixes may be omitted from library file names
as long as the default of "lb" is correct.
"libby" responds with a help message explaining the various commands to the
commands "?" or "help",
and case distinctions are ignored (except in file names).
Thus the following might be a record of a terminal session
accomplishing the same effect as the previous example
@begin(example)
% libby
libby> c lib 17
libby> q
%
@end(example)

@Section(The Compiler)

@Index(argus)@Index(compiler)@Index(argus, running it)
The compiler is called "argus", and its executable can be found in
@value(ArgusBin).
It produces binary files with the default suffix "obj".
The default suffix for Argus source files is "arg".
Some documentation for "argus" may be found in @value(ArgusDir)/argus@.doc.

The compiler may be invoked interactively by giving it no command line
arguments, or non-interactively by giving it command line options
(each preceded by a sharp sign "#").
For example, if the user types to the shell the following
@begin(example)
argus #library lib.lb #compile myfile.arg
@end(example)
the file "myfile@.arg" will be compiled using the given library,
and the user will see a response like the following.
@begin(example)
Compiling /usr/gtl/myfile.arg
Done in 1.683
@end(example)
(The library "lib.lb" must already have been created with "libby".)
The compiler may also be invoked interactively, in which case it prompts with
the string "argus>@ ".
Commands may be abbreviated as long as no ambiguity arises,
suffixes may be omitted from file names as long as the defaults are correct.
"argus" responds with a help message explaining the various commands to the
commands "?" or "help",
and case distinctions are ignored (except in file names).
Thus the following might be a record of a terminal session
accomplishing the same effect as the previous example
@begin(example)
% argus
argus> l lib
argus> co myfile
Compiling /usr/gtl/myfile.arg
Done in 1.683
argus> q
%
@end(example)

Besides compilation, "argus" can do various kinds of checking as well as
interact with a library.
There are commands to create description units (DUs)@Index(DU) so that circular
dependencies may be entered into the library (the "type" command),
and commands to enter specifications into a library (the "spec" command).
By default, the specifications in a library may not change.

@Section(The Linker)

@Index(alink)@Index(linker)@Index(alink, running it)
The linker is called "alink", and its executable can be found in
@value(ArgusBin).
It works with binary object files having the default suffix "obj".
"alink" links several object files together into an executable program
with a default name of "a.out".
When execution begins, the procedure "start@us<>up" is called first,
and must therefore be in one of the object files linked.
"alink" will complain if the object files it is given to link are
inconsistently compiled.

"alink" may be invoked interactively by giving it no command line
arguments, or non-interactively by giving it command line arguments
(with options each preceded by a sharp sign "#").
For example, if the user types to the shell the following
@begin(example)
alink start@us<>up.obj hello.obj ^ hello
@end(example)
the program file "hello" will be created from the object files given.
"alink" may also be invoked interactively.
Suffixes may be omitted from object file names
as long as the default of "obj" is correct.
"alink" responds with a help message explaining the various commands to the
commands "?" or "help",
and case distinctions are ignored (except in file names).
Thus the following might be a record of a terminal session
accomplishing the same effect as the previous example.
@begin(example)
% alink
JCL is: Object@us<>File {<sep>Object@us<>File} {#option}  [^output@us<>file]
    or: ?
 where: Object@us<>File is either the name of a binary file
        or "@@X@us<>Load@us<>file" and <sep> is ",", space, or tab.
JCL = start@us<>up hello ^ hello
JCL = ^D%
@end(example)
The interactively invoked linker is exited by typing an end-of-file character
(control-D, written as ^D), or an interrupt character (often ^C)
or quit character (often ^G), which kills it.

@Section(The Debugger)

@Index(debugger)@Index(the debugger, running it)
The dbugger can be linked into a program
by specifying the "#debug" option to "alink".
Once the object modules are linked by "alink",
invoking the program ("a.out" file) starts up the debugger's listen loop.
For example, to debug start@us<>up and hello, one types
@begin(example)
% alink start@us<>up.obj hello.obj #debug ^ hello
% hello
@DebugPrompt() 
@end(example)
which shows the linking and
loading of the debugger and the user's program
with the debugger's prompt of "@DebugPrompt()@ ".
The debug prompt includes the process number of the process being debugged,
in this case it is 3.
The debugger's commands may be abbreviated with prefixes
as long as no ambiguity arises.
The debugger responds with a help message explaining the various commands to the
commands "?" or "help",
and case distinctions are ignored (except in file names).
The debugger is exited by the "quit" command (and not by an end-of-file).
The debugger reads commands from the terminal,
and writes its output to the terminal,
including errors and prompts.
It repeatedly prompts for a command and then carries out its instructions.

The details of the debugger's commands are explained in following chapters.
The appendix contains a collected syntax for easy reference.

@Chapter(Input and Output)

Examples of computer type-in and type-out are presented as follows.
@begin(example)
@DebugPrompt() something-you-type
@end(example)
This shows the debugger typing its prompt "@DebugPrompt()@ " and the user typing
"something-you-type" followed by a carriage return ("return" key).
The UNIX prompt is shown as a percent sign "%@ ", as in the following.
@begin(example)
% argus
argus> 
@end(example)

@Section(The Listen Loop)

@Index(listen loop)
The cycle of prompting, reading a command, carrying it out, and printing
any results is called the "listen loop".
The prompt for the listen loop is "@DebugPrompt()@ ".
(In some versions this prompt may be different.)
There may be several nested listen loops at any one time.

Within a listen loop the following special characters are recognized.
@begin(itemize, spread=1)
@Index(control characters)
^U, (control-U) which deletes the current line,

^W, which deletes the last word on a line,

^R, which redisplays the current line,

^X, which starts a new blank line,

^L, which clears the screen and redisplays the current line, and

^\, which continues the current line.
@end(itemize)

@Index(line continuation)
The end of a line ends the command and starts the evaluation process,
unless the line was continued with ^\.

Evaluation may be stopped with the ^G character.

Case distinctions are ignored except in file names.
File names may not contain blanks or commas in them.
Spaces, tabs, and commas may generally be used interchangeably as separators.
@Index(separators)

Commands may be abbreviated by prefixes when it is unambiguous to do so.
For example,
@begin(example)
@DebugPrompt() evaluate/time f()
@end(example)
and
@begin(example)
@DebugPrompt() e/t f()
@end(example)
both specify the same command.
The slash character and at least the first character of an option
must always appear if desired.
For example,
@begin(example)
@DebugPrompt() e/ f()
eh?
@end(example)
is illegal, and
@begin(example)
@DebugPrompt() e f()
@end(example)
is the same as
@begin(example)
@DebugPrompt() evaluate f()
@end(example)
but not the same as
@begin(example)
@DebugPrompt() evaluate/time f()
@end(example)

Dashes (-) which appear in command names may also be used in abbreviations,
for example,
@begin(example)
@DebugPrompt() t-e overflow
@end(example)
is the same as the following.
@begin(example)
@DebugPrompt() trace-exception overflow
@end(example)
However, note that
@begin(example)
@DebugPrompt() t hello
@end(example)
is the same as the following.
@begin(example)
@DebugPrompt() trace hello
@end(example)

Commands will always be written out in full in the examples.

A blank (or tab) must always separate the command name from its arguments.
For example,
@begin(example)
@DebugPrompt() evaluate/timef()
eh?
@end(example)
is illegal (since there is no comand named "evaluate/timef()").

Arguments may be separated from each other by either blanks, tabs, or commas.

If a command has options, several options may be combined into one command,
provided that they do not conflict.
For instance,
@begin(example)
@DebugPrompt() trace/returns/signals foo
@end(example)
is the same as the following two commands.
@begin(example)
@DebugPrompt() trace/returns foo
@DebugPrompt() trace/signals foo
@end(example)
But some options make no sense together,
for instance,
@begin(example)
@DebugPrompt() printing/flat/noflat
@end(example)
makes no sense, since the options "flat" and "noflat" are mutually exclusive.

@SubSection(The Help Command)

@Index(help command)@Index(?)
The "help" command and the "?" both
print out a terse listing of the the debugger's commands when invoked with no
arguments.
When given a list of command names, a short description of each is printed.
The only difference is that the "?" command, when invoked without arguments,
prints a list of the command names in tabular form, as opposed to the long
listing produced by the "help" command when it is invoked with no arguments.

@SubSection(The Quit Command)

@Index(quit command)
The "quit" command terminates the debugging session,
and kills the debugger, as shown in the following example.
@begin(example)
@DebugPrompt() quit
%
@end(example)

@SubSection(The Defaults Command)

@Index(defaults command)
The command "defaults" without any arguments, prints information about the
defaults currently in effect for various commands.
For example,
@begin(example)
@DebugPrompt() defaults
CURRENT	CURRENT/SHORT
EVALUATE	EVALUATE
FRAMES	FRAMES/SHORT
LOADED	LOADED
PRINTING	PRINTING
TRACE	TRACE
UNTRACE	UNTRACE
@end(example)
shows the default options for the various commands that have options,
the values shown are those used when the debugger is started.
Some of the commands do not have options currently set, for instance,
the default for "evaluate" is "evaluate" and not "evaluate/time".

When given one argument, a command name,
the "defaults" command prints the defaults for that command, if any.
For example,
@begin(example)
@DebugPrompt() defaults frames
FRAMES	FRAMES/SHORT
@end(example)
shows that the current default for the "frames" command is "frames/short".

When given a command name and a list of options,
the "defaults" command sets the options for the given command to default
to the given options.
For example,
@begin(example)
@DebugPrompt() defaults trace calls exits
TRACE	TRACE/CALLS/EXITS
@end(example)
sets the default options for the trace command to be "calls" and "exits".
With these defaults, typing
@begin(example)
@DebugPrompt() trace foo
@end(example)
is equivalent to typing the following
@begin(example)
@DebugPrompt() trace/calls/exits foo
@end(example)
But when one types
@begin(example)
@DebugPrompt() trace/returns foo
@end(example)
or any other defaulted command with specific options listed,
those options listed override the defaults.

The options in the options list may be separated by slashes as well,
thus,
@begin(example)
@DebugPrompt() defaults trace /calls/exits
@end(example)
is perfectly ok.

Use the psuedo-option "default" to reset things to the way they are
when the debugger starts up by default.
For example,
@begin(example)
@DebugPrompt() defaults current default
@end(example)
sets the "current" command to print in short format (as it does by default
when the debugger is first started up).

@Section(Matching)

@Index(matching)@Index(*)
The asterisk character (*) may be used as a wildcard in the names of routines
and clusters.  It matches zero or more characters.
It is not meaningful for variable names or command names.
For example,
@begin(example)
a*ay
@end(example)
matches "array",
and
@begin(example)
*p*m*$s*pth*
@end(example)
matches "pstream$set@us<>max@us<>depth".

@Section(Object Printing)

@Index(printing)@Index(object printing)@Index(pstream)
The format of object printing may be controlled by the
"printing", "printing-depth", and "printing-width"
commands.

Note that the type of an object is not known at runtime, and thus printing of
some objects can only be best guesses.  Objects of user-defined types print
according to their representation.  Objects of built-in types and type
generators print as follows:

@begin(Display)@TabSet(2in)
@b(null)@\@b(nil) prints as 0
@b(bool)@\@b(true) prints as 1, @b(false) as 0
@b(char)@\print as small integers
@b(int)@\print in decimal
@b(real)@\print as integers
@b(string)@\usual form, with escape sequences
@b(array)@\[low: element-list]   or   [low..high: element-list]
@b(sequence)@\[element-list]
@b(record)@\[component-list]
@b(struct)@\[component-list]
@b(variant)@\<integer-tag: value>
@b(oneof)@\<integer-tag: value>
@b(proctype)@\string name without parameters
@b(itertype)@\string name without parameters
@end(Display)

Uninitialized variables print as "???".

For records and structs, component names are not printed.  The components are
reordered in increasing lexicographic order based on their component names.
For variants and oneofs, the tags are numbered based on increasing
lexicographic order of their names.

When the width or depth of printing is  exceeded,  this  is  indicated  in  the
output with "...".
@Index(...)

@SubSection(The Printing Command)

@Index(printing command)
The "printing" command with no options displays the current
object printing formats.
For example,
@begin(example)
@DebugPrompt() printing
NOFLAT
WRAP
DEPTH	4
WIDTH	4
@end(example)
shows the formats in effect when the debugger is started up.

With options given, it changes the default formats for object printing
according to the options given.
The available options are as follows.
@begin(Description)
flat@\Makes object printing be all on one line (flat).

noflat@\Makes object printing be on multiple
lines with reasonable indentation (not flat).

wrap@\Makes object printing wrap at the end of the screen.

nowrap@\Makes object printing not wrap at the end of the screen.
@end(Description)

For example,
@begin(example)
@DebugPrompt() printing/flat
PRINTING/FLAT
@end(example)
makes object printing be all on one line by default.

@SubSection(The Printing-Width Command)

@Index(printing-width command)
To set the number of objects printed at any given depth in object printing,
use the "printing-width" command, which takes a single integer argument.
For example,
@begin(example)
@DebugPrompt() printing-width 5
PRINTING-WIDTH	5
@end(example)
sets the maximum number of objects printed at any given depth to 5.

@SubSection(The Printing-Depth Command)

@Index(printing-depth command)
To set the depth of nesting to which objects are printed,
use the "printing-depth" command, which takes one integer argument.
For example,
@begin(example)
@DebugPrompt() printing-depth 1
PRINTING-DEPTH	1
@end(example)
sets the maximum depth of nesting to which objects are printed to 1.

@Chapter(Testing)

@Index(testing mottos)
@begin(FlushRight)
@begin(i)
 Testing is the process of executing a program
with the intent of finding errors.

 A good test case is one that has a high probability of
detecting an as-yet undiscovered error.

 A successful test case is one that detects an
as-yet undiscovered error.

@Dash()G. J. Myers
@end(i)
@end(FlushRight)

@*

@*

@Index(testing)
There are two parts to testing any program
@begin(enumerate, spread=1)
making sure the program does what it is supposed to do, and

making sure the program does not do what it is not supposed to do.
@end(enumerate)
In general it is usually impossible to prove that a program has no bugs by
testing, since the scope of possible errors is so broad.
Thus one must concentrate on finding an economical set of test cases that have
a good chance of detecting a program's bugs.
Testing is a demanding, creative task.

The debugger does not relieve the user of the tasks of planning and thinking
creatively when developing tests.
However, the debugger does provide some clerical assistance,
which comes in two parts.
@begin(enumerate, spread=1)
Interactive evaluation, provided by the "evaluate" command, and

Scripting, provided by the "script" and "unscript" commands.
@end(enumerate)

In addition there are some other commands that aid testing,
which are also discussed in this chapter.

@Section(The Evaluate Command)

@Index(evaluate command)
The "evaluate" command takes one argument, an expression, which it evaluates.
After the expression has been fully evaluated,
the debugger prints the results (if any) of the expression.
During the evaluation, if any exceptional events occur (such as a crash
or a breakpoint), the user is notified (it may be that a new listen loop is
entered when this happens).
@Label(EvaluateTime)
If the "time" option is given, the CPU time used is noted.
An example is the following.
@begin(example)
@DebugPrompt() evaluate 5
5
@DebugPrompt() evaluate int$add(2, 3)
5
@DebugPrompt() evaluate int$div(5, 0)
signals zero@us<>divide
@DebugPrompt() evaluate int$lt(5, 3)
0
@DebugPrompt() evaluate int$div(10, int$add(2, 3))
2
@DebugPrompt() evaluate int$from@us<>to(0, 2)
1:	0
2:	1
3:	2
@DebugPrompt() evaluate ai = array[int]
array[int]
@DebugPrompt() evaluate xai = ai$[1, 2]
[1: 1, 2]
@DebugPrompt() evaluate xai
[1: 1, 2]
@DebugPrompt() evaluate xai := ai$[2]
[1: 2]
@DebugPrompt() evaluate ai$equal(xai, xai)
1
@end(example)

The following demonstrate timing.
@begin(example)
@DebugPrompt() evaluate/time int$add(2, 3)
5

0.167 seconds
@DebugPrompt() evaluate/time int$div(5, 0)
signals zero@us<>divide

0.184 seconds
@end(example)

To be useful in testing program performance,
there should be no breakpoints or tracing set when timing.

@SubSection(Current Frame and Process)

@Index(current frame)@Index(current module)@Index(current process)
The debugger maintains a current frame or module and a current process for the user.
When debugging begins, there is only one process (aside from the debugger itself),
and this process is the current process.
There is always either a current frame or a current module,
the phrase "current module" is used
when there is no stack frame which is "current",
although when there is a current frame, the routine associated with
that frame may also be considered "current".
When debugging begins, the stack of the current process is empty,
so there is no current frame, and thus there is a current module,
which is normally "start@us<>up"
(but can be changed with the "main" option of "alink").
These defaults remain in effect until either
@begin(enumerate, spread=1)
the user changes the current frame and/or process with the "current" command,
or

a breakpoint is taken (see below),
in which case the current frame and process become the frame and process
where the breakpoint occurred.
@end(enumerate)

@SubSection(Expressions)

@Index(expression)
The expressions used as arguments to evaluate may be almost any Argus expression,
plus equates and assignment statements which assign to variables,
but the entire syntax is not supported.
In particular, the prefix and infix operators (+, -, *, /, **, //, etc.)
are not supported, as the debugger has no ideas about types.
(Debugging session variables may refer to types, just as equates can in Argus.)

Identifiers used in expressions
are resolved in one of two ways: a context may be explicitly given,
or dynamic scoping is used from the current frame or module.
To explicitly give a context to an identifier,
the form:  "idn [ ! context [ ! process ] ]"
may be used when an identifier (idn) is legal in an expression.
(See section @ref(PointsOfReference) for definitions of process and context
and how they are specified.)
If the explicit context for an identifier is given,
then identifier is expected to be a variable visible from that context
(in the current or given process).
If no context or process is given, then an identifier is sought
(1) in the debugging session variables, (2) in the names of modules
loaded, (3) in the current frame or module's variables, and (4)
in the variables of the frames earlier on the stack
(if the current context is a frame).

If an identifier being assigned a value is not explicitly given
a context, and is not found in either the debugging session variables
or the variables of the some stack frame or module,
it defines a new debugging session variable, which is assigned the value.
Such definitions may be removed by the "undefine" command.

@Section(Scripting)

@Index(scripting)
Throw-away test cases are only for throw-away programs.
To save you the trouble of remembering a set of debugging commands,
the debugger has "scripting" which records your commands and their results.
It is also possible to tell the debugger to read input from a file
(via the "read" command)
of recorded input so that test cases may be rerun.

@SubSection(The Script Command)

@Index(script command)
If arguments are given,
then the "script" command displays the current script settings.
For example,
@begin(example)
@DebugPrompt() script
output record files:
	/usr/gtl/test.result
input record files:
	/usr/gtl/test.in
@end(example)

The "script" command optionally takes one or two arguments, both file names.
These files will be created if necessary.
The files are (in argument order)
@begin(enumerate, spread=1)
the output record file, and

the input record file.
@end(enumerate)
The output record file records all output sent to
the terminal by the debugger, and
the input record file records all input received by the debugger
(from the terminal).
The output includes the input echoing and all of the debugger's output.
If the input record file name is not specified,
there is no input record file scripting.
If either of these files exists, the record is @i(appended) to its end.
If scripting is already in effect, then the "script" command adds the new
files to the list of script files.

For example,
@begin(example)
@DebugPrompt() script test.result
@end(example)
sends an output record to "test.result",
and there is no input record scripting.
@begin(example)
@DebugPrompt() script res2 /usr/gtl/cmd2
@end(example)
sends the output record to "res2" and the input record to "/usr/gtl/cmd2".

@SubSection(The Unscript Command)

@Index(unscript command)
The "unscript" command turns off all scripting.  It takes no arguments.
For example,
@begin(example)
@DebugPrompt() unscript
@end(example)
turns off all scripting.

@SubSection(The Read Command)

@Index(read command)
The "read" command directs the debugger to read its input from
the file name argument given.
Input is read from the given file exactly as if the user had typed the
characters in the file to the terminal.
The debugger may be exited by some commands or characters in the file.
When the debugger reaches an end-of-file,
input is again taken from the previous source of the debugger's input,
thus "read" commands appearing in such files nest properly.
For example, if the file "cmds" contains "evaluate 5",
then one should see the following.
@begin(example)
@DebugPrompt() read cmds
5
>
@end(example)

@Section(Other Features for Testing)

@SubSection(The Go Command)

@Index(go command)
The "go" command can be used to start the program running.
Thus the following two commands are equivalent:
@begin(example)
@DebugPrompt() go
@end(example)
and
@begin(example)
@DebugPrompt() evaluate start@us<>up()
@end(example)

If the name of a module is given,
that module is run as the main program, with no arguments and no results.
For example,
@begin(example)
@DebugPrompt() go main
@end(example)
is equivalent to the following.
@begin(example)
@DebugPrompt() evaluate main()
@end(example)

@SubSection(Comments @Dash() the % Command)

@Index(comment command)@Index(%)
Comments may be given to the debugger, to document the expected results or note
things about test cases.
The comment command ignores the remainder of a line following
a percent sign (%).
@begin(example)
@DebugPrompt() % this is a comment (command), which does nothing
@end(example)

@SubSection(Stub Routines)

@Index(stub routines)
The linker will automatically provide "stub" routines for routines which are
called but are not yet implemented.  In a production program, invoking a stub
routine will cause the program to crash.  When debugging, invoking a stub
routine will cause the name of the routine to be printed, and a new listen loop
will then be entered.
For instance,
@begin(example)
@DebugPrompt() evaluate start@us<>up()
stub routine: notyet
>
@end(example)
The stack frame corresponding to this call will show as
the routine "@us<>@us<>stub@us<>routine".
One can then use the "return", "signal",
or "yield" commands to force the appropriate return, signal, or yield, and
continue with the computation.

@SubSection(The Return Command)

@Index(return command)
The return command returns from the current frame with an optional list
of result expressions enclosed in parenthesis.
Any more recent frames (invocations) are simply discarded.
Suspended iterators cannot be forced to return.
For example,
@begin(example)
@DebugPrompt() return
@end(example)
returns from the current frame with no results,
and
@begin(example)
@DebugPrompt() return(1, 2, true)
@end(example)
returns from the current frame with three result objects.

Optionally, a frame specification (a number or module) and a process
may be supplied following the optional result list,
which specifies a different frame or a different frame in the given process
to return from, instead of the current frame.
For example,
@begin(example)
@DebugPrompt() return 3
@end(example)
returns from frame 3 with no results,
and
@begin(example)
@DebugPrompt() return (3) hello
@end(example)
returns from the most recent invocation of "hello" with the result "3",
and
@begin(example)
@DebugPrompt() return hello 3
@end(example)
returns from the most recent invocation of "hello" in process 3
with no results.

@SubSection(The Signal Command)

@Index(signal command)
The signal command signals a given exception
from the current frame with an optional list
of expression results enclosed in parenthesis.
Any more recent frames (invocations) are simply discarded.
Suspended iterators cannot be forced to signal.
For example,
@begin(example)
@DebugPrompt() signal foo
@end(example)
raises the "foo" exception from the current frame,
and
@begin(example)
@DebugPrompt() signal failure("Elementary")
@end(example)
signals failure with the exception result string "Elementary".

Optionally, a frame specification (a number or module) and a process
may be supplied following the optional exception result list,
which specifies a different frame or a different frame in the given process
to signal from.
For example,
@begin(example)
@DebugPrompt() signal foo 3
@end(example)
raises the signal "foo" from frame 3 with no exception results,
and
@begin(example)
@DebugPrompt() signal bar(3) hello
@end(example)
signals "bar" with exception result "3"
from the most recent invocation of "hello",
and
@begin(example)
@DebugPrompt() signal foo hello 3
@end(example)
signals "foo" with no exception results
from the most recent invocation of "hello" in process 3.

@SubSection(The Yield Command)

@Index(yield command)
The "yield" command yields from the current frame (which must be an iterator)
with an optional list
of result expressions enclosed in parenthesis.
However, note that the state of a suspended iterator (its variables)
is not changed by the "yield" command.
Any more recent frames (invocations) are simply discarded.
For example,
@begin(example)
@DebugPrompt() yield
@end(example)
yields from the current frame with no results,
and
@begin(example)
@DebugPrompt() yield(1, 2, true)
@end(example)
yields from the current frame with three result objects.

Optionally, a frame specification (a number or module) and a process
may be supplied following the optional result list,
which specifies a different frame or a different frame in the given process
to yield from.
For example,
@begin(example)
@DebugPrompt() yield 3
@end(example)
yields from frame 3 with no results,
and
@begin(example)
@DebugPrompt() yield (3) int$from*
@end(example)
yields from the most recent invocation of an iterator
matching "int$from*" with the result "3",
and
@begin(example)
@DebugPrompt() yield (3) 4 3
@end(example)
yields from the fourth frame of process 3 with the result "3".

@SubSection(The Retry Command)

@Index(retry command)
The "retry" command retries the current or given frame's invocation.
Any stack frames which are more recent than the one retried are discarded.
The arguments used are the current arguments of that routine.
For example,
@begin(example)
@DebugPrompt() retry
@end(example)
retries the current frame,
while
@begin(example)
@DebugPrompt() retry hello
@end(example)
retries the most recent invocation of "hello" on the stack.

Optionally, a process argument may also be given after a frame argument,
and this specifies that the retry is to be done from the given frame in
the given process.
For example,
@begin(example)
@DebugPrompt() retry hello 3
@end(example)
specifies that the most recent invocation of "hello" in process 3's stack
is to be retried.

@SubSection(The Restart command)

@Index(restart command)
The "restart" command throws out all processes and their stacks,
and discards all nested listen loops.
Breakpoints remain after this command.
It takes no arguments.
For example, after
@begin(example)
@DebugPrompt() restart
@end(example)
there is only the initial process again, and no nested listen loops.

@SubSection(The Loaded Command)

@Index(loaded command)
The "loaded" command
prints a list of all loaded modules that match the given
module patterns.
For example,
@begin(example)
@DebugPrompt() list hel* start*
hello
start@us<>up
@end(example)
shows the modules matching the given module patterns.
Note that an operation of a cluster is not considered to be a module,
thus,
@begin(example)
@DebugPrompt() list int
int
@end(example)
shows the "int" cluster but not its operations (such as "int$add").

The following options for the "loaded" command delimit the search
for routines and modules to specific sets.
Some options ("callers", "iterators", "procedures")
also consider operations of clusters in the search.
@begin(Description)
callers@\Only routines calling the matching routines are listed.

dus@\Matching modules are printed with their DUs.

iterators@\Only matching iterators are listed.

procedures@\Only matching procedures are listed.

stubs@\Only matching stub modules (unimplemented routines) are listed.

types@\Only matching types (clusters) are listed.
@end(Description)

For example,
@begin(example)
@DebugPrompt() loaded/callers hel* stream$putl
hello:
	start@us<>up

stream$putl:
	hello
@end(example)
shows that start@us<>up calls hello,
and that stream$putl is called by hello.

@SubSection(The Actions Command)

@Index(actions command)
The "actions" command prints a list of all actions
along with their states.
For example,
@begin(example)
@DebugPrompt() actions
T1@@1	running
@end(example)
shows one action "T1@@1" which is running.

@SubSection(The Processes Command)

@Index(processes command)
@Label(ListProcesses)
The "processes" command prints a list of all processes
(excluding the debugger)
along with their status.
For example,
@begin(example)
@DebugPrompt() processes
  3 running (stream$putl, hello, start@us<>up)
  4 blocked on object 21 (@us<>process$lock, stream$putl, ...)
  7 running (stream$putl, goodbye, start@us<>up)
@end(example)
shows three processes, two of which are running, and one of which is blocked.
An abbreviated form of their stacks is shown in parenthesis.

@SubSection(The Defined Command)

@Index(defined command)
The "defined" command prints a list of all defined debugging session
variables along with their values.
It takes no arguments.
For example,
@begin(example)
@DebugPrompt() defined
ai = array[int]
x = 4
xai = [1: 1, 2, 3]
@end(example)
shows three variables and their values.

@SubSection(The Undefine Command)

@Index(undefine command)
The "undefine" command removes the given identifiers from the list of
debugging session variables.
For example,
@begin(example)
@DebugPrompt() undefine x y
@end(example)
removes the definitions of "x" and "y",
that is, it makes "x" and "y" no longer be debugging session variables.

@Chapter(Locating the Causes of Bugs)

@Index(locating bugs)
The second major task that the debugger can help you with is locating the cause
or causes of program bugs.
There are several kinds of program errors, and the debugger has a feature or features
that assist you in finding the cause of each one.
@begin(description)
Crashes@\
which may be caused by any number of things, including unhandled exceptions,
uses of uninitialized variables, lack of space, etc.
The debugger tries to prevent crashes and
gives control to the user (a listen loop) at the point of the crash.
In this way the user can determine the cause of the crash by examining the
program's state.
For example, the "frames" command allows you to look at the stack
of the process that would cause the crash.

Uninitialized Variable Usages@\
are supposed to be trapped, but, alas, are not always caught.
There is a rather expensive command "trace-uninitialized" which watches more
carefully for them (but will still leave some unnoticed).

Deadlocks@\
The debugger allows you to examine the status of all processes and actions,
to see what locks they hold and what locks they are waiting for.

General Errors@\
that is, a program which does not match its specification.
These kind of errors are very difficult to pinpoint exactly, even with the debugger's
help.  However, the debugger does have several features which can help you find out
what part or parts of your program are not doing the correct thing.
For instance, you can single step through a routine's execution using the
"trace" command.  The entire trace facility is geared to helping you narrow
down the focus of your search for the culprit or culprits.

Performance Errors@\
that is, a program which does not meet its time or space constraints.
The debugger has the "evaluate/time" command
(see section @ref(EvaluateTime))
which may be of some assistance in tracking
down performance errors.

Low-Level Errors@\
including compiler errors, errors in the run-time system,
and errors in the debugger itself are nearly impossible to debug with the debugger.
There is a low-level debugger "bug"@Index(bug, low-level debugger)
which can be used in such situations.
@end(description)

@Section(Points of Reference)

@Label(PointsOfReference)
In a running Argus program there may be many things going on,
in many places.
This section describes some points of reference,
which help you locate things in your program in both time and space.

@SubSection(Modules)

@Index(module)
A @i(module) is the identification a cluster or a routine.
It is specified by a string of letters, digits, underscores, and asterisks (*),
not beginning with a digit.
The asterisk@Index(*) stands for any number of these characters,
and performs matching on loaded module and routine names.
A module name which has a DU (a cluster or a routine outside a cluster),
may be optionally followed by a sharp sign (#)
@Index(#)
and two integer literals separated by a sharp sign, which give, respectively,
the "cid" (a unique identifier)
and "tid" (a time stamp)
of a DU in decimal (as in "set#1243526#1435465[int]").
If the module has parameters, these can be specified after the DU (if any),
for example
@begin(example)
@DebugPrompt() trace x#234234#234132[int]$hi y[foo]$bar[real]
x$hi
y$bar
@end(example)
Finally, as shown, after a cluster name, DU, and parameters (if any)
the name of a routine may be given following a dollar sign ($),
with any of its parameters.

Modules may be assigned to debugging session variables if desired.
The "list" command lists the modules loaded and their associated DUs.
However, only "real" modules, that is not operations of clusters, have DUs.

@SubSection(Slots)

The basic point of reference in any routine is the slot.
@Index(slot)
A @i(slot) is a location in a routine's compiled code
from which a call (subroutine jump) of some kind takes place.
Not all of the invocations in your program's text correspond to slots
in the compiled code, since the optimizer expands some invocations in-line
(notably, operations of the built-in types).

Breakpoints (tracing) can occur at slots, but not between slots,
thus when tracing the slots of a routine
one cannot see events taking place between slots.

The slots of a routine are numbered,
starting at one, for each routine.

Slots are used in two ways in tracing.
Since there is a routine called from every slot, the slot can be used
as a way to name that routine (as a context),
so that tracing can happen whenever that routine is invoked.
Alternatively, the slot itself, that is that particular call site,
can be traced, and in this case the routine is only traced if it is
activated through that slot.

@Paragraph(The Slots Command)

@Index(slots command)
The slots in a routine can be listed by the "slots" command.
When given only context arguments,
they list all the slots in the given contexts.
For example,
@begin(example)
@DebugPrompt() slots hel*
hello:
1	stream$putl
@end(example)
shows one slot in "hello", for "stream$putl".

Optionally, one may also specify a list of slots (a "slotlist")
following a semi-colon (;) the context arguments.
@Index(slotlist)@Index(;)
This list consists of slot numbers, or pairs of slot numbers separated
by a colon (e.g., "3:5"), and the components of the list may be separated
by blanks, tabs, or commas.
A pair of slot numbers specifies an inclusive range of numbers,
for example, "3:5", specifies slots 3, 4, and 5.
As a shorthand, the slotlist consisting of an asterisk (*)
stands for all the slots in a routine.
For example,
@begin(example)
@DebugPrompt() slots hello ; 1
hello:
  1 stream$putl
@DebugPrompt() slots hel* start*up ; 5 1:2
hello:
  1 stream$putl

start@us<>up:
  5 stream$putl
  1 hello
  2 stream$putl
@end(example)

@SubSection(Processes)

A @i(process) is a sequential thread of control.
Processes are given numbers for communication with the debugger,
but it is most helpful to think of them as identified by their stacks.
Each process has its own stack.
When execution begins, there is only a single process,
but new processes may be created as the program executes by the @b(coenter)
statement.

The "processes" command lists all processes
(see section @ref(ListProcesses)).

@SubSection(Contexts)

@Index(context)
A @i(context) is either a module, a frame, or a routine object,
but it is not restricted to being in the stack (as is a frame).
Contexts are meant to refer to routines,
thus a cluster is not properly a context in most cases.

Routine objects may be specified by the form "module^integer"
or "frame^integer"
@Index(slot in a module)@Index(^)
that is either a module name (pattern) or frame number
followed by an integer literal which specifies a
slot number in that module,
and thus the routine object associated with that slot.
Routine objects may be assigned to debugger session variables if desired.

@SubSection(Frames)

A stack consists of several @i(frames)
@Index(frame)
containing the local variables and other information needed by an invocation.
A frame is specified to the debugger by
either an integer literal specifying a frame on the stack
or a module (pattern) specifying the most recent invocation of that module
on the stack.
1 is the top of the stack (the most recent frame),
2 is the next most recent, etc.

The "frames" command displays the stack frames of the current process, and the
"current" command display the current stack frame or module
of the current process (see section @ref(CurrentCommand)).

@Paragraph(The Frames Command)

@Index(frames command)
The "frames" command with no arguments shows the entire stack of the current
process.
For example,
@begin(example)
@DebugPrompt() frames
  1[3]:    hello(how: "hello", who: "world")
  2[1];3   int$from@us<>to@us<>by(from: 1, to: 3, by: 2)
  3[1]:    start@us<>up()
@end(example)
shows 3 frames in the stack of the current process (in short format);
frame 1, which is called from slot 3 of the routine in the
next non-suspended frame (i.e., frame 3)
, is the frame for "hello",
frame 2, which is a suspended iterator called from slot 1 of the rouine
in frame 3,
and frame 3.
Note that the frame number and slot of a suspended iterator
is followed by a semi-colon (;)
@Index(;)
and the frame number of the calling frame
in contrast to procedures which are shown with a colon(:)
@Index(:)
and are called by the next non-suspended frame.

The "frames" command may also be given a list of frames as an argument,
these are displayed in the default format in the order in which they
are given.
@Index(framelist)
In a list of frames, a range of frames may also be given,
a range is specified by two frame specifications separated by a colon.
For example,
@begin(example)
@DebugPrompt() frames 1
  1[3]:    hello(how: "hello", who: "world")
@DebugPrompt() frames 2:3 1
  2[1];3   int$from@us<>to@us<>by(from: 1, to: 3, by: 2)
  3[1]:    start@us<>up()
  1[3]:    hello(how: "hello", who: "world")
@end(example)
Frames may also be specified by module name,
which specifies the most recent invocation of that module on the stack.
For example,
@begin(example)
@DebugPrompt() frames hello
  1[3]:    hello(how: "hello", who: "world")
@end(example)

The options to the "frames" command control the style of frames printing.
@begin(Description)
long@\Makes frames show with each argument on a separate line.

process@\Makes frames show the frames in the given process without changing
the current process.

short@\Makes frames show with arguments on one line.

variables@\Makes frames show with each argument on a separate line, as well as
all the local variables (including owns) of the routine.
@end(Description)

@Index(long format frame printing)
@Index(process format frame printing)
@Index(short format frame printing)
@Index(variables format frame printing)
For example,
@begin(example)
@DebugPrompt() frames/short 2 hel*
  2[1];3   int$from@us<>to@us<>by(from: 1, to: 3, by: 2)
  1[3]:    hello(how: "hello", who: "world")

@DebugPrompt() frames/long
  1[3]:    hello
	how: "hello"
	who: "world"
  2[1];3   int$from@us<>to@us<>by
	from: 1
	to: 3
	by: 2
  3[1]:    start@us<>up

@DebugPrompt() frames/process 3
  1:    start@us<>up

@DebugPrompt() frames/variables
  1[3]:    hello
	how: "hello"
	who: "world"
	sep: ???
  2[1];3   int$from@us<>to@us<>by
	from: 1
	to: 3
	by: 2
  3[5]:    start@us<>up
@end(example)
shows the local variable "sep" in hello is uninitialized.

@Paragraph(The Current Command)

@Index(current command)
@Label(CurrentCommand)
The "current" command shows and optionally changes the current frame or module.
(This command can safely be ignored by the novice.)
When given a single context argument, it changes the current frame
or module to that specified by the given context.
For example,
@begin(example)
@DebugPrompt() current 1
process 3
  1[3]:   hello(how: "hello", who: "world")
@end(example)
changes the current frame to be the most recent frame on the stack,
which in this case is an invocation of "hello",
of the current process (which in this case is 3),
and
@begin(example)
@DebugPrompt() current hello
process 3
  1[3]:   hello(how: "hello", who: "world")
@end(example)
changes the current frame to be the most recent invocation of "hello" on
the stack of the current process,
or if "hello" is not on the stack of the current process,
then the current module is made "hello" and the stack is no longer current.

Optionally, a second argument may be given to the "current" command
to change the process as well as the frame or module.
For example
@begin(example)
@DebugPrompt() current 1 3
process 3
  1[3]:   hello(how: "hello", who: "world")
@end(example)
changes the current frame to be the most recent invocation frame on the
stack of process 3,
and
@begin(example)
@DebugPrompt() current hello 3
process 3
  1[3]:   hello(how: "hello", who: "world")
@end(example)
changes the current frame to be the most recent invocation of "hello" on
the stack of the process 3,
or if "hello" is not on the stack of the process 3,
then the current module is made "hello" and the stack is no longer current,
but the current process is now 3.

The "current" command with no arguments just
shows the current stack frame in the default format.
If the current module is not in a stack, then only the identity of the
current module is displayed.
For example, assuming the default format is "short"
and the current frame is "1",
the "current" command acts as follows.
@begin(example)
@DebugPrompt() current
process 3
  1[3]:   hello(how: "hello", who: "world")
@end(example)
Assuming the current frame is "remember" which is not on the stack,
the display command acts as follows.
@begin(example)
@DebugPrompt() current
process 3
	remember (not on the stack)
@end(example)

The options for the "current" command are the same as those for the "frames"
command, and have the same meaning.
@begin(Description)
long@\Makes frames show with each argument on a separate line.

short@\Makes frames show with arguments on one line.

variables@\Makes frames show with each argument on a separate line, as well as
all the local variables (including owns) of the routine.
@end(Description)

@Section(Tracing)

@Index(tracing)
This section describes the commands for turning various kinds of tracing
on and off and commands for showing the tracing status of the debugger.
Tracing is accomplished by setting breakpoints at places in the load module,
the next section describes what happens when a breakpoint is hit.
Breakpoints may be set at the entries and exits of routines and at the
slots within a routine.
The basic idea is that by setting the right breakpoints,
one can watch the program's execution and discover the location of some bug
or bugs.

For each routine, one may trace that routine (in all parameterizations)
or just a specific activation of that routine.
The "trace" commands all trace routines in all activations,
however the "S" breakpoint command traces just a specific activation.

There is also a hierarchy of kinds of tracing in a routine.
The most general kind of tracing is gotten with the "trace" command,
and the most specific with options like "trace/returns".
Here is the hierarchy.
@begin(display)
I. Breakpoint tracing ("trace" command)
   A. "at" (calls and exits) breakpoints
      1. "calls" (entry and resumption of iterators) breakpoints 
      2. "exits" breakpoints
         a. "returns" breakpoints
	 b. "yields" breakpoints
	 c. "signals" breakpoints
   B. "slots" breakpoints
II. Exception tracing ("trace-exception" command)
III. Uninitialized variable tracing ("trace-uninitialized" command).
@end(display)

Each kind of tracing specified can be turned on for a routine
(or a set of routines) with the "trace" or "trace-exception" commands.
The various kinds of breakpoint tracing can also be applied
to activations from particular slots in a routine.
Each kind of tracing may be turned off with the corresponding untrace- command.
The "untrace" command and options turn off all kinds of tracing
which are subsidiary in the tracing hierarchy.
For instance "untrace" turns off all tracing of every kind,
while "untrace/slots/calls" turns off only tracing of calls and slots,
but not exits.

Note the difference between the tracing of a routine "foo" and the tracing
of a slot in some routine which happens to be a call to "foo".
The tracing of "foo" will cause the specified kind of breakpoints
to occur whenever "foo" is called, from any slot,
the tracing of a slot which calls "foo" only causes the specified kind
of breakpoints to occur in "foo" when "foo" is called from that slot.

Not every routine can be traced.
Some system and user routines will be implemented as "qprocs"
which do not build stack frames, and thus will be impossible to trace.

@SubSection(The Trace Command)

@Index(trace command)
The "trace" command without options puts a breakpoint in every place in its
given context arguments.
That is, it sets breakpoints at the entry to, resumes of,
returns, signals, and yields from,
and all calls, resumes, returns, signals, and yields of all the slots in
the given contexts.
Thus, you are notified when the routine is called, and single-stepped through
its execution.
For example,
@begin(example)
@DebugPrompt() trace hel*
@end(example)
sets a trace on all routines matching "hel*", for instance "hello".

The options for the "trace" command limit the breakpoints set.
There is one option for each member of the "trace" hierarchy given above.
@begin(Description)
at@\Only sets breakpoints at the entries and exits of a routine.

calls@\Only sets breakpoints at the entries of a routine.

exits@\Only sets breakpoints at the exits from a routine.

returns@\Only sets breakpoints at the returns from a routine.

signals@\Only sets breakpoints at the signals from a routine.

yields@\Only sets breakpoints at the yields of an iterator.
@end(Description)

Optionally, a list of slots preceded by a semi-colon (;)
@Index(;)
may follow the list of contexts.
This slot list specifies that only the given slots are to be traced,
instead of every slot (if the list is omitted).
A slot list is a list of integer literals or two integer literals
separated by a colon (as in "3:6" which specifies slots 3, 4, 5, and 6),
@Index(:)
which specifies a list of slots.
As a shorthand, the slotlist consisting of an asterisk (*)
@Index(*)
stands for all the slots in a routine.
When the slotlist is given as part of a "trace" command,
it specifies that the tracing applies not to a routine given as a context,
but to routines called through the given slots in the context arguments.
For example,
@begin(example)
@DebugPrompt() trace/slots hello remember ; 1 3:5
@end(example)
sets breakpoints at slots of routines called from slots 1, 3, 4, and 5
of the routines "hello" and "remember" when they are called from those slots.
These breakpoints only apply to activations of those routines called through
one of the traced slots.

@SubSection(The Untrace Command)

@Index(untrace command)
The "untrace" command without any options
turns off all tracing in the given contexts,
effectively undoing the "trace" command.
For example,
@begin(example)
@DebugPrompt() untrace *
@end(example)
turns off all tracing set by the "trace" command in in all modules.

The untrace command takes the same options as the "trace" command.
Each option limits the set of breakpoints that are undone.
There is also an optional slot list argument, as in the "trace" command.
For example,
@begin(example)
@DebugPrompt() untrace/at 1 hello ; *
@end(example)
clears entry and exit breakpoints at all slots of in "hello" and
the routine associated with frame 1,
but leaves any breakpoints at the entries or exits of these routines unchanged.

@SubSection(The Trace-Exception Command)

@Index(trace-exception command)
The "trace-exception" command causes a breakpoint whenever
the particular signals named (preceeding the semi-colon ";") are raised
from the given context arguments (which follow the semi-colon).
@Index(;)
@begin(example)
@DebugPrompt() trace-exception overflow ; *
@end(example)
breaks at all signals of the exception "overflow" in all routines.
Thus, whenever "overflow" is signalled, the user will be notified.
Note that this tracing is quite separate in the hierarchy from the "trace"
command.

@SubSection(The Untrace-Exception Command)

@Index(untrace-exception command)
The "untrace-exception"
undoes the work of the "trace-exception" command for the given exception names
in the given contexts.
For example,
@begin(example)
@DebugPrompt() untrace-exception overflow ; *
@end(example)
turns off all trace-exception tracing of the "overflow" exception
in all routines.

@SubSection(The Restore-Tracing Command)

@Index(restore-tracing command)
The "restore-tracing" command restores tracing temporarily
inhibited by the Q, Y, or T breakpoint commands
(see the next section).
It takes no arguments.

@SubSection(The Trace-Uninitialized Command)

@Index(trace-uninitialized command)
The "trace-uninitialized" command tries to catch uninitialized variable
references as soon as possible, however, even this is not always possible.
It takes a list of context arguments from which uninitialized variables
are to be watched for.
This tracing is fairly expensive.
For example,
@begin(example)
@DebugPrompt() trace-uninitialized *
@end(example)
traces uninitialized variables in all routines for which this is can be done.
When an uninitialized variable usage is detected, a breakpoint will be taken.

@SubSection(The Untrace-Uninitialized Command)

@Index(untrace-uninitialized command)
The "untrace-uninitialized" command turns off uninitialized variable tracing
in the given contexts.
For example,
@begin(example)
@DebugPrompt() untrace-uninitialized *
@end(example)
turns off all uninitialized variables tracing set by the "trace-uninitialized"
command.

@SubSection(The List-Trace Command)

@Index(list-trace command)
The "list-trace" command lists the tracings in effect for the given contexts.
This includes tracing set by the "trace", "trace-exception", and
"trace-uninitialized" commands.
For example,
@begin(example)
@DebugPrompt() list-trace *
hello:
	trace/at
	trace/slots 2 3 5
	
start@us<>up:
	trace/slots
@DebugPrompt() list-trace hello
hello:
	trace/at
	trace/slots 2 3 5
@end(example)
shows the tracings in effect.

@SubSection(The Continue Command)

@Index(continue command)
The "continue" command is useful from within a nested listen loop,
that is, from a listen loop started up when a breakpoint is hit
by typing a carriage return to the "--@ next?@ --@ " prompt.
It resumes execution following the breakpoint.
It takes no arguments, as in the following example.
@begin(example)
@DebugPrompt() continue
@end(example)

@Section(Breakpoint Commands)

When a breakpoint is hit, the current activation of the
routine being entered or exited (the callee) is always displayed.
The current activation of the routine's caller is displayed next,
unless the "f" breakpoint command has been used to suppress it.
These frames are printed in short format by default,
but their format may be changed by the "x" breakpoint command.

After these frames are displayed,
the prompt "--@ next?@ --@ " will be printed
and a single character specifying a
breakpoint command is requested; case does not matter.
(In some versions, the process number of the process being debugged will be
printed before the prompt.)
Although the command set is basically the same for every breakpoint,
the activation the command refers to is generally different when
entering a routine than when exiting a routine.

Typing a carriage return to the "--@ next?@ --@ " prompt causes a new (nested) listen
loop to start up, from which you can do arbitrary things before continuing via
the "continue" command.

The following commands always have the same interpretation.
@begin(Description)
?@\
@Index(? breakpoint command)
Print a terse listing of the commands and their meanings for this breakpoint,
and ask for another command.

<space>@\
@Index(<space> breakpoint command)
(or blank) Continue until the next breakpoint is hit.

<return>@\
@Index(<return> breakpoint command)
(the return key)
Start up a new listen loop.  When continued via a "continue" command
from the listen loop, redisplay and ask for another command.

f@\
@Index(f breakpoint command)
(flip)
Flip the callee-only/callee-caller frame printing mode, redisplay, and ask for
another command.

q@\
@Index(q breakpoint command)
(quit)
Quit showing breakpoints until the listen loop is reentered.  This is
not the same as doing "untrace *",  because  none  of  the  permanent
breakpoints go away, they are just temporarily ignored.

x@\
@Index(x breakpoint command)
Exchange the current frame printing mode (long/short/variables) with
the next mode in a cyclic list, redisplay, and ask for another command.
@end(Description)

Other breakpoint commands depend on the kind of breakpoint for their
interpretation.  Each command makes some change to the set of breakpoints and
then continues execution unless indicated otherwise.  Below we use R as the
name of the routine of interest.  When calling or resuming an iterator, R
refers to the routine being called or resumed.  When returning, signalling, or
yielding, R refers to the routine being returned to, signalled to, or yielded
to.

@begin(Description)
y@\
@Index(y breakpoint command)
(yield)
Let R yield or terminate by inhibiting all tracing of calls made by the
current activation of R, including all calls made from those calls, etc.  Hence
the next breakpoint to be hit will be when R returns, signals, or yields.
Other, more recent, activations in the stack, and subsequent activations not
created during calls from the current activation, will have breakpoints set
just as if this command was never typed.

t@\
@Index(t breakpoint command)
(terminate)
Terminate R by
ignoring breakpoints until the current activation of R returns or signals.
This has the same effect as the "yield" breakpoint command, but in addition,
if R is an iterator, breakpoints will no longer be hit when the current
activation of R yields or is resumed.  However, a breakpoint will still
be hit when R returns or signals.

a@\
@Index(a breakpoint command)
(allow)
Allow breakpoints
by undoing the effect of the "y" (yield) breakpoint command for the current
activation.
This command is only useful when resuming an iterator after a "y" (yield)
breakpoint command  has  previously  been  given for the activation.

s@\
@Index(s breakpoint command)
(single-step)
Single step by doing a temporary "trace/at R" command
which only applies to the
current activation of R, and no other.
The "trace/at" will even apply to iterators
called by R that are currently suspended.
Note that the "s" breakpoint command is not the same as doing a
"trace/at R",
since the breakpoints only apply to the current activation of R.

u@\
@Index(u breakpoint command)
(un-single-step)
Undo the effect of the "s" (single-step) breakpoint command
for the current activation only.
This command is only useful when a "step" breakpoint command has
previously been given for the activation.
Note that undoing the effect of the
"s" breakpoint command is not the same as doing a temporary
"untrace/at R".
Breakpoints previously set in R by a "trace" command will still be hit.
This includes breakpoints for any iterators called by R
that are currently suspended.  However, tracing within those suspended
iterators remains unaffected.
@End(Description)

For the following two commands a breakpoint is assumed to have been hit for an
invocation of R.
R may be being called, returning, yielding, signalling, or
being resumed.  In all cases, the slot affected is the one in R's caller
corresponding to this call of R.
@begin(Description)
p@\
@Index(p breakpoint command)
(permanent)
Make the breakpoint permanent
by doing the equivalent of a "trace/at" command on
R's caller for the particular slot from which R was called.
This  command  is  only  useful  when  a "s" (single-step)
breakpoint command  has
previously been given for R's caller.
Thus  you  can initially step through a routine initially via
the "s" breakpoint command,
setting permanent breakpoints on particular  invocations
with  the "permanent" breakpoint command,
and then possibly even use the "u" (un-single-step)
breakpoint command to remove
the remaining temporary breakpoints.
Execution does not continue after
this command; another command is asked for.

d@\
@Index(d breakpoint command)
(delete)
Delete the breakpoint hit by
doing the equivalent of an "untrace" command on R's caller for the
particular slot from which P was called.
This command is only useful when the breakpoint has been hit as
a result of a "trace" command on R's caller.
This command has the effect
of an "untrace" on R's caller for the particular slot from  which
R was called.   Thus you can initially do a "trace/slots"  on  a  routine,
and  then selectively remove  those breakpoints  that are  not  useful.
Execution does not continue after  this  command;  another  command  is
asked for.
@End(Description)

@Appendix(Command Syntaxes)

@Index(command syntaxes)
In the command syntax descriptions that follow,
command names are written in UPPER CASE to distinguish them from category names
which are written in lower case, however the utilities are insensitive to
case distinctions except in file names.
@Index({})
Items surrounded by curly brackets ({}) may be repeated zero or more times.
Each individual item may be separated from the others by a blank, tab, or comma
as the user wishes.
@Index([])
Items surrounded by square brackets ([]) are optional.
Items or syntax surrounded by brackets
@Index(...)
followed by three dots (...) may be repeated one or more times,
separated by blanks, tabs, or commas.

@AppendixSection(libby)

@Index(libby command syntax)
The jcl line has the form:  [ # cmnd ] args { # cmnd args } [ ^ [ filename ] ].@*
A command line has the form:    cmnd   args { # cmnd args } [ ^ [ filename ] ].@*
The default cmnd in the jcl line is LIBRARY.
Commands can be specified with unambiguous prefixes.
Output goes to primary output unless ^ is specified.
Default filename after ^ is "libby.junk".

@begin(Description)
ADD filename { module }@\
Add the given modules (and all modules they depend on) from the given
library to the current library, not allowing any existing interfaces
in the current library to change.

ADD/CHANGE filename { module }@\
Add the given modules (and all modules they depend on) from the given
library to the current library, allowing interface changes.

CREATE filename int@\
Create a new, empty library with the given hash size and make it the
current library.  Libraries all have suffix ".lb" (which you need not
type), and will have an associated lock file with suffix ".LB".

DELETE { module }@\
Delete the given modules from the current library, provided no other
modules depend on them.

EXTRACT	{ filespec }@\
Extract the modules from the given binary files, and add them (with no
interface information) to the current library.  Useful only when you
have clobbered a library and need to reconstruct it from scratch.
Default suffix for files is "obj".  Any "filespec" can also be
"@@filename".  Default suffix for an @@-file is "xload".  An @@-file
contains a sequence of filespecs (including @@-files), separated by
newlines.

FORGE module cid tid@\
Forge a new DU with the given name, creation-id, and time-id.

GC@\
Rebuild the current library with the same hash size, removing garbage.

HELP@\
Print command summary.

INFORMATION { module }@\
Print machine and time of creation of each module's DU.

KILL@\
Kill libby.

LIBRARY	[ filename ]@\
Make the given library the current library, or print the name of the
current library if no argument is given.

MERGE { filename }@\
Add all of the modules from each of the given libraries to the current
library, not allowing any existing interfaces in the current library
to change.

MERGE/CHANGE { filename }@\
Add all of the modules from each of the given libraries to the current
library, allowing interface changes.

NAMES { pattern }@\
Print the names of all modules in the current library matching the
given patterns (with * matching any sequence of characters).

REBUILD	int@\
Rebuild the current library with the given hash size, removing garbage.

QUIT@\
Kill libby.

STATS@\
Print statistics about the current library, including wasted space,
current hash size, number of modules, and the expected number of
comparisons when looking up an existing module.

TRY int [ int [ real ] ]@\
Test a different hash size (if one argument) or a range of hash sizes
(if two or three arguments) to find either the hash size with the
least expected number of comparisons and the minimum variance, or
the first hash size with an expected number of comparisons less than
a given value (the third argument).

USED-BY	{ module }@\
Print all types/generators used directly by the given modules.

USERS-OF { type }@\
Print all modules directly using the given types/generators.

*USED-BY { module }@\
Print all types/generators depended on (directly or indirectly) by the
given modules.

*USERS-OF { type }@\
Print all modules depending on (directly or indirectly) the given
types/generators.

?@\
Print command summary.
@end(Description)

@AppendixSection(argus)

@Index(argus command syntax)
The jcl line has the form:  [ # cmnd ] args { # cmnd args } [ ^ [ filename ] ].@*
A command line has the form:    cmnd   args { # cmnd args } [ ^ [ filename ] ].@*
The default cmnd in the jcl line is COMPILE.
Commands can be specified with unambiguous prefixes.
Output goes to primary output unless ^ is specified.
Default filename after ^ is argus.junk.
Default suffix for PARSE, CHECK, and COMPILE input files is "arg".
Default suffix for SPECS input files is "spc" or "arg".
Default suffix for CE and XCE input files is "equ".
Default suffix for XFILE input files is "xfile".

Below, "filespec" can also be "@@filename".  Default suffix for an @@-file is
"xload".  An @@-file contains a sequence of filespecs (including @@-files),
separated by newlines.

@begin(Description)
BINDIRECTORY [ directory ]@\
Set or clear the directory where binary files are placed.  The default
is to use the source file's directory.

BINSUFFIX [ suffix ]@\
Set or clear the suffix used for the binary file.  The default suffix
is "obj".  Use "*" to indicate an empty suffix.

CE { filespec }@\
Discard all current global equates, and make the equates in the given
files global, with equates in earlier files available for use in later
files.  If an error occurs in a file, all of its equates are discarded.
Each file can only contain equates.  BUG: module interface changes do
not cause global equates to be rechecked; e.g., an equate to a routine
will continue to be used as if the routine had its old interface.

CHECK { filespec }@\
Check the given files for semantic errors.

COMPILE	{ filespec }@\
Compile the given files.  No binary is produced for files with errors.

DIRECTORY [ directory ]@\
Set or clear the default working directory, which can be specified
relative to the current working directory.

ESCAPE@\
Temporarily exits the compiler (like ^Z).

GENERATOR { module }@\
Pseudo-define the given modules as type generators.  Instantiations of
these types will be considered legal, but invocations of operations
will be illegal, until the type is truly defined.  The TYPE command
is only needed when defining groups of types with cyclic dependencies.
First pseudo-define the types, then SPECS them.

HELP@\
Print command summary.

IMMUTABLE@\
Do not allow module interfaces to change.  This is the default.

KILL@\
Kill compiler.

LIBRARY	[ filename ]@\
Discard all global equates and the current library and make the given
library the current library, available for both reading and writing.

MUTABLE@\
Allow module interfaces to change.

PARSE { filespec }@\
Check the given files for syntactic errors.

QUIT@\
Kill compiler.

RLIBRARY [ filename ]@\
Discard all global equates and the current library and make the given
library the current library, available for reading only.  Beware, as
you can then create DUs that can never be referenced again.

SPECS { filespec }@\
Check the interfaces of modules in the given files for semantic errors.

TYPE { module }@\
Pseudo-define the given modules as unparameterized types.  Uses of
these types will be considered legal, but invocations of operations
will be illegal, until the type is truly defined.  The GENERATOR
command is only needed when defining groups of types with cyclic
dependencies.  First pseudo-define the types, then SPECS them.

XCE { filespec }@\
Make the equates in the given files global, with existing global
equates and equates in earlier files available for use in later files.
If an error occurs in a file, all of its equates are discarded.
Each file can only contain equates.

XFILE { filespec }@\
Execute the commands in the given files.

?@\
Print command summary.
@end(Description)

@*

@*

@b[Compiler language extensions:]
@Index(compiler language extensions)

Extra character escapes:
@begin(Description)
\#**@\
where * is a hex digit

\^*@\
where * is one of a-z, A-Z, @us<>, @@, [, \, ], ?
escape for obvious control character, with \^? = \177

\!*@\
where * is one of a-z, A-Z, @us<>, @@, [, \, ], ?
escape for 0200 + \^*

\&*@\
where * is a printing char
escape for 0200 + *
@end(Description)

Integer escapes:
@begin(Description)
\*@\
where * is any number of octal digits

\#*@\
where * is any number of hex digits
@end(Description)

@AppendixSection(alink)

@Index(alink command syntax)
The jcl line has either has the form:  Object@us<>File ... { #option }  [ ^ output@us<>file ]@*
or is a single question mark (?).
Object@us<>File is either the name of a binary file
or "@@X@us<>Load@us<>file".

The default suffix for an Object@us<>File is "obj".
The default suffix for an @@X@us<>Load@us<>file is "xfile".
Object file names may be separated by either commas, spaces, or tabs.
If the output@us<>file is not given, "a.out" is assumed.

The following options are defined.
@begin(Description)
ALL@us<>LIST@\
A combination of the CONSTS@us<>LIST, EXCEPTS@us<>LIST, OPERATIONS@us<>LIST,
OWNS@us<>LIST,
ROUTINES@us<>LIST, and SEG@us<>MAP@us<>LIST options.

CONSTS@us<>LIST@\
List the constants (equates, strings, etc.) in the program and their locations
in the linked program.

DEBUG@\
Link a debugger with the program, instead a production program.

EXCEPTS@us<>LIST@\
List information about exceptions.

HEAP@us<>SIZE:integer@\
Specifies a heap size instead of the default 1M.

LIST@us<>FILE:filename@\
Specifies a file to send LIST options output to, instead of primary output.

MAIN:idn@\
Gives the name of a main routine instead of the default "start@us<>up" to
be called when execution begins.

OPERATIONS@us<>LIST@\
List the clusters linked into the program.

OPTIMIZE@us<>EXCEPTS@\
Optimize exception handling.

OWNS@us<>LIST@\
List the own variables in the linked program and their locations.

ROUTINES@us<>LIST@\
List the routines and operations of the linked program and their locations.

SEARCH:directory@\
Search the given directory's files for symbols that are undefined.

SEG@us<>MAP@us<>LIST@\
Print the segment map, where various divisions are in the linked program
and their sizes.

SYMBOLS@\
Generate symbols for the debugger to use.
@end(Description)

Options without arguments may be preceeded by a minus sign (-) to negate
the sense of the option.

@AppendixSection(the debugger)

@Index(debugger command syntax)
When  typing  a line, rubout deletes a character typed in error, ^U deletes the
current line, ^W deletes the last word on the line, ^R redisplays  the  current
line,  ^X  starts  a  new  blank  line, ^L clears the screen and redisplays the
current line, and ^\ causes line continuation.
An end-of-file causes the debugger to read input from the terminal

The following categories are used in the descriptions of the syntax which
follow.
@Begin(Description)
cmdname@\
@Index(cmdname)
is the name of one of the debugger's commands or an unambiguous prefix to one
of the debugger's commands.

context@\
@Index(context)
is either a module, a frame, or a routine object,
but it is not restricted to being in the stack (as is a frame).
Routine objects may be specified by the form "module^integer",
@Index(slot in a module)@Index(^)
that is a module name followed by an integer literal which specifies a
slot number in that module,
and thus the routine object associated with that slot.
A context which is a frame specifies the routine associated with that
frame in cases where a routine object is required.

expression@\
may be any Argus expression excluding the infix and
prefix operators.
(Debugging session variables may refer to types, just as equates can in Argus.)
Identifiers used in expressions
are resolved in one of two ways: a context may be explicitly given,
or dynamic scoping is used from the current frame or module.
To explicitly give a context to an identifier,
the form:  "idn [ ! context [ ! process ] ]"
@Index(!)
may be used when an identifier (idn) is legal in an expression.
If the explicit context for an identifier is given,
then identifier is expected to be a variable visible from that context
(in the current or given process).
If no context or process is given, then an identifier is sought
(1) in the debugging session variables, (2) in the names of modules
loaded, (3) in the current frame or module's variables, and (4)
in the variables of the frames earlier on the stack
(if the current context is a frame).

expression-or-assignment@\
@Index(expression-or-assignment)
is either an expression or
an equates or assignment statement,
both of which perform assignment to variables.

filename@\
@Index(filename)
is a normal file name, with * matching, but it must not
contain spaces, tabs, or commas.

frame@\
@Index(frame)
is either an integer literal specifying a frame on the stack
or a module (pattern) specifying the most recent invocation of that module
on the stack.
(Such a pattern must be unambiguous.)
1 is the top of the stack (the most recent frame),
2 is the next most recent, etc.

framelist@\
@Index(framelist)
is a list of frames or pairs of frames separated by spaces, tabs, or commas.
Pairs of frames
are separated by a colon to specify an inclusive range of frames
(as in "3:6" which specifies frames 3, 4, 5, and 6
and 5:3 which specifies frames 5, 4, and 3).
@Index(:)

module@\
@Index(module)
is the name of a cluster or a routine.
It is a string of letters, digits, underscores, and asterisks (*).
@Index(*)
It must not begin with a digit.  An asterisk appearing anywhere within
it performs pattern matching on the names of modules loaded.
The module name may be optionally followed by a sharp sign (#) and two
@Index(#)
integer literals separated by a sharp sign, which give, respectively,
the "tid" and "cid" of a DU in decimal (as in "set#155243526#152435465").
@Index(DU)
Following this, a module may have any parameters specified, and if it is a
cluster, the name of a routine and its parmeters may follow, preceeded by
a dollar sign as always.

options@\
@Index(options)
is the name of an option, or several (compatible) options in a list
(separated by blanks, tabs, commas, or slashes).

process@\
@Index(process)
is an integer literal.

slotlist@\
@Index(slotlist)
Is a list of integer literals or two integer literals
separated by a colon (as in "3:6" which specifies slots 3, 4, 5, and 6),
@Index(:)
which specifies a list of slots.
The asterisk (*) can also be used (alone) as a slot list,
it specifies all the slots of a routine.
@end(Description)

The commands available are the following.
@Include(masterlist.mss)

@big[Breakpoint Commands]

When a breakpoint is hit, some information is displayed, showing the stack
frames involved, and the prompt "--@ next?@ --@ " will be printed.  At each
appearance of "--@ next?@ --@ " a single character
specifying a breakpoint command
is requested; case does not matter.  Although the command set is
basically the same for every breakpoint, the activation to which the command
refers to is generally different when entering a routine than when exiting a
routine.
In what follows "R" refers to the name of the routine of interest.
When calling or resuming an iterator, R refers to the routine being called or
resumed.  When returning, signalling, or yielding, R refers to the routine
being returned to, signalled to, or yielded to.

@begin(Description)
<RETURN>@\
(the return key)
Start up a new listen loop.

<SPACE>@\
(or blank) Continue until the next breakpoint is hit.

?@\
Print a help listing, and ask for another command.

A@\
(ALLOW)
Allow breakpoints by undoing the effect of the Y breakpoint command.

D@\
(DELETE)
Delete the breakpoint hit, ask for another command.

F@\
(FLIP)
Flip the callee-only/callee-caller frame printing mode, and redisplay.

P@\
(PERMANENT)
Make the breakpoint hit permanent, ask for another command.

Q@\
(QUIT)
Quit showing breakpoints until the listen loop is reentered.

S@\
(SINGLE-STEP)
Single step the current activation of R.

T@\
(TERMINATE)
Terminate R by ignoring breakpoints until R returns or signals.

U@\
(UN-SINGLE-STEP)
Undo the effect of the S breakpoint command for the current activation.

Y@\
(YIELD)
Let R yield or terminate by inhibiting all tracing.

X@\
(EXCHANGE) Exchange the frame printing mode for the next one, redisplay.
@End(Description)
