/*
 *
 * UNIX CLU/Argus Debugger
 *
 */

#include	"defs.h"
#include	"object.h"


/* This file contains the object printer.  It recursively
 * traces objects much like debug, printing up to a max object width
 * and a max recursion depth.  It is called with a reference
 * (which is either an integer or an object pointer) and a current
 * recursion depth.
 */


int	max_depth = 4;			/* default max recursion depth */
int	max_width = 4;			/* default max object width */
extern	int	dotinc;			/* dot increment value */
extern	STRING	errflg;
extern	INT	mkfault;

object(obj, depth, force, space)

/* Force is true iff the obj value is to be interpreted as a reference
 * irrespective of its space.
 */

register unsigned long	obj;
int	depth;
int	force;
{
	register unsigned long	objhdr;	/* header of current object */
	int	inc;			/* increment for current obj */

	if (depth == 0)
		printc('\n');

	if (depth > max_depth) {
		tab(depth);
		printf("...");
		return;
	}

	if (force)
		goto ref;

	switch (reftype(obj)) {

case NONREF:				/* non-reference */
		tab(depth);
		printf("Non-reference %X\n", obj);
		inc = 4;
		break;

case POSINT:				/* positive integer */
case NEGINT:				/* negative integer */
		tab(depth);
		printf("%R", obj);
		inc = 4;
		break;

case OBJPTR:				/* object pointer */
ref:
		objhdr = get(obj, space);
		if (errflg || mkfault) {
			errflg = mkfault = 0;
			tab(depth);
			printf("%R", obj);
			inc = 4;
			break;
		}
		switch (objtype(objhdr)) {

case BVEC:				/* bvec - string or bytevec */
			bvec(obj + sizeof(objhdr), bveclen(objhdr), depth, space);
			inc = ((bveclen(objhdr) + 3) & ~3) + 4;
			break;

case CELL:				/* cell - oneof, variant, or anyize */
			cell(celltag(objhdr), obj + sizeof(objhdr), depth, space);
			inc = 8;
			break;

case VEC:				/* vec - record or struct */
			vec(obj + sizeof(objhdr), veclen(objhdr), depth, space);
			inc = (veclen(objhdr) << 2) + 4;
			break;

case ADV:				/* array dope vector */
{			register unsigned long	aref; /* array reference */
			register int	alow; /* low index in array */
			register int	asize; /* number of elements */

			alow = chkget(obj + sizeof(objhdr), space);
			asize = chkget(obj + (2 * sizeof(objhdr)), space);
			aref = chkget(obj + (3 * sizeof(objhdr)), space);
			adv(aref, alow, asize, depth, space);
			inc = 20;
			break;
}

default:				/* unknown object type */
			printf("Unknown object header %X\n", objhdr);
			inc = 4;
			break;
		}
	}
	if (depth == 0) {
		printc('\n');
		dotinc = inc;
	}
}


bvec(ref, len, depth, space)

/* Print the contents of the _bvec starting at address ref, with length
 * len (in bytes).
 */

register unsigned long	ref;		/* ref to start of bvec data */
register int	len;			/* length of bvec in bytes */
int	depth;				/* recursion depth */
{
	register unsigned char	nxtbyt;	/* next byte of bvec */
	register int	i;		/* max bound on loop */

	tab(depth);
	i = (len > (BVECFUDGE * max_width) ? (BVECFUDGE * max_width) : len);
	printc('"');
	while (i-- > 0) {
		nxtbyt = bchkget(ref, space);
		printbyt(nxtbyt);
		endline();
		ref++;
	}
	if (len > max_width)
		printf("...");
	printc('"');
}


cell(tag, ref, depth, space)

/* Print the contents of a tagcell.  Print it as tag: obj.
 */

int	tag;				/* object tag */
register unsigned long	ref;		/* reference to object */
int	depth;
{
	tab(depth);
	printf("< %D: ", tag);
	object(chkget(ref, space), depth + 1, FALSE, space);
	printf(" >");
}


vec(ref, len, depth, space)

/* Print the vec located at ref with length len.
 */

register unsigned long	ref;		/* reference to start of vec */
register int	len;			/* length of vec in longwords */
int	depth;
{
	register int	i;		/* width */

	tab(depth);
	i = (len > max_width ? max_width : len);
	printf("[ ");
	while (i-- > 0) {
		object(chkget(ref, space), depth + 1, FALSE, space);
		ref += sizeof(unsigned long);
		if (i != 0)
			printc('\n');
	}
	if (len > max_width)
		printf("...");
	printf(" ]");
}


adv(ref, low, size, depth, space)

/* Print the array beginning at address ref, with low index low and and
 * size size.
 */

register unsigned long	ref;		/* address of first element */
int	low;				/* index of low element */
register int	size;			/* number of elements in array */
int	depth;
{
	register int	i;		/* width */

	tab(depth);
	i = (size > max_width ? max_width : size);
	printf("[ %D: ", low);
	while (i-- > 0) {
		object(chkget(ref, space), depth + 1, FALSE, space);
		ref += sizeof(unsigned long);
		if (i != 0)
			printc('\n');
	}
	if (size > max_width)	
		printf("...");
	printf(" ]");
}


tab(depth)

/* Tab out to the appropriate nesting depth.
 */

register int depth;
{
	register int	curpos;		/* current char pos */

	for (curpos = charpos(),depth *= NSPLVL; curpos < depth; curpos++){
		printc(SP);
		endline();
	}
}


printbyt(byt)

/* Print a single eight-bit byte as a character, using escape conventions
 * for non-printing characters - ie print them as '\XX' where the XX are
 * hex digits.
 */

register unsigned char	byt;
{
	if (byt < 0x20 || byt >= 0x7f) { /* control or meta character? */
		register unsigned char	lo, hi;
		printc('\\');		/* yes, print as '\XX' */
		hi = byt >> 4; lo = byt & 0xf;
		printc(hi + (hi <= 9 ? '0' : 'a' - 10));
		printc(lo + (lo <= 9 ? '0' : 'a' - 10));
		return;
	}
	printc(byt);
}
