@device(postscript)
@make(report)
@style(font TimesRoman, size 11, spacing 2)
@begin(text)

@begin(titlepage)
@begin(titlebox)
@blankspace(0.5 inch)
@majorheading(Programming X in Argus)
@skip(1)
@center(Jang Hyuck Chun)
@center(July 7, 1989)
@end(titlebox)
@end(titlepage)

@chapter(Introduction)

This manual is intended to be used as an introductory guide for an
Argus programmer who plans to utilize the X Window System.  More
specifically, using winsrv implemented in X11 as an example, it will
illustrate how a programmer may communicate with the window server.

The manual consists of three general sections.  The first section of the
manual explains the fundamental X Window Protocol.  It describes the
standard format used in the communication between the client and the
host display server.  The display server used for this manual is the
X11, Release 2, running on the DEC VAX Station II.  The second section
of the manual goes on to provide specific examples of Argus programs
that communicate with the aforementioned server.  It illustrates how a
client "talks" with the X Window Server, and additionally, describes
step-by-step the procedures that are necessary to startup a winsrv
base window.   The third section deals with the use of X library by
other programmers.  Since the X library procedures are not as flexible
as they can be, any reader planning to use the library must be
familiar with the constraints that are present.

This manual assumes the reader to be familiar with the X Window System
and the Argus System, in particular, the Argus data structures and the
winsrv.  It is not the purpose of this manual to provide detailed and
definitive information on either of them.  There exist more
authoritative documents on both the X Window System and the Argus
System, and the interested readers are advised to consult them.  The
listing of these documents appear at the end of this document.

@chapter(X Protocol, Version 11)

In order for an application program to communicate with the X Window
Server, it must establish a connection via some interprocess
communication path, such as TCP.  Such connections are termed
"clients".  Therefore, any number of connections originating from a
single application program, will be seen as distinct and separate
clients.  Nevertheless, for the purpose of facilitating the
understanding of this concept, we will use "client" in a much looser
sense as referring to the application program itself, with the
implicit assumption that all the clients originating from this
particular program are mutually independent of each other.

The following sections explain some aspects of X11 and the protocol
that are necessary in order to perform fundamental window
manipulations.

@section(Pixmap)

Every "window" created is associated with a number of
@i(pixmaps), which make up its background and border.  A
pixmap consists of three dimensional array of bits.  In other words, it
is a stack of two dimensional pixmaps, or @i(bitmaps), where each
bitmaps are two dimensional arrays of bits representing one of the
number of bits that represent a pixel.  Therefore, the @i(depth) of
the stack, or the number of bitmaps making up a pixmap, determines how
many bits a pixel is represented by.

An easier way of looking at this is to think of the plane of the
screen as the topmost bitmap with the stack growing into the screen.
Thus, for any given pixel on the screen, the x-y plane, its bits are
aligned in the z-direction, where the z-axis points into the screen.

The number of bits effectively decide the color capability.  There are
a number of visual types that are supported by the server, and they
differ in the methods used to translate the values of the pixel bits
to color.

Both the depth and the visual type information is available from the
server when the initial connection is made, and therefore, there is no
difficulty in finding out which values to use.

@section(Colormap and Graphics Context)

The X server uses @i(colormap) and @i(graphics context) for the
purpose of managing color displays and graphics operations,
respectively.  Any number of colormaps and graphics context can be
created, although only one will be associated with a given window or
graphics operation.

@subsection(Colormap)

Colormaps can basically be thought of as an artist's palettes that
contain different groups of color paints.  The color produced by a
given pixel value is determined by the content of the colormap.  Thus,
by changing either the colormap associated with a window, or any
number of entries in an installed colormap, it is possible to change
the colors of the window.

The window server has a default colormap which is automatically
installed during the connection set-up.  Since this colormap already
contains defined color values, it is sufficient for the purpose of
creating windows.  However, a user may create his own colormap if the
need arises.

The user must specify the visual type when creating a colormap.  For
all the @i(static) visual types, the entries in the colormap are
pre-defined and read-only.  For the other classes, the entries are
either non-existent or undefined.  For these types, the colormap
entries must be explicitly defined.  Once created, the colormap must
be "installed" in order to take effect.

@subsection(Graphics Context)

Graphics context is a data structure in the X server that contains
various formats to be used for graphics operations.  Typical
informations stored in the graphics context, also called gcontext or
gc, include line style, line width, shape of the end of a line, how
lines should be joined when they are at an angle, etc.  Every time a
graphics operation such as drawing a box, for example, is performed,
an appropriate graphics context must be called.

Therefore, a user who wants to differentiate windows for different
purposes may have available to him a set of different graphics context
that he can use.  Graphics context, in effect, provides for a
separation of tasks that involve "what" and "how".  Although the
application program may be programmed to create same size window, by
changing the appropriate entries in the graphics context, the user can
very easily change @i(how) the window will look.

However, the various graphics-related operations do not make use of
every entry in the graphics context.  An operation that draws a line
and an operation that fills a rectangle, for example, will share only
a small amount of information from the graphics context.

The server also has provisions for changing the entries in the
graphics context.  This removes the need to have several graphics
contexts when it is more efficient to maintain one graphics context and
changing the entries as required.  The purpose and usage of graphics
context will become clearer when we look at its application.

@section(Connection Set-up)

An application that attempts to communicate to the window server must
first send a byte which specifies the byte order that will be used for
all subsequent data transmissions, except where explicitly noted.

The connection is established when the client sends the correct
protocol version numbers.  The first byte returned from the server
indicates whether the connection was successful or not.  If
successful, the server will also return various information regarding
the limitations and conventions of the server.  Examples of them are
depth, visual types, default colormap, etc.  In most cases, this
information is sufficient enough to allow the client to start making
windows and manipulate them.

Once the connection is established, the user can begin communicating
data with the server.

@section(Data Formats)

All data exchanged between the client and the server are transmitted
as blocks of consecutive bytes.  Any unused bytes are not required to
be zero.

@subsection(Request Formats)

Every @i(request) begins with 4 bytes of header block which consists
of 1 byte of request code, 1 byte of optional data, and 2 bytes of
request length.  The requests are always a multiple of 4 bytes long and
the request length field must correspond to it.

When the server receives the request code, also called the major
opcode, it expects a certain number of data, corresponding to the
given opcode, to follow.  Unless the request length field matches this
number, an error is produced.

For some request, it is necessary to calculate pad bytes to ensure the
length of request to be a multiple of 4 bytes.  All requests are
implicitly assigned a sequence number for identification purposes.

@subsection(Reply Format)

A client may receive information from the server by sending a
particular request that prompts a @i(reply) to be returned.  All
replies are at least 32 bytes long, and a multiple of 4 bytes.  The
reply length field specifies the number bytes beyond the 32nd byte,
and is also a multiple of 4.  A reply always begins with a 1 as its
first byte, followed by 1 byte of optional data, 2 bytes of sequence
number, and 4 bytes of reply length.  The sequence number corresponds
to the request which prompted the reply.  A reply whose first byte is
0 indicates an error.

@subsection(Event Format)

Certain requests, such as that which moves a window, causes the server
to return an @i(event), which, in effect, confirms the request and
indicates changes caused by the request.  All events are 32 bytes
long, and have 1 byte of event code, followed by 1 byte of optional
data, and 2 bytes of sequence number as its header block.  The
sequence number corresponds to the request which caused the event.

@chapter(Winsrv)

In order to get a good idea of how to use the X server using Argus, it
helps immensely to take a look at winsrv procedures.  Since winsrv
creates an artificial window environment on top of the X window
environment, it makes a comprehensive use of the X server's function.
After a brief description of winsrv, several sections follow which
show the process of establishing connection and communicating with
the server.  Any other issues that require attention are also
discussed.

Also note that some of the procedures discussed in this chapter may
@i(not) need to be called explicitly by the programmer.  The
procedures in this chapter simply illustrate the @i(steps) necessary
to get started.  The procedures that a programmer, who is using the X
library of the winsrv, needs to call are discussed in the following
chapter.

@section(Brief Description of Winsrv)

When winsrv is invoked, it creates a window which serves the purpose
of providing a separate environment where windows can be maintained.
Since winsrv is built on top of the X environment, it must be able to
take mouse clicks and perform window manipulations much like the
window manager.@footnote(The window manager here refers to @c(uwm).)
When the alternate key is held down, all mouse clicks
are directed towards the window manager, whereas when the mouse
pointer is inside the winsrv root window and alternate key is not
pressed, all mouse clicks are directed at the winsrv.  The mouse
buttons used for window manipulation in winsrv correspond exactly to
those used for the window manager.

The winsrv procedures are roughly separated into two groups, and their
functions are reflected in the name of the directories they reside in:
@i(winsrv) and @i(x).  The procedures in the @i(winsrv) directory are
the top level procedures which interact with the user and does most of
the work for many window operations.  However, the responsibilities of
some of the procedures in this directory duplicate those that are done
by the window manager as well.  The inefficiency is caused by the fact
that the winsrv was first programmed with X10 in mind, and when it was
updated to run on X11, no significant changes were made to the
internals of winsrv to take advantage of the X11's added features.
Nevertheless, for the purpose of portability, it may be better to
allow winsrv to support itself.

The procedures in the @i(x) directory, on the other hand, act as the
bridge connecting the winsrv procedures and the server.  They perform
the necessary tasks required in order to send a valid request to the
server.  All of the procedures here can be used as X library
subroutines by programs wishing to use the X server, and this is
discussed in more detail in the following chapter.

@section(Resource ID)

When creating windows, pixmaps, graphics context, colormaps, cursors,
and fonts, the client must assign a resource identification number.
The IDs should be unique with respect to all other IDs.  Additionally,
a resource ID is not considered valid unless it is numerically ORed
with the resource-id-mask.  The id-mask is returned during the
connection set-up and the client must retain it for any further use
thereafter.  Since the client must assign an ID to everything it
creates, it should have some method of efficiently maintaining and
assigning valid IDs.

@section(Data Format)

As mentioned earlier, the data are transmitted as blocks over a
network connection.  In argus, this can be achieved using records.
The winsrv uses 2 records, to cover a field that contains 40 bytes of
data.  Each record has 10 fields, each of which take 4 bytes of data.
Then, the records are offset by 2 bytes, allowing the client to
access every 2 bytes, as well as every 4 bytes.  An extra record is
used for the sole purpose of making the latter access easier.  The
pointer for the two records access a single output buffer such that
@i(oreq) is overlapping @i(ereq).

@begin(programexample)
lreq = record[code, dwid, l0, l1, l2, l3, l4, l5, 
	      l6, l7: int]
ereq = record[code, dwid, s00, s02, s04, s06, s08, s10, 
	      s12, s14: int]
oreq = record[length, lgarbage, s01, s03, s05, s07, s09, s11, 
	      s13, s15: int]
 ___________________________________________________
|CODE |DWID |L0  |L1  |L2  |L3  |L4  |L5  |L6  |L7  |
 ===================================================
|CODE |DWID |S00 |S02 |S04 |S06 |S08 |S10 |S12 |S14 |
 ------------------------------------------------------
  | LEN  |LGAR |S01 |S03 |S05 |S07 |S09 |S11 |S13 |S15 |
   ----------------------------------------------------
@end(programexample)

Note that since every field is 4 bytes long, when the client stores
data at 2-byte intervals, 2 bytes of the previous data will be lost.
Therefore, it is obvious that this scheme works with the assumption
that data are transmitted least significant bits first.  This works
well since, when the client stores a 2 byte data into a 4 byte field,
the top 2 bytes are not needed anyway.  Also, to store two 1 byte data,
the second data can be shifted by 8 bits, added to the first data, and
then stored as a 2 byte data.  The X protocol never requires the
client to store a 2 or 4 byte data starting at an odd numbered
position.

The spacing between each field and the offsetting of the two bytevecs
are done using pointers.  The two pointers for ereq and oreq are
placed by an oculus procedures, as shown:

@begin(programexample)
or: oreq, er: ereq := x_buf$get(N)
@end(programexample)

Here, N is the length of the request field.  The data stored are then
sent to the server when the pointer reaches the end of the output
buffer or when x_flush is called.  An example of how data are put into
the output buffer is shown in Section @ref(GC).

@section(Connection Set-up)

In order to establish connection with the server, the client must
first establish a TCP connection by specifying the location of the
server and the display.  This is shown in
@b[x_display$init(@c(host))], which will not be reproduced here.  The
2 byte address family indicates the location of the client, followed
by a 2 byte port number, which is 6000 + N, where N is the display
number as specified by @c(host).  Finally, a 4 byte address of the
server completes the standard UNIX connection protocol.

The winsrv then specifies "little-endian" transmission, followed by
the major and minor protocol versions.  Unless the major version
supplied by the client matches the server's, connection will fail.  The
server may or may not grant connection when the minor version is
different.  For our winsrv, the version numbers of 11 and 0,
respectively, works.  The server also expects an optional
authorization protocol name and data.  The winsrv does not make use of
this additional authorization device, and the length of such name and
data are 0.  All data are 2 bytes long, except the first, which is 1
byte long.  This is shown below.

@begin(programexample)
strt: bv := bv$create(12)
bv$bstore(strt, 1, 108)	% Indicates LSB first
bv$bstore(strt, 2, 0)	% Unused byte; need not be zero
bv$wstore(strt, 3, 11)	% Major protocol version
bv$wstore(strt, 5, 0)	% Minor protocol version
bv$wstore(strt, 7, 0)	% authorization name length
bv$wstore(strt, 9, 0)	% authorization data length
bv$wstore(strt, 11, 0)	% Unused bytes; need not be zero
x_buf$send_data(strt, 1, 12) % Send _bytevec to output buffer
x_buf$flush()		% Send data
@end(programexample)

The first byte returned by the server indicate whether the connection
attempt was successful or not; 1 for success, and 0 for failure.
Therefore, it is necessary to check this byte before proceeding.
If everything is successful, the server returns a number of
information, most of which are not necessary for typical operations.
There are several information that must be retained, however.  The
first step in reading these information is to put the data into a
bytevec, which facilitates access to them greatly.  Once this is done,
the client can read the contents using bytevec fetch functions.

@begin(programexample)
conn: bv := bv$create(300)	% Create bytevec
x_buf$receive_auth(conn)	% Put data into bytevec
if bv$bfetch(conn,1) = 0	% Check for failure and signal
   then signal error("") end	%  error.
rid := bv$lfetch(conn, 13)	% Resource-id-mask
keymin := bv$bfetch(conn, 29)
keymax := bv$bfetch(conn, 30)
base := i2w(bv$lfetch(conn, offset))
cmap := bv$lfetch(conn, offset + 4) % Default colormap
wpixel := bv$lfetch(conn, offset + 8)
bpixel := bv$lfetch(conn, offset + 12)
rwidth := bv$wfetch(conn, offset + 20)
rheight := bv$wfetch(conn, offset + 22)
rvisual := bv$lfetch(conn, offset + 32)
numplanes := bv$bfetch(conn, offset + 38)
numcells := 3
@end(programexample)

The offset was calculated using the length of vendor, the length of pad
bytes, the number of screens, and the number of formats.

@section(Creating a Graphics Context)
@label(GC)

The winsrv creates two graphics contexts during its initialization
stages.  One is a full-depth graphics context, and the other is a
depth-1 graphics context used for making bitmaps.  Since the winsrv
does not create a separate graphics context for every request that
requires one, and instead uses one graphics context for different
purposes by storing new entries every time it is used, the graphics
context initially created contains default values. This procedure is
illustrated in @b(x_window$init):

@begin(programexample)
own rid: int := x_display$resource_mask()
own GCid: int := _obj$ori(rid, 127)

init = proc (parent: cvt)
	or: oreq, er: ereq := x_buf$get(4)
	er.code := xproto$CreateGC
	or.length := 4
	er.dwid := GCid
	er.s00 := parent
	er.s02 := 0
	end init
@end(programexample)

Examining the above procedure, he first two lines above show how a
resource ID is created.  After the pointers @b(or) and @b(er) have
been set, we can start storing the appropriate data into the output
buffer.  The first is the major opcode, followed by an optional data,
which in this case is unused.@footnote(All the opcodes are stored in
@b(xproto.arg), and are accessed by their names according to @u(X
Protocol, Version 11, Revision 2) by Robert Scheifler.)  The length of
the request is 4, followed by the ID for the graphics context to be
created.  The graphics context must also be specified to be used for
a particular type of drawables, either a window or a pixmap, and in this
case, it should be a window, or more specifically, the parent window
that was passed down as an argument.  The next field is a value-mask,
where the client can enter mask values to specify particular fields
whose content it wants to change or store.  Since the winsrv changes
the contents whenever the graphics context is used, nothing is done at
this point, as the contents of graphics context will reflect the
default values.

Noticing that the last 3 entries are each 4 bytes long, it is also
possible to store the data using the record created specifically for
this purpose.  The example below shows its use:

@begin(programexample)
	lr(er).dwid := GCid
	lr(er).l0 := parent
	lr(er).l1 := 0
@end(programexample)

The choice is arbitrary and it is up to the programmer to choose
the method he likes better.

@section(Creating the Winsrv Base Window)
@label(cons)

Creating the winsrv's base window takes considerably more work than
creating any other window subsequent to it.  This is mainly because
many of the initializing processes take place at this point.
Additionally, when winsrv is invoked, a flashing rectangular box is
created on the screen until a mouse button has been pressed and
released.  This procedure requires several steps to be taken to
produce the window of expected dimension at an expected location.
Here, the "expected" dimension and location are the dimension and
location specified by the arguments to winsrv or by the actions of
the mouse.

Further, initializing processes include creating border and background
pixmaps with the appropriate colors, creating temporary cursor and
font, and checking for valid dimensions.  Most of these processes take
place in @b(x_cons.arg), with some preparatory steps in @b[init()]
within @b(top.arg), a file in winsrv directory.  @b(X_cons) takes as
arguments 2 pixmaps for background and border, border width, minimum
width and height, a name for the base window, and a string of
client-specified dimension and location.  All of these arguments are
provided by @b[init()].

In order to create pixmaps, @b[init()] first checks the default file
for color entry.  If it exists, it can be parsed into red, green, and
blue components, and then stored in a colormap.  Once this is done, a
pixmap can be made.  The necessary lines showing the above steps are
shown below, where @b(@c(N)) is less than or equal to 4:

@begin(programexample)
prog: string := program_name()
cbdr: string := x_default(prog, "Border")
pixs: xdefs$pixellist, planemask: int := 
			x_display$alloc_cells(4, 1, false)
bdrpix := pixs[N]
r, g, b: int := x_parse_color(cbdr)
x_display$store_color(bdrpix, r, g, b)
border: x_pixmap := x_pixmap$tile(bdrpix)
@end(programexample)

The dimension and location of the base window is read from the string
arguments used at the time of winsrv invocation.

@begin(programexample)
defgeom: string := "=" ||
	       int$unparse(x_display$width() - 2 * bw - 2) ||
	       "x" ||
	       int$unparse(x_display$height() - 2 * bw - 2) ||
	       "+1+1"
@end(programexample)

If no arguments were given, then the string is empty and the width and
height take on the default values.  However, these default values
actually represent the @i(minimum) values, which may be overrided by
the actions of the mouse.  When @b(x_cons) is done retrieving the
arguments, it immediately verifies that they are valid.  If the client
specified the location of the window when invoking winsrv, then the
window is automatically created, and @b(x_cons) exits.  However, as
may be observed from the last line of the code above, winsrv's X
library does not support this feature.

Therefore, assuming that the location of the window has not been
specified, the @b(x_cons) must create a flashing box and wait for the
user to press and release a mouse button.  He may, at this point,
"drag" the window while holding down the middle button, or create a
full screen size window by clicking on the other two
buttons.@footnote(Button clicks, "dragging" the window, or any other
set of actions on the mouse that causes a flashing box to change to a
window will be called a "mouse action".)  While the box is flashing,
there are three things that take place.  Besides the flashing box,
there is also a cursor at the upper left corner of flashing box, and a
highlighted string, specified by the argument @c(name), at the upper
left corner of the screen.  For these, a font and a cursor must be
created.

@subsection(Creating Font and Text)

Creating a font is done relatively easily, as all that's required is
to read if any font is specified by the user, and sending a request
to the server.  This is shown below:

@begin(programexample)
fn: string := x_default(prog, "MakeWindow.BodyFont")
	except when not_found:  fn := "8x13" end
font: x_font := x_font$create(fn)
fwidth, fheight: int, f, l: char,
	bs: int, x: bool := x_font$query(font)
popw: int := x_font$width(font, name) + 2 * ibw
poph: int := fheight + 2 * ibw    % IBW: internal border width
@end(programexample)

The last 3 lines illustrate a few of the way informations about font
size can be obtained and used.  The @b(x_cons) uses these for the
purpose of blocking the @c(name), creating a "highlighting" effect.
To actually display the string, ImageText8 request is used, which is
accessed as follows:

@begin(programexample)
root: x_window := x_display$root()
pop := x_window$create(0, 0, popw, poph, backmap, root,
			pbw, bdrmap)
x_window$map(pop)
x_window$text(pop, name, font, pfore, pback, ibw, fheight - 3)
@end(programexample)

@c(Pop) is the window which will be enclosing the string of characters.
As can be seen above, there are two steps in putting a window on the
screen.  After "creating" the window, it must then be "mapped", only
after which the window becomes visible on the screen.  @c(Backmap) and
@c(bdrmap) are pixmaps for background and border, respectively, and
@c(root) is the root window of the screen.  @c(Pbw) is the border
width.  @c(Pfore) and @c(pback) respectively represent the foreground
and background color pixels.  Calling @b(x_window$text) will write the
string specified by @c(name) at coordinates given by @c(ibw) and
(@c(fheight) - 3) in the window @c(pop).

@subsection(Creating Cursor)

There are two cursor shapes that are used by the winsrv.  The cursor
that appears along with the flashing box is a cross-shaped cursor,
while the cursor used during normal winsrv operation is an
arrow-shaped one.  The shape of the cursors are all user-defined in a
separate file which, in winsrv's case, is @b(cursor.arg).  The
following line shows how a cursor may be created:

@begin(programexample)
cr: x_cursor := x_cursor$cons(cursors$cross_width,
		     cursors$cross_height,
		     cvt[string,_bytevec](cursors$cross),
		     cvt[string,_bytevec](cursors$cross_mask),
		     mfore, mback, cursors$cross_x,
		     cursors$cross_y, xdefs$GXcopy)
@end(programexample)

This invokes a series of procedures which prepare the arguments
required by CreateCursor request.  These include the image and mask
bitmaps, parsed color values for the foreground and background, and
the coordinates of the cursor's hotspot.  

@subsection(Drawing the Box)

When drawing a box, the coordinates corresponding to the corners of
the box must be provided as an argument to the PolyLine request.
Calling @b(x_window$draw), as below, starts a process which will first
create an appropriate graphics context by storing new entries, and
then send the PolyLine request.

@begin(programexample)
for i: int in int$from_to_by(1, count, 4) do
   x_vlist$store(box, i, xa, ya, 0)
   if i = count then break end
   x_vlist$store(box, i + 1, xb, ya, 0)
   x_vlist$store(box, i + 2, xb, yb, 0)
   x_vlist$store(box, i + 3, xa, yb, 0)
end
x_window$draw(root, box, count, pback, 1,1,xdefs$GXinvert,1)
@end(programexample)

@c(count) represents the number of points that are stored, and in the
case of a rectangle, is 5.  @b(X_vlist$store) stores each of its two
entries, the coordinate pair, as 2 byte entities.  The third
information is a flag, and is not used in winsrv.

To create the flashing effect, another box with the same dimension and
coordinates must be created, with graphics context function being
Invert.  This will have the effect of making the box appear as
flashing by changing its colors.

@b(X_cons) will continue flashing the box until there is mouse action.
Depending on the particular action, a dimension for the base window
will be chosen.  Using these dimensions, along with background and
border pixmaps that @b(x_cons) received as arguments, a window is
created.  @b(X_cons) then returns this window with its dimensions and exits.
@b[Init()] then maps this window on the screen.

@chapter(Using the X Library)

A programmer who chooses to use the X library used by winsrv, instead
of writing his own, may do so with relative ease.  Although many of
the procedures are self-explanatory, the X library as yet is not
totally flexible, and the programmer will need to be aware of the
conventions used within the X library.  First, a description of
general organization of the X library will be given.

@section(Organization of the X Library)

@begin(itemize)
@b(@c(x_display))@*
This cluster contains a procedure which performs the connection
set-up.  It takes a display host as its argument and reads various
information returned by the server.  The other procedures in this
cluster are used to access these stored informations.  Generally,
@b[x_display$init(@c(host))] will be the first call a client will make
to the server.

@b(@c(x_cons)) and @b(@c(x_tcons))@*
@b(X_cons) is used when creating the base window for winsrv.  It's
functions have already been discussed in the previous section.  Also
considered part of the initialization, the winsrv calls this procedure
from its @b[init()] procedure, after @b(x_display$init) has been
called.@*
@b(X_tcons) is similar to @b(x_cons) in that it also creates a
flashing box, and then a window upon a mouse action.  The difference
in the two procedures are in the fact that the windows created by
@b(x_tcons) are geared towards text I/O, and therefore, has dimensions
that are some multiple of the font height and width.

@b(@c(x_geometry))@*
This procedure takes two strings, one of which contains the standard
geometry-specifying format, and returns the coordinate and the
dimension as integers, in addition to the three boolean expressions,
one of which indicates whether a location was specified or not.  This
procedure is most likely to be used only during the initialization
stages.

@b(@c(x_input))@*
This cluster contains the procedures that deal with input from the
user and the server.  Calling @b[x_input$pending()] will return a
boolean expression indicating whether an input was received or not.
@b(X_input$deq) will dequeue an event returned by the server from the
input buffer, and store it in a pre-defined record that corresponds to
the event structure.

@b(@c(x_bitmap, x_font, x_misc, x_pixmap, x_window))@*
These clusters contain additional procedures that send requests to the
server.  In many cases, the procedures will send one or more requests
before the primary request, in order to prepare the server to perform
the action specified by the client.  The most typical case of this is
to store new entries into a graphics context.  This also implies that
the procedures may be required to do some basic operations to derive
the appropriate arguments required by the server.  The procedures are
grouped according to their related functions.

@end(itemize)

@section(Winsrv X Library Conventions)

As long as the programmer is aware of the restrictions of the winsrv X
library, it is possible for him to simplify his programs
significantly.  Many of the procedures discussed in the previous
chapter need not be explicitly called, for example.  The reader can
get a good idea of how to initialize and create a window by examining
@b(top.arg) in the @i(winsrv) directory.  Nonetheless, some of the
basic issues are described below.

@subsection(@c(x_display$init))
@begin(programexample)
init = proc (display: string) signals (error(string))
@end(programexample)

This should be one of the first calls to be made by the client.  Its
argument is the host display number.  If a null string is passed down
as an argument, then a pre-defined @c(display) environment is used.
If this informations is unavailable, then error is returned.  After
connection is made and essential information retained, 2 other
initialization procedures are called.

The first of these calls are @b[x_window$init(parent)], where the root
window of the screen is used as @c(parent).  This procedure will
create 2 graphics contexts, where one is full depth and the other 1
plane deep.  The 1 plane deep graphics context is used only for bitmap
manipulations by procedures in @b(x_bitmap.arg) and @b(x_cursor.arg).

It is important to note that these two graphics contexts are the only
graphics contexts created by the X library.  Although CreateGC request
exists, direct access to it was not made possible.  Part of the
complication of allowing such access would be the added responsibility
of the clients to maintain the valid and appropriate ID numbers for
all the graphics contexts created.

The second initialization call made is @b[x_input$init()].  This
procedure resets a number of flags and prepares the input queue.

Additionally, as part of retaining the information returned by the
server during the connection set-up stage, @b(x_display$init) will
choose the default colormap returned by the server as the designated
colormap.  Again, this is the only colormap that will be used by the X
library and there is no provision to create another colormap, for the
same reason cited above.

@subsection(@c(x_cons) and @c(x_tcons))
@begin(programexample)
x_cons = proc (name: string, back, border: x_pixmap,
		spec, defspec: string, minwidth: int,
		minheight, bwidth: int)
	returns (x_window, int, int)
@end(programexample)

This procedure's functions have been described in detail in Section
@ref(cons), along with how to create the background and border
pixmaps.  The programmer should keep in mind that this procedure is
used to create only the base window.  To create windows where text I/O
will take place, there is @b(x_tcons.arg) which will create a window
whose size is a function of the font size.  Although winsrv does not
call @b(x_tcons), all clients that puts an input/output window on
top of the winsrv base window makes a call to this procedure.

Both @b(x_cons) and @b(x_tcons) only "create" a window, leaving it up
to the client to explicitly "map" them.

@subsection(@c(x_input))

There are three types of information that a client will explicitly ask
for.  They are the information returned by the server during the
connection set-up, the @i(replies) from the server regarding the
client's query on some particular information, and @i(events), which
may include notification from the server on the changes of the screen
display and inputs from the keyboard and mouse.  These types of
information are retrieved distinctively from each other, as described
below.

A special procedure was written for the exclusive purpose of
retrieving data returned by the server during the connection set-up
stage.  This was made necessary due to the irregular data structure of
the returned information.  This procedure, @b[x_buf$receive_auth(b)],
where b is a bytevec, will read the data from the queue and store them
into the specified bytevec.  However, this procedure need not be
called explicitly by the client, since @b(x_display$init) does.

Whenever a user calls an X library procedure that sends a request for
some information, the procedure also calls @b[x_buf$receive_data(b)]
immediately afterwards, where b is a bytevec into which the data will
be stored.  The main function of @b(x_buf$receive_data) is to gather
the information returned by the server, and store it in the specified
bytevec.  However, if it determines that the data returned is an
event, it will then call @b[x_buf$enq], which removes the event
from the input buffer, and places it into an unspecified event record,
which it saves for later processing by calliing @b[x_input$enq()].
After this, @b(x_buf$receive_data) will try to read the reply once
more.

Retrieving the event requires an explicit call by the client, as well
as a number of more internal procedure calls.  The procedure that
needs to be explicitly made is @b[x_input$deq].  However, the client
should first make a call to @b[x_input$pending()], which returns a
boolean expression indicating whether the queue is empty or not.  Only
if the queue is non-empty should the client try to move the event from
the queue and store it in a record.  This last step is done by calling
@b[x_input$deq(r)], where r is the record into which the event will be
stored.

@subsection(Miscellaneous)

As noted above, the coordinates of a window cannot be specified as an
argument at the command line.  This is due to the particular
implementation of the @b(x_geometry.arg) procedure.

There are number of requests which have not been implemented in the X
library.  All of them were requests not used by winsrv, with many
being requests that one would not use for basic functions.

Winsrv's X library routines can be a valuable help to the programmers
if they use it with care; nevertheless, the library stands as being
not fully flexible and the programmer must be aware of its
restrictions.

@chapter(References)

Any reader wishing to obtain more specific information about argus and
X server is advised to refer to the following sources:
@skip(1)
@begin(itemize)
Liskov, Barbara, et al.  @u(Argus Reference Manual), Programming
Methodology Group Memo 54, Laboratory for Computer Science,
Massachusetts Institute of Technology, Cambridge, MA 02139, March 1987.

Closkey, Cynthia.  @u(Argus System Manual), Programming Methodology
Group, Laboratory for Computer Science, Massachusetts Institute of
Technology, Cambridge, MA 02139, 1988.

Scheifler, Robert.  @u(X Window Protocol, Version 11, Revision 2),
Laboratory for Computer Science, Massachusetts Institute of
Technology, Cambridge, MA 02139, 1988.

@end(itemize)
@end(text)

