

%This cluster implements Mercury-level connections, on top of tcp connections


connection = cluster is is_open, equal, send_reply, create, getb,
                        connect, do_rpc
	       

out_queue = mutex[array[out_msg]]
aom = array[out_msg]

rep = record[tcp:btcp,
             is_open: mutex[bool],
	     current_stream:maybe[stream],	%server side
	     current_port:maybe[port],		%server side
	     connected_to: maybe[module_id],	%value only after connect msg
	     all_streams:array[stream],
	     out_q: out_queue,
	     tq:trigger_queue,
	     resp_stream:maybe[stream_id],
	     resp_queues:array[resp_q]]
             
out_msg = _obj

stream = _obj

port = _obj

in_msg = _obj





dispatch_tab = array[proctype(connection, in_msg)signals(break_connection)]

mm = maybe[in_msg]
mp = maybe[port]
ms = maybe[stream]
connect_op = 0

agent_id = int

seq_id = struct[local_id:int,		
                netid:_bytevec]
		
stream_id = struct[agent:agent_id,
                   seqid:seq_id]
		   
resp_q = record[agent:agent_id,			%impl. depends on agentid=processid
                msg: maybe[in_msg],
		tq:trigger_queue]

conns_list = mutex[array[connection]]

own optable:dispatch_tab := dispatch_init()
own all_conns:conns_list := conns_list$create(array[connection]$new())

create = proc(tcp_port:btcp) returns (connection)
                             signals (not_possible(string))
			            

  c:connection := up(rep${tcp:tcp_port,
                is_open: mutex[bool]$create(false),
                current_stream:ms$none(),
		current_port:mp$none(),
		all_streams:array[stream]$new(),
		connected_to: maybe[module_id]$none(),
		out_q:out_queue$create(aom$new()),
		tq:trigger_queue$create(),
		resp_stream:maybe[stream_id]$none(),
		resp_queues:array[resp_q]$new()})
		
  %wait for mercury connect first
  m:in_msg := in_msg$next(c)
     except when not_possible(err:string): signal not_possible(err)
            when end_of_file: signal not_possible("end of file")
	    end

  if m.opcode ~= connect_op 
     then signal not_possible("no mercury connect")
     end
  dest:module_id := m.dest
  bv:_bytevec := out_msg$m2bv(out_msg$make_connect_nack())
  sz:int := _bytevec$size(bv)
  if dest ~= mercury$get_module_id()
     then btcp$putb(tcp_port, bv, sz)
              resignal not_possible
	  signal not_possible("bad connect message")
     end
  seize all_conns do
  ac:array[connection] := all_conns.value
  for conn:connection in array[connection]$elements(ac) do
      if maybe[module_id]$value(down(conn).connected_to) = m.source
         then close(conn)
              btcp$putb(tcp_port, bv,sz) 
                   resignal not_possible
	      signal not_possible("duplicate connection")
         end
      end %for
  end %seize
  
%good connection
  ct:rep := down(c)
  seize ct.is_open do
  ct.is_open.value := true
  ct.connected_to := maybe[module_id]$make(m.source)
  seize all_conns do
    ac:array[connection] := all_conns.value
    array[connection]$addh(ac, c)
    end
  fork sender(c)
  fork manager(c)
  send(c, out_msg$make_connect_ack())
  return(c)
  end  
  end create


%this is the process code that monitors each connection
manager = proc(c:connection)
  while true do
   m:in_msg := in_msg$next(c)
   opcode:int := m.opcode
   optable[opcode](c, m)
     except when break_connection:
                 close(c)
		 return
		 end
   end
     except when not_possible(err:string):
         close(c)
	 return
       when end_of_file:
         close(c)
	 return
	 end
  end manager

is_open = proc(c:cvt) returns(bool)
     seize c.is_open do
     return(c.is_open.value)
     end
   end is_open
   

close = proc(ct:cvt)
     seize ct.is_open do
       ct.is_open.value := false
       for s:stream in array[stream]$elements(ct.all_streams) do
         stream$close_stream(s)
         end
       trigger_queue$trigger(ct.tq)		%triggers sender

       seize all_conns do
         ac:array[connection] := all_conns.value
         for index:int in array[connection]$indexes(ac) do
          conn:connection := ac[index]
          if conn = up(ct) then 
	    array_delete[connection](ac, index)
            break
	    end %if
	  end %for
         end %seize
       btcp$close(ct.tcp)       
       end %seize
  end close

equal = proc(c1:cvt, c2:cvt) returns (bool)	%assumes conn. obj's never 
   return(c1.connected_to = c2.connected_to)    %leave module & no duplicates
   end equal
   

getb = proc(c:cvt, size:int) returns (_bytevec) 
         signals (end_of_file, not_possible(string))
       buf:_bytevec := _bytevec$create(size)
       actual:int
       tcp_port:btcp := c.tcp
       actual := btcp$getb(tcp_port, buf)	%reads concurrently with writing
	   resignal not_possible, end_of_file
       if actual ~= size then signal not_possible("INCOMPLETE") end
       return(buf)
       end getb
       
send = proc(c:cvt, m:out_msg) signals (not_possible(string))
       rq:out_queue := c.out_q
       am:array[out_msg]
       seize c.is_open do
         if ~c.is_open.value then signal not_possible("connection broken") end
         end      
       seize rq do
         am := rq.value
	 array[out_msg]$addh(am, m)
         trigger_queue$trigger(c.tq)
	 end
       end send

sender = proc(c:cvt)
     while true do
         trigger_queue$wait(c.tq)
         seize c.is_open do
           if ~ c.is_open.value then return end	 
	   seize c.out_q do				%one write at a time
             m:out_msg := aom$reml(c.out_q.value)
   	     if aom$empty(c.out_q.value) then trigger_queue$reset(c.tq) end
             b:_bytevec := out_msg$m2bv(m)
	     btcp$putb(c.tcp, b, _bytevec$size(b))
	       except when not_possible(reason:string): 
	                          close(up(c))
				  end
	     end %seize
           end %seize
	 end %while
	end sender
	
send_reply = proc(c:cvt, m:out_msg, rs:stream) signals (not_possible(string))
       seize c.is_open do
        if ~c.is_open.value then
          signal not_possible("connection broken")
	  end
        end
       set_respstr:out_msg := out_msg$make_response_stream(rs.seqid,rs.agent_id)
       seize c.out_q do
         q:aom:= c.out_q.value
	 aom$addh(q, set_respstr)
	 aom$addh(q, m)
	 trigger_queue$trigger(c.tq)
	 end
       end send_reply




%server side message handling

disconnect = proc(c:connection, m:in_msg) signals (break_connection)
       send(c,out_msg$make_disconnect_nack())
    end disconnect
    
keepalive = proc(ct:cvt, m:in_msg) signals (break_connection)
    seize ct.is_open do
      if ct.is_open.value then send(up(ct),out_msg$make_keepalive_ack()) end
      end
    end keepalive
    
request_stream = proc(ct:cvt, m:in_msg) signals (break_connection)

    for s:stream in array[stream]$elements(ct.all_streams) do
       if seqid_equal(s.seqid, m.seqid) cand s.agent_id = m.agentid
          then ct.current_stream := maybe[stream]$make(s)
               return
	  end
       end     
    ct.current_stream := 
      maybe[stream]$make(stream$create(up(ct), m.seqid, m.agentid))      
    array[stream]$addh(ct.all_streams, ms$value(ct.current_stream))
    fork stream$manager(ms$value(ct.current_stream))
    end request_stream

    
invoke = proc(c:cvt, m:in_msg) signals (break_connection)
    pid:int := m.portid
    for p:port in sequence[port]$elements(mercury$get_ports()) do
        if p.id = pid
	   then c.current_port := mp$make(p)
                return
	   end
       end
      c.current_port := mp$none()
    %Can't respond to bad port until request comes in
 
    end invoke
    
request_data= proc(ct:cvt, m:in_msg) signals (break_connection)
    st:stream := ms$value(ct.current_stream)
        except when wrong_tag: signal break_connection end
    if stream$cm_exists(st) then signal break_connection end   
    begin
      p:port := port$copy(mp$value(ct.current_port))
      p.seqid := st.seqid
      in_msg$set_dest_port(m, mp$make(p))	%set in first message fragment only
      end except when wrong_tag: in_msg$set_dest_port(m, mp$none()) end
    in_msg$set_stream(m, st)
    if in_msg$is_last(m)
       then stream$enqueue(st, m)
       else st.current_message := m
       end
    end request_data

request_continue = proc(c:cvt, m:in_msg) signals (break_connection)
    st:stream := ms$value(c.current_stream)
       except when wrong_tag: signal break_connection end
    in_msg$append(st.current_message, m)
      except when none: signal break_connection end
    if in_msg$is_last(m)
       then stream$enqueue(st, st.current_message)
            stream$clear_current_message(st)
       end
    end request_continue
    
       
bad_command = proc(c:connection, m:in_msg) signals (break_connection)
       signal break_connection
       end bad_command
       
restart = proc(c:cvt, m:in_msg) signals (break_connection)
       seq:seq_id := m.seqid
       aid:int := m.agentid
       strm:stream
       for s:stream in array[stream]$elements(c.all_streams) do
           if seqid_equal(s.seqid, seq) cand s.agent_id = aid
	      then stream$reset(s)
	           return
	      end
	   end
       %okay to restart non-existent stream
       s:stream := stream$create(up(c), seq, aid)
       array[stream]$addh(c.all_streams, s)
       fork stream$manager(s)
       end restart
       
broken_ack = proc(c:cvt, m:in_msg) signals (break_connection)
       seq:seq_id := m.seqid
       aid:int := m.agentid
       strm:stream
       for s:stream in array[stream]$elements(c.all_streams) do
           if seqid_equal(s.seqid, seq) cand s.agent_id = aid
	      then stream$reset(s)
	           return
	      end
	   end
       %what to do if no such stream??
       return
       end broken_ack
       
request_abort = proc(c:cvt, m:in_msg) signals (break_connection)
       s:stream := ms$value(c.current_stream)
          except when wrong_tag: return end
       stream$clear_current_message(s)
       return
       end request_abort
		 
%client side message handling

connect = proc(tcp:int, netid:_bytevec, m:module_id) returns (connection)
            signals (not_possible(string))
  newc:connection
  b:btcp
  seize all_conns do
   ac:array[connection] := all_conns.value
   int_addr:int := _bytevec$lfetch(netid, 1)
   for c:connection in array[connection]$elements(ac) do
      dc:rep := down(c)
      net, rest, foreign:int := btcp$get_foreign_address(dc.tcp)
      if foreign = tcp cand m = maybe[module_id]$value(dc.connected_to)
         then return(c) end
      end
   %no existing connection
   b := btcp$open(int_addr, tcp)
     resignal not_possible
   newc := up(rep${tcp:b,
                          is_open: mutex[bool]$create(false),
                          current_stream:ms$none(),
		          current_port:mp$none(),
		          all_streams:array[stream]$new(),
		          connected_to: maybe[module_id]$make(m),
		          out_q:out_queue$create(aom$new()),
		          tq:trigger_queue$create(),
			  resp_stream:maybe[stream_id]$none(),
			  resp_queues:array[resp_q]$new()})
   array[connection]$addh(ac, newc)
   end %seize
   cm:_bytevec := out_msg$m2bv(out_msg$make_connect(mercury$get_module_id()))
   btcp$putb(b, cm, _bytevec$size(cm))
   ack:in_msg := in_msg$next(newc)
   if ack.opcode ~= merc$connect_ack_op 
     then signal not_possible("can't connect") end
   seize down(newc).is_open do
     down(newc).is_open.value := true
     end
   fork manager(newc)
   fork sender(newc)
   return (newc)
   end connect


do_rpc = proc( conn:cvt, pt:port, msg:out_msg) returns (in_msg)
		signals (exception_reply(in_msg), unavailable)
%assumes no streaming - next reply matches this request   
   agent :agent_id := _process$current_id()
   seqid:seq_id := pt.seqid
   
   begin
   seize conn.is_open do
        if ~conn.is_open.value then
          signal unavailable
	  end
        end
   set_strm:out_msg := out_msg$make_request_stream( agent, seqid)
   set_port: out_msg := out_msg$make_invoke(pt)
   seize conn.out_q do
     q:aom:= conn.out_q.value
     aom$addh(q, set_strm)
     aom$addh(q, set_port)
     aom$addh(q, msg)
     trigger_queue$trigger(conn.tq)
     end
   end except when not_possible(s:string): close(up(conn))
                                           signal unavailable
					   end
   m:in_msg := await_reply(conn, agent) resignal exception_reply, unavailable
   return(m)
   end do_rpc
   
await_reply = proc(conn:rep, agent:agent_id) returns (in_msg)
              signals (exception_reply(in_msg), unavailable)
   respq:resp_q
   begin
   for rq:resp_q in array[resp_q]$elements(conn.resp_queues) do
       if rq.agent = agent
          then respq := rq
	       exit found
          end
       end	       
   respq := resp_q${agent:agent, 
                    tq:trigger_queue$create(),
		    msg: mm$none()}
   array[resp_q]$addh(conn.resp_queues, respq)
   end except when found: end
   trigger_queue$wait(respq.tq)
   msg:in_msg := maybe[in_msg]$value(respq.msg)
     except when wrong_tag: signal unavailable end
   respq.msg := mm$none()
   if msg.opcode = merc$response_data_op then return(msg)
      elseif msg.opcode = merc$response_exception_op
        then signal exception_reply(msg) 
      end
   end await_reply
     
   
response_stream = proc(c:cvt, m:in_msg) signals (break_connection)
  c.resp_stream := maybe[stream_id]$make(stream_id${ agent:m.agentid, 
                                                     seqid:m.seqid})
  end response_stream
  
restart_ack = proc(c:cvt, m:in_msg) signals (break_connection)  
  end restart_ack
  
response_data = proc(c:cvt, m:in_msg) signals (break_connection)

  respq:resp_q
  agent:agent_id := maybe[stream_id]$value(c.resp_stream).agent
    except when wrong_tag:signal break_connection end
  begin
  for rq:resp_q in array[resp_q]$elements(c.resp_queues) do
       if rq.agent = agent
          then respq := rq
	       exit found
          end
       end	       
  signal break_connection	%should never get response for unknown agent
  end except when found: respq.msg := mm$make(m) 
                         trigger_queue$trigger(respq.tq)
			 end
  return
  end response_data
  
response_exception = proc(c:cvt, m:in_msg) signals (break_connection)
  respq:resp_q
  agent:agent_id := maybe[stream_id]$value(c.resp_stream).agent
      except when wrong_tag: signal break_connection end
  begin
  for rq:resp_q in array[resp_q]$elements(c.resp_queues) do
       if rq.agent = agent
          then respq := rq
	       exit found
          end
       end	       
  signal break_connection
  end except when found: respq.msg := mm$make(m) 
                         trigger_queue$trigger(respq.tq)
			 end
  return
  end response_exception
  
response_continue = proc(c:cvt, m:in_msg) signals (break_connection)  
  %not expected from argus server
  signal break_connection
  end response_continue
  
response_error = proc(c:cvt, m:in_msg) signals (break_connection)  
  signal break_connection
  end response_error
  
response_abort = proc(c:cvt, m:in_msg) signals (break_connection)  
  signal break_connection
  end response_abort
  
  

dispatch_init = proc() returns (dispatch_tab)
  dt:dispatch_tab := dispatch_tab$fill(0, 133, bad_command)
  dt[0] := bad_command		%connect - but shouldn't happen here
  dt[1] := bad_command		%connect_ack
  dt[3] := bad_command		%connect_nack
  dt[4] := disconnect
  dt[5] := bad_command		%disconnect_ack
  dt[7] := bad_command		%disconnect_nack
  dt[10] := keepalive
  dt[11] := bad_command		%keepalive_ack
  dt[32] := request_stream
  dt[33] := response_stream
  dt[34] := restart
  dt[35] := restart_ack
  dt[64] := invoke
  dt[97] := request_data
  dt[104] := request_abort
  dt[111] := request_continue
  dt[113] := response_data
  dt[114] := response_exception
  dt[116] := response_error
  dt[120] := response_abort
  dt[127] := response_continue
  dt[129] := bad_command	%send_ack
  dt[130] := bad_command	%send_exception
  dt[132] := broken_ack
  return(dt)
  end dispatch_init  

		       
  
end connection

seqid_equal = proc(s1:seq_id, s2:seq_id) returns(bool)
  seq_id = struct[local_id:int, netid:_bytevec]
  
  return(s1.local_id = s2.local_id cand _bytevec$similar(s1.netid, s2.netid))
  end seqid_equal
  