stream = cluster is create, manager, reset, break_stream, get_agent_id, get_seqid,
                    enqueue, get_current_message, set_current_message,
		    clear_current_message, equal, cm_exists, close_stream


seq_id = struct[local_id:int, netid:_bytevec]
puttype = proctype(out_msg) returns (out_msg)

connection = _obj

in_msg = _obj

out_msg = _obj

rq = mutex[array[in_msg]]		 
rep = record[request_queue: rq,
             trigger:trigger_queue,
             seqid: seq_id,
	     agent_id:int,
	     broken:bool,	%used to check for reset
	     close:bool,
	     conn:connection,
	     current_message:maybe[in_msg],
	     restart_cnt:int,
	     synch_flag:bool]

create = proc(c:connection, seqid:seq_id, agent_id:int) returns (cvt)
      return( rep${request_queue:rq$create(array[in_msg]$new()),
                   trigger:trigger_queue$create(),
                   seqid:seqid, 
	           agent_id:agent_id, 
	           conn:c,
	           broken:false,
		   close:false,
		   current_message:maybe[in_msg]$none(),
		   restart_cnt:0,
		   synch_flag:false})
      end create		   

	     
	     
%process body to execute requests on this stream. to be forked by
%connection process only. doesn't ever terminate

manager = proc(s:cvt)
    req:in_msg
    while true do
       trigger_queue$wait(s.trigger)
       if s.close then return end
       while s.restart_cnt > 0 do	%assumes messages  before  restart  are
				        %deleted before cnt incremented
          s.broken := false   	%might allow useless work to proceed
          connection$send_reply(s.conn,out_msg$make_restart_ack(up(s)),up(s))
	  s.restart_cnt := s.restart_cnt -1
	  end
       reply:out_msg       
       if s.broken then 
            p:puttype := bind out_msg$string_put(*, "stream broken")
	    reply  :=  out_msg$make_except_arg("FAILURE", p)
            out_msg$set_break(reply)
            s.broken := true
	    connection$send_reply(s.conn, reply, up(s))
		    except when not_possible(reason:string): return end
            continue
	    end
       seize s.request_queue do
          reqs:array[in_msg] := s.request_queue.value
          req:= array[in_msg]$reml(reqs)
                 except when bounds: continue end
	  if array[in_msg]$empty(reqs) then trigger_queue$reset(s.trigger) end
          end         

       p:maybe[port] := req.dest_port		%will be none if bad port set


       pt:port := maybe[port]$value(p)
	  except when wrong_tag: 
            put:puttype := bind out_msg$string_put(*, "Bad Port")
	    reply  :=  out_msg$make_except_arg("FAILURE", put)
            out_msg$set_break(reply)
            s.broken := true
	    connection$send_reply(s.conn, reply, up(s))
		    except when not_possible(reason:string): return end
            continue
	    end
       enter topaction
	 reply := pt.invoke(req, pt)
           except when failure(reason:string): 
                     put2:puttype := bind out_msg$string_put(*, reason)
	             reply  :=  out_msg$make_except_arg("FAILURE", put2)
                     out_msg$set_break(reply)
                     s.broken := true
                  when unavailable: 
		     reply := out_msg$make_except("UNAVAILABLE")
		     out_msg$set_break(reply)
		     s.broken := true
		  end
         if in_msg$is_send(req) then 
	    if out_msg$is_except(reply)
              then reply := out_msg$make_send_except()
              else continue 
	      end
	    end  
	 connection$send_reply(s.conn, reply, up(s))
		    except when not_possible(reason:string): return end
	 end %topaction
    end %while
    end manager

%reset is called from connection process when a restart message is received
%depends on fact that only connect process accesses current_message.

reset = proc(s:cvt)
      s.request_queue.value := array[in_msg]$new()
      s.restart_cnt := s.restart_cnt + 1
      s.current_message:= maybe[in_msg]$none()      
      s.synch_flag := false
      trigger_queue$trigger(s.trigger)
      end reset
      
close_stream = proc(s:cvt)
   s.close := true
   trigger_queue$trigger(s.trigger)
   end close_stream

break_stream = proc(s:cvt)
      s.broken := true
      end break_stream
      
get_agent_id = proc(s:cvt) returns(int)
      return(s.agent_id)
      end get_agent_id
      
get_seqid = proc(s:cvt) returns(seq_id)
      return(s.seqid)
      end get_seqid

enqueue = proc(s:cvt, m:in_msg)
      seize s.request_queue do
         am:array[in_msg] := s.request_queue.value
         array[in_msg]$addh(am, m)
	 trigger_queue$trigger(s.trigger)
	 end
      end enqueue

get_current_message = proc(s:cvt) returns (in_msg) signals (none)
      return(maybe[in_msg]$value(s.current_message))
          except when wrong_tag: signal none end
      end get_current_message
      
set_current_message = proc(s:cvt, m:in_msg)
      s.current_message := maybe[in_msg]$make(m)
      end set_current_message
      
clear_current_message = proc(s:cvt)
      s.current_message := maybe[in_msg]$none()
      end clear_current_message

cm_exists = proc(s:cvt) returns (bool)
      return( maybe[in_msg]$exists(s.current_message))
      end cm_exists
      
equal = proc(s1,s2:cvt) returns(bool)
      if s1.seqid = s2.seqid cand s1.agent_id = s2.agent_id
         then return(true)
	 else return(false)
	 end
      end equal
      
end stream

